/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2006 by Walther Neuper
 * created Jul 25, 2006 2:48:53 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.bridge;

import isac.interfaces.ICalcIterator;
import isac.interfaces.IToCalc;
import isac.interfaces.IToUser;
import isac.kestore.KEStoreServices;
import isac.util.CalcChangedEvent;
import isac.util.Formalization;
import isac.util.formulae.CalcHead;
import isac.util.formulae.ContextTheory;
import isac.util.formulae.KEStoreKey;
import isac.util.formulae.Position;
import isac.util.parser.FormalizationDigest;
import isac.wsdialog.IContextProvider.ContextType;

import java.util.Vector;

import junit.framework.TestCase;

public class TestContext extends TestCase {

    Formalization fmz_;

    MathEngine me_;

    /**
     * start the mini-subpbl
     * 
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
        KEStoreServices ks = new KEStoreServices("isac/xmldata");
        Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_1b.xml");

        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        fmz_ = fd.parseFormalization((String) xmlstr);

        MathEngine.init("localhost");
        me_ = MathEngine.getMathEngine();
    }

    public void testContext() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestContext#testContext");
        // start the calculation
        IToCalc ct = me_.getCalcTree(fmz_);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChangedEvent cce = null;
        CalcIterator ci = null;
        CalcHead ch = null;
        System.out.println(".end step_a-----");
        // end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        // autoCalculate
        System.out.println(".autoCalculate");
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
        // end step_b ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        KEStoreKey key = new KEStoreKey("thy_isac_Test-rls-Test_simplify");
        ContextTheory cont1 = new ContextTheory();
        ContextTheory cont2 = new ContextTheory();
        cont1.setKEStoreKey(key);

        Position pos = new Position();
        pos.addInt("1");
        pos.setKind("Res");
        ci = new CalcIterator((CalcTree) ct, pos);
        cont2 = (ContextTheory) ci.initContext(ContextType.CONTEXT_THEORIES);
        System.out.println(cont2.getAppliedTo());
//        assertEquals(cont2.getAppliedTo(), "x + 1 + -1 * 2 = 0");
        System.out.println(cont2.getResult());
//        assertEquals(cont2.getResult(), "-1 + x = 0");

        System.out.println("---END isac.bridge.TestContext#testContext");
    }

}
