/**
 * @author Robert Koenighofer
 * created on Jul 10, 2006
 */

package isac.util;

import junit.framework.TestCase;

/**
 * This is a Junit test class for KEStoreCommunication, which tests the
 * loadHierarchy and the loadContent functionality for all items of the
 * knowledge base (problems, methods, theories and examples)
 * 
 * @author Robert Koenighofer
 * @see isac.util.KEStoreCommunication#loadHierarchy(String, String)
 * @see isac.util.KEStoreCommunication#loadContent(String, String, String)
 * 
 */
public class TestKEStoreCommunication extends TestCase {

	/**
	 * This method loads the problem hierarchy by use of the
	 * KEStoreCommunication.loadHierarchy(). Some primitive checks are made to
	 * ensure that the returned String is no nonsense.
	 * 
	 * @see isac.util.KEStoreCommunication#loadHierarchy(String, String)
	 */
	public void testLoadHierarchyForProblems() {
		System.out
				.println("---BEGIN isac.util.TestKEStoreCommunication#testLoadHierarchyForProblems");
		String hierarchy_string = "";
		try {
			hierarchy_string = KEStoreCommunication.loadHierarchy("", "pbl");
		} catch (Exception ex) {
			assertTrue("exception caught", false);
		}
		assertNotNull("returned string is null", hierarchy_string);
		assertFalse("returned string is empty", hierarchy_string.equals(""));
		String expected_beginning = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
		assertTrue("returned string starts strange", hierarchy_string
				.startsWith(expected_beginning));
		System.out
				.println("---END isac.util.TestKEStoreCommunication#testLoadHierarchyForProblems");
	}

	/**
	 * This method loads an item of the problem hierarchy by use of the
	 * KEStoreCommunication.loadContent(). The problem hierarchy is loaded
	 * first, to get the name of an item to load. The names of hierarchy items
	 * are encapsulated by <CONTENTREF> and </CONTENTREF> tags. Some primitive
	 * checks are made to ensure that the returned String is no nonsense.
	 * 
	 * @see isac.util.KEStoreCommunication#loadContent(String, String, String)
	 */
	public void testLoadContentProblemItem() {
		System.out
		.println("---BEGIN isac.util.TestKEStoreCommunication#testLoadContentProblemItem");
		// getting an item out of the hierarchy:
		String hierarchy_string = KEStoreCommunication.loadHierarchy("", "pbl");
		int first_item_start = hierarchy_string.indexOf("<CONTENTREF>");
		first_item_start += new String("<CONTENTREF>").length();
		int first_item_end = hierarchy_string.indexOf("</CONTENTREF>");
		String hierarchy_item = hierarchy_string.substring(first_item_start,
				first_item_end);
		hierarchy_item = hierarchy_item.trim();

		// loading the item
		String xml_item_string = "";
		try {
			xml_item_string = KEStoreCommunication.loadContent("", "pbl",
					hierarchy_item + ".xml");
		} catch (Exception ex) {
			assertTrue("exception caught", false);
		}
		assertNotNull("returned string is null", xml_item_string);
		assertFalse("returned string is empty", xml_item_string.equals(""));
		String expected_beginning = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
		assertTrue("returned string starts strange", xml_item_string
				.startsWith(expected_beginning));
		System.out
		.println("---END isac.util.TestKEStoreCommunication#testLoadContentProblemItem");
	}

	/**
	 * This method loads the method hierarchy by use of the
	 * KEStoreCommunication.loadHierarchy(). Some primitive checks are made to
	 * ensure that the returned String is no nonsense.
	 * 
	 * @see isac.util.KEStoreCommunication#loadHierarchy(String, String)
	 */
	public void testLoadHierarchyForMethods() {
		System.out
		.println("---BEGIN isac.util.TestKEStoreCommunication#testLoadHierarchyForMethods");
		String hierarchy_string = "";
		try {
			hierarchy_string = KEStoreCommunication.loadHierarchy("", "met");
		} catch (Exception ex) {
			assertTrue("exception caught", false);
		}
		assertNotNull("returned string is null", hierarchy_string);
		assertFalse("returned string is empty", hierarchy_string.equals(""));
		String expected_beginning = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
		assertTrue("returned string starts strange", hierarchy_string
				.startsWith(expected_beginning));
		System.out
		.println("---END isac.util.TestKEStoreCommunication#testLoadHierarchyForMethods");
	}

	/**
	 * This method loads an item of the method hierarchy by use of the
	 * KEStoreCommunication.loadContent(). The method hierarchy is loaded first,
	 * to get the name of an item to load. The names of hierarchy items are
	 * encapsulated by <CONTENTREF> and </CONTENTREF> tags. Some primitive
	 * checks are made to ensure that the returned String is no nonsense.
	 * 
	 * @see isac.util.KEStoreCommunication#loadContent(String, String, String)
	 */
	public void testLoadContentMethodItem() {
		System.out
		.println("---BEGIN isac.util.TestKEStoreCommunication#testLoadContentMethodItem");
		// getting an item out of the hierarchy:
		String hierarchy_string = KEStoreCommunication.loadHierarchy("", "met");
		int first_item_start = hierarchy_string.indexOf("<CONTENTREF>");
		first_item_start += new String("<CONTENTREF>").length();
		int first_item_end = hierarchy_string.indexOf("</CONTENTREF>");
		String hierarchy_item = hierarchy_string.substring(first_item_start,
				first_item_end);
		hierarchy_item = hierarchy_item.trim();

		// loading the item
		String xml_item_string = "";
		try {
			xml_item_string = KEStoreCommunication.loadContent("", "met",
					hierarchy_item + ".xml");
		} catch (Exception ex) {
			assertTrue("exception caught", false);
		}
		assertNotNull("returned string is null", xml_item_string);
		assertFalse("returned string is empty", xml_item_string.equals(""));
		String expected_beginning = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
		assertTrue("returned string starts strange", xml_item_string
				.startsWith(expected_beginning));
		System.out
		.println("---END isac.util.TestKEStoreCommunication#testLoadContentMethodItem");
	}

	/**
	 * This method loads the theory hierarchy by use of the
	 * KEStoreCommunication.loadHierarchy(). Some primitive checks are made to
	 * ensure that the returned String is no nonsense.
	 * 
	 * @see isac.util.KEStoreCommunication#loadHierarchy(String, String)
	 */
	public void testLoadHierarchyForTheory() {
		System.out
		.println("---BEGIN isac.util.TestKEStoreCommunication#testLoadHierarchyForTheory");
		String hierarchy_string = "";
		try {
			hierarchy_string = KEStoreCommunication.loadHierarchy("", "thy");
		} catch (Exception ex) {
			assertTrue("exception caught", false);
		}
		assertNotNull("returned string is null", hierarchy_string);
		assertFalse("returned string is empty", hierarchy_string.equals(""));
		String expected_beginning = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
		assertTrue("returned string starts strange", hierarchy_string
				.startsWith(expected_beginning));
		System.out
		.println("---END isac.util.TestKEStoreCommunication#testLoadHierarchyForTheory");
	}

	/**
	 * This method loads an item of the theory hierarchy by use of the
	 * KEStoreCommunication.loadContent(). The theory hierarchy itself is loaded
	 * first, to get the name of an item to load. The names of hierarchy items
	 * are encapsulated by <CONTENTREF> and </CONTENTREF> tags. Some primitive
	 * checks are made to ensure that the returned String is no nonsense.
	 * 
	 * @see isac.util.KEStoreCommunication#loadContent(String, String, String)
	 */
	public void testLoadContentTheoryItem() {
		System.out
		.println("---BEGIN isac.util.TestKEStoreCommunication#testLoadContentTheoryItem");
		// getting an item out of the hierarchy:
		String hierarchy_string = KEStoreCommunication.loadHierarchy("", "thy");
		int first_item_start = hierarchy_string.indexOf("<CONTENTREF>");
		first_item_start += new String("<CONTENTREF>").length();
		int first_item_end = hierarchy_string.indexOf("</CONTENTREF>");
		String hierarchy_item = hierarchy_string.substring(first_item_start,
				first_item_end);
		hierarchy_item = hierarchy_item.trim();

		// loading the item
		String xml_item_string = "";
		try {
			xml_item_string = KEStoreCommunication.loadContent("", "thy",
					hierarchy_item + ".xml");
		} catch (Exception ex) {
			assertTrue("exception caught", false);
		}
		assertNotNull("returned string is null", xml_item_string);
		assertFalse("returned string is empty", xml_item_string.equals(""));
		System.out.println("xml_string: "+xml_item_string);
		String expected_beginning = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
		assertTrue("returned string starts strange", xml_item_string
				.startsWith(expected_beginning));
		System.out
		.println("---END isac.util.TestKEStoreCommunication#testLoadContentTheoryItem");
	}

	/**
	 * This method loads the example hierarchy by use of the
	 * KEStoreCommunication.loadHierarchy(). Some primitive checks are made to
	 * ensure that the returned String is no nonsense.
	 * 
	 * @see isac.util.KEStoreCommunication#loadHierarchy(String, String)
	 */
	public void testLoadHierarchyForExamples() {
		System.out
		.println("---BEGIN isac.util.TestKEStoreCommunication#testLoadHierarchyForExamples");
		String hierarchy_string = "";
		try {
			hierarchy_string = KEStoreCommunication.loadHierarchy("", "exp");
		} catch (Exception ex) {
			assertTrue("exception caught", false);
		}
		assertNotNull("returned string is null", hierarchy_string);
		assertFalse("returned string is empty", hierarchy_string.equals(""));
		String expected_beginning = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
		assertTrue("returned string starts strange", hierarchy_string
				.startsWith(expected_beginning));
		System.out
		.println("---END isac.util.TestKEStoreCommunication#testLoadHierarchyForExamples");
	}

	/**
	 * This method loads an item of the example hierarchy by use of the
	 * KEStoreCommunication.loadContent(). The example hierarchy itself is
	 * loaded first, to get the name of an item to load. The names of hierarchy
	 * items are encapsulated by <CONTENTREF> and </CONTENTREF> tags. Some
	 * primitive checks are made to ensure that the returned String is no
	 * nonsense.
	 * 
	 * @see isac.util.KEStoreCommunication#loadContent(String, String, String)
	 */
	public void testLoadContentExampleItem() {
		System.out
		.println("---BEGIN isac.util.TestKEStoreCommunication#testLoadContentExampleItem");
		// getting an item out of the hierarchy:
		String hierarchy_string = KEStoreCommunication.loadHierarchy("", "exp");
		int first_item_start = hierarchy_string.indexOf("<CONTENTREF>");
		first_item_start += new String("<CONTENTREF>").length();
		int first_item_end = hierarchy_string.indexOf("</CONTENTREF>");
		String hierarchy_item = hierarchy_string.substring(first_item_start,
				first_item_end);
		hierarchy_item = hierarchy_item.trim();

		// loading the item
		String xml_item_string = "";
		try {
			xml_item_string = KEStoreCommunication.loadContent("", "exp",
					hierarchy_item + ".xml");
		} catch (Exception ex) {
			assertTrue("exception caught", false);
		}
		assertNotNull("returned string is null", xml_item_string);
		assertFalse("returned string is empty", xml_item_string.equals(""));
		String expected_beginning = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
		assertTrue("returned string starts strange", xml_item_string
				.startsWith(expected_beginning));
		System.out
		.println("---END isac.util.TestKEStoreCommunication#testLoadContentExampleItem");
	}
}
