/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2004 by Walther Neuper 
 * created Sep 8, 2004, 2:41:34 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to license terms.
 */
package isac.util.formulae;

import org.apache.log4j.PropertyConfigurator;

import junit.framework.TestCase;

/**
 * @author Walther Neuper Sep 8, 2004, 2:41:34 PM @
 */
public class TestPosition extends TestCase {//([],*)..([],*)

    protected void setup() {
        //PropertyConfigurator.configure("../../isac/src/java/log4j.properties");
    }

    public void testCompareKind() {
        String k1 = "Frm";
        String k2 = "Frm";
        assertEquals("compareKind Frm Frm", 0, Position.compareKind(k1, k2));
        k1 = "Frm";
        k2 = "Pbl";
        assertEquals("compareKind Frm Pbl", 0, Position.compareKind(k1, k2));
        k1 = "Frm";
        k2 = "Res";
        assertEquals("compareKind Frm Pbl", -1, Position.compareKind(k1, k2));
    }

    public void testLessEquals() {
        Position p1 = new Position();
        Position p2 = new Position();

        p1.setKind("Frm");
        /*                      */p2.setKind("Pbl");
        assertEquals("compareTo ([],Frm)..([],Pbl)", 0, p1.compareTo(p2));
        assertFalse("less ([],Frm)..([],Pbl) is false", p1.less(p2));
        assertTrue("equals ([],Frm)..([],Pbl)", p1.equals(p2));

        /*                      */p2.setKind("Res");
        assertEquals("compareTo ([],Frm)..([],Res)", -1, p1.compareTo(p2));
        assertTrue("less ([],Frm)..([],Res)", p1.less(p2));
        assertFalse("equals ([],Frm)..([],Res)", p1.equals(p2));

        p1.addInt("1");
        p1.setKind("Frm");
        assertEquals("compareTo ([1],Frm)..([],Res)", -1, p1.compareTo(p2));
        assertTrue("less ([1],Frm)..([],Res)", p1.less(p2));
        assertFalse("equals ([1],Frm)..([],Res) is false", p1.equals(p2));

        /*                      */p2.setKind("Pbl");
        assertEquals("compareTo ([1],Frm)..([],Pbl)", 1, p1.compareTo(p2));
        assertFalse("less ([1],Frm)..([],Pbl)", p1.less(p2));
        assertFalse("equals ([1],Frm)..([],Pbl)", p1.equals(p2));

        p1.addInt("2");
        p1.addInt("3");
        /*                      */p2.addInt("1");
        /*                      */p2.addInt("2");
        /*                      */p2.addInt("4");
        assertEquals("compareTo ([1,2,3],Frm)..([1,2,4],Pbl)", -1, p1
                .compareTo(p2));
        assertTrue("less ([1,2,3],Frm)..([1,2,4],Pbl)", p1.less(p2));
        assertFalse("equals ([1,2,3],Frm)..([1,2,4],Pbl) is false", p1
                .equals(p2));

        /*                      */p2 = new Position();
        /*                      */p2.addInt("1");
        /*                      */p2.addInt("2");
        /*                      */p2.setKind("Res");
        assertEquals("compareTo ([1,2,3],Frm)..([1,2],Res)", -1, p1
                .compareTo(p2));
        assertTrue("less ([1,2,3],Frm)..([1,2],Res)", p1.less(p2));
        assertFalse("equals ([1,2,3],Frm)..([1,2],Res) is false", p1.equals(p2));

        /*                      */p2.setKind("Frm");
        assertEquals("compareTo ([1,2,3],Frm)..([1,2],Frm)", 1, p1
                .compareTo(p2));
        assertFalse("less ([1,2,3],Frm)..([1,2],Frm) is false", p1.less(p2));
        assertFalse("equals ([1,2,3],Frm)..([1,2],Frm) is false", p1.equals(p2));

        p1 = new Position();
        p1.addInt("1");
        p1.addInt("2");
        p1.setKind("Frm");
        /*                      */p2.addInt("3");
        assertEquals("compareTo ([1,2],Frm)..([1,2,3],Frm)", -1, p1
                .compareTo(p2));
        assertTrue("less ([1,2],Frm)..([1,2,3],Frm)", p1.less(p2));
        assertFalse("equals ([1,2],Frm)..([1,2,3],Frm) is false", p1.equals(p2));

        p1.setKind("Res");
        assertEquals("compareTo ([1,2],Res)..([1,2,3],Frm)", 1, p1
                .compareTo(p2));
        assertFalse("less ([1,2],Res)..([1,2,3],Frm)", p1.less(p2));
        assertFalse("equals ([1,2],Res)..([1,2,3],Frm) is false", p1.equals(p2));

    }

    public void testOnEndOfCalculation() {
        Position p1 = new Position();
        p1.setKind("Res");
        assertTrue("([], Res) is onEndOfCalculation", p1.onEndOfCalculation());

        p1.addInt("1");
        assertFalse("([1], Res) is NOT onEndOfCalculation", p1
                .onEndOfCalculation());

        p1 = new Position();
        p1.setKind("Pbl");
        assertFalse("([], Pbl) is NOT onEndOfCalculation", p1
                .onEndOfCalculation());
    }

}