/*
 * @author Robert Koenighofer, member of the ISAC-team, 
 * Copyright (c) 2006 by Robert Koenighofer
 * created Jul 18, 2006 10:09:13 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */

package isac.util.genhtml;

import isac.util.genhtml.Encoder;

import java.io.StringWriter;

import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import junit.framework.TestCase;



/**
 * This class tests the stylesheets in isac.util.genhtml.templates.*
 * Some xml-testfiles in isac.util.genhtml are encoded. The result
 * is checked with a regular expression.
 * 
 * @see isac.util.genhtml.templates.*
 * @see isac.util.genhtml.Encoder#encodeInformation(Source, Source, Writer)
 * @author Robert Koenighofer
 */
public class TestStyleSheets extends TestCase {

	public void testPblNodeContent() {

		Source xslSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/templates/pbl_node_content.xsl"));

		Source xmlSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/test_problem.xml"));

		StringWriter out = new StringWriter();

		Encoder.encodeInformation(xslSource, xmlSource, out);
		
		String result = out.toString();
		//System.out.println(out);
		
		//The regular expression 'regex' is used to check the html-result of the
		//encoding process. A character by character comparison with an expected
		//output is not done since small changes in the stylesheets (which lead
		//to additional whitespaces or something like that) would cause the tests
		//to fail. However if one of the main componets of the result is missing,
		//this regular expression will lead to a Failure.
		String any = "([\\S\\s]*)";
		String regex = any+"<html>"+any+"<body>"+any+"<div class=\"context\""+any+"test_headline"+any;
		regex += "<hr>"+any+"<table"+any+"Given"+any+"test_given1"+any+"test_given2"+any;
		regex += "Where"+any+"test_where"+any+"Find"+any+"test_find"+any+"Relate"+any;
		regex += "nbsp"+any+"</table>"+any+"</div>"+any+"<hr>"+any+"<EXPLANATIONS>"+any;
		regex += "</EXPLANATIONS>"+any+"c"+any+"test_autor"+any+"</body>"+any;
		regex += "</html>"+any;
		
		assertTrue("string does not match regular expression", result.matches(regex));
	}
	
	public void testPblNodeContentNoModel() {

		Source xslSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/templates/pbl_node_content_nomodel.xsl"));

		Source xmlSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/test_problem.xml"));

		StringWriter out = new StringWriter();

		Encoder.encodeInformation(xslSource, xmlSource, out);
		
		String result = out.toString();
		//System.out.println(out);
		String any = "([\\S\\s]*)";
		
		//The regular expression 'regex' is used to check the html-result of the
		//encoding process. A character by character comparison with an expected
		//output is not done since small changes in the stylesheets (which lead
		//to additional whitespaces or something like that) would cause the tests
		//to fail. However if one of the main componets of the result is missing,
		//this regular expression will lead to a Failure.
		String regex = any+"<html>"+any+"<body>"+any+"<EXPLANATIONS>"+any;
		regex += "</EXPLANATIONS>"+any+"c"+any+"test_autor"+any+"<hr>"+any+"</body>"+any;
		regex += "</html>"+any;
		
		assertTrue("string does not match regular expression", result.matches(regex));
	}
	
	public void testPblNodeContentAutor() {

		Source xslSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/templates/pbl_node_content_author.xsl"));

		Source xmlSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/test_problem.xml"));

		StringWriter out = new StringWriter();

		Encoder.encodeInformation(xslSource, xmlSource, out);
		
		String result = out.toString();
		//System.out.println(out);
		String any = "([\\S\\s]*)";
		
		//The regular expression 'regex' is used to check the html-result of the
		//encoding process. A character by character comparison with an expected
		//output is not done since small changes in the stylesheets (which lead
		//to additional whitespaces or something like that) would cause the tests
		//to fail. However if one of the main componets of the result is missing,
		//this regular expression will lead to a Failure.
		String regex = any+"<html>"+any+"<body>"+any+"<div class=\"context\""+any+"test_headline"+any;
		regex += "<hr>"+any+"<table"+any+"Given"+any+"test_given1"+any+"test_given2"+any;
		regex += "Where"+any+"test_where"+any+"Find"+any+"test_find"+any+"Relate"+any;
		regex += "nbsp"+any+"</table>"+any+"</div>"+any+"<hr>"+any+"<EXPLANATIONS>"+any;
		regex += "</EXPLANATIONS>"+any+"c"+any+"test_autor"+any+"</body>"+any;
		regex += "</html>"+any;
		
		assertTrue("string does not match regular expression", result.matches(regex));
	}
	
	public void testMetNodeContentAutor() {

		Source xslSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/templates/met_node_content_author.xsl"));

		Source xmlSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/test_method.xml"));

		StringWriter out = new StringWriter();

		Encoder.encodeInformation(xslSource, xmlSource, out);
		
		String result = out.toString();
		//System.out.println(out);
		
		//The regular expression 'regex' is used to check the html-result of the
		//encoding process. A character by character comparison with an expected
		//output is not done since small changes in the stylesheets (which lead
		//to additional whitespaces or something like that) would cause the tests
		//to fail. However if one of the main componets of the result is missing,
		//this regular expression will lead to a Failure.
		String any = "([\\S\\s]*)";
		String regex = any+"<html>"+any+"<body>"+any+"Simplifiers"+any;
		regex += "for evaluating the preconditions"+any+"test_pre"+any;
		regex += "for evaluating the conditions in rewrite rules"+any+"test_cond"+any;
		regex += "for evaluating the list expressions"+any;
		regex += "test_expr_"+any;
		regex += "for evaluating 'check_elementwise"+any+"test_check"+any;
		regex += "for formulae"+any+"test_normal"+any;
		regex += "Rewriteorder for rules in intermediate steps"+any+"test_reworder"+any;
		regex += "Script"+any+"test_script"+any;
		regex += "<div class=\"context\""+any+"<table"+any+"Given"+any+"test_given1"+any+"test_given2"+any;
		regex += "test_given3"+any;
		regex += "Where"+any+"nbsp"+any+"Find"+any+"test_find"+any+"Relate"+any;
		regex += "test_relate1"+any+"test_relate2"+any+"</table>"+any+"</div>"+any;
		regex += "<EXPLANATIONS>"+any+"test_explanation"+any+"</EXPLANATIONS>"+any;
		regex += "test_autor"+any+"</body>"+any+"</html>"+any;
		
		assertTrue("string does not match regular expression", result.matches(regex));
	}
	
	public void testMetNodeContent() {

		Source xslSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/templates/met_node_content.xsl"));

		Source xmlSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/test_method.xml"));

		StringWriter out = new StringWriter();

		Encoder.encodeInformation(xslSource, xmlSource, out);
		
		String result = out.toString();
		//System.out.println(out);
		
		//The regular expression 'regex' is used to check the html-result of the
		//encoding process. A character by character comparison with an expected
		//output is not done since small changes in the stylesheets (which lead
		//to additional whitespaces or something like that) would cause the tests
		//to fail. However if one of the main componets of the result is missing,
		//this regular expression will lead to a Failure.
		String any = "([\\S\\s]*)";
		String regex = any+"<html>"+any+"<body>"+any;
		regex += "Script"+any+"test_script"+any;
		regex += "<div class=\"context\""+any;
		regex += "<table"+any+"Given"+any+"test_given1"+any+"test_given2"+any;
		regex += "test_given3"+any;
		regex += "Where"+any+"nbsp"+any+"Find"+any+"test_find"+any+"Relate"+any;
		regex += "test_relate1"+any+"test_relate2"+any+"</table>"+any+"</div>"+any;
		regex += "<EXPLANATIONS>"+any+"test_explanation"+any+"</EXPLANATIONS>"+any;
		regex += "test_autor"+any+"</body>"+any+"</html>"+any;
		
		assertTrue("string does not match regular expression", result.matches(regex));
	}
	
	public void testMetNodeContentNoModel() {

		Source xslSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/templates/met_node_content_nomodel.xsl"));

		Source xmlSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/test_method.xml"));

		StringWriter out = new StringWriter();

		Encoder.encodeInformation(xslSource, xmlSource, out);
		
		String result = out.toString();
		//System.out.println(out);
		
		//The regular expression 'regex' is used to check the html-result of the
		//encoding process. A character by character comparison with an expected
		//output is not done since small changes in the stylesheets (which lead
		//to additional whitespaces or something like that) would cause the tests
		//to fail. However if one of the main componets of the result is missing,
		//this regular expression will lead to a Failure.
		String any = "([\\S\\s]*)";
		String regex = any+"<html>"+any+"<body>"+any;
		regex += "Script"+any+"test_script"+any;
		regex += "<EXPLANATIONS>"+any+"test_explanation"+any+"</EXPLANATIONS>"+any;
		regex += "test_autor"+any+"</body>"+any+"</html>"+any;
		
		assertTrue("string does not match regular expression", result.matches(regex));
	}
	
	public void testThyTheoremData() {

		Source xslSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/templates/thy_node_content.xsl"));

		Source xmlSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/test_theorem_data.xml"));

		StringWriter out = new StringWriter();

		Encoder.encodeInformation(xslSource, xmlSource, out);
		
		String result = out.toString();
		//System.out.println(out);
		
		//The regular expression 'regex' is used to check the html-result of the
		//encoding process. A character by character comparison with an expected
		//output is not done since small changes in the stylesheets (which lead
		//to additional whitespaces or something like that) would cause the tests
		//to fail. However if one of the main componets of the result is missing,
		//this regular expression will lead to a Failure.
		String any = "([\\S\\s]*)";
		String regex = any+"<html>"+any+"<body>"+any+"<table"+any;
		regex += "Theorem"+any+"test_theorem_id"+any+"test_theorem"+any+"</table>"+any;
		regex += "<div class=\"context\""+any+"</div>"+any+"test_explanaitions"+any;
		regex += "(c)"+any+"test_mathautor1"+any+"(math-autor)"+any;
		regex += "(c)"+any+"test_mathautor2"+any+"(math-autor)"+any;
		regex += "(c)"+any+"test_course_designer"+any+"(course-designer)"+any;
		regex += "</body>"+any+"</html>"+any;
		
		assertTrue("string does not match regular expression", result.matches(regex));
	}
	
	public void testThyHTMLData() {

		Source xslSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/templates/thy_node_content.xsl"));

		Source xmlSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/test_html_data.xml"));

		StringWriter out = new StringWriter();

		Encoder.encodeInformation(xslSource, xmlSource, out);
		
		String result = out.toString();
		//System.out.println(out);
		
		//The regular expression 'regex' is used to check the html-result of the
		//encoding process. A character by character comparison with an expected
		//output is not done since small changes in the stylesheets (which lead
		//to additional whitespaces or something like that) would cause the tests
		//to fail. However if one of the main componets of the result is missing,
		//this regular expression will lead to a Failure.
		String any = "([\\S\\s]*)";
		String regex = any+"<html>"+any+"<body>"+any+"test_explanaiton"+any;
		regex += "(c)"+any+"test_mathautor1"+any+"(math-autor)"+any;
		regex += "(c)"+any+"test_mathautor2"+any+"(math-autor)"+any;
		regex += "(c)"+any+"test_course_designer"+any+"(course-designer)"+any;
		regex += "</body>"+any+"</html>"+any;
		
		assertTrue("string does not match regular expression", result.matches(regex));
	}
	
	public void testThyRulesetData() {

		Source xslSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/templates/thy_node_content.xsl"));

		Source xmlSource = new StreamSource(getClass().getResourceAsStream(
				"/isac/util/genhtml/test_ruleset_data.xml"));

		StringWriter out = new StringWriter();

		Encoder.encodeInformation(xslSource, xmlSource, out);
		
		String result = out.toString();
		//System.out.println(out);
		
		//The regular expression 'regex' is used to check the html-result of the
		//encoding process. A character by character comparison with an expected
		//output is not done since small changes in the stylesheets (which lead
		//to additional whitespaces or something like that) would cause the tests
		//to fail. However if one of the main componets of the result is missing,
		//this regular expression will lead to a Failure.
		String any = "([\\S\\s]*)";
		String regex = any+"<html>"+any+"<body>"+any;
		regex += "<table"+any+"Identifier"+any+"test_ruleset_id"+any;
		regex += "</table>"+any+"<div class=\"context\">"+any+"</div>"+any;
		regex += "Rules"+any+ "<table"+any;
		regex += "test_tag1"+any+"href=\"../pbl/pbl_test_guh1.html"+any+"test_string1"+any;
		regex += "test_tag2"+any+"href=\"../exp/exp_test_guh2.html"+any+"test_string2"+any;
		regex += "test_tag3"+any+"href=\"../met/met_test_guh3.html"+any+"test_string3"+any;
		regex += "test_tag4"+any+"href=\"../thy/thy_test_guh4.html"+any+"test_string4"+any;
		regex += "</table>"+any;
		regex += "For applying rules"+any+"<table"+any+"Rewriteorder"+any+"dummy_ord"+any;
		regex += "</table>"+any;
		regex += "For evaluating conditions in rules"+any+"<table"+any+"Rewrite order"+any;
		regex += "<a href=\"../thy/thy_isac_DiffApp-rls-e_rls.html\"> e_rls </a>"+any;
		regex += "</table>"+any;
		regex += "test_explanations"+any;
		regex += "(c)"+any+"test_mathautor1"+any+"(math-autor)"+any;
		regex += "(c)"+any+"test_mathautor2"+any+"(math-autor)"+any;
		regex += "(c)"+any+"test_course_designer"+any+"(course-designer)"+any;
		regex += "</body>"+any+"</html>"+any;
		
		assertTrue("string does not match regular expression", result.matches(regex));
	}

}
