/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2006 by Walther Neuper
 * created Jul 7, 2006 10:03:42 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.util.parser;

import isac.util.ResponseWrapper;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.ContextMethod;
import isac.util.formulae.ContextProblem;
import isac.util.formulae.ContextTheory;
import isac.util.formulae.ModelItem;

import org.apache.log4j.Logger;

import junit.framework.TestCase;

/**
 * 
 */
public class TestMathEngineXML extends TestCase {

    private XMLParserDigest xml_parser_digest_;

    ResponseWrapper wrapper_ = null;

    static Logger logger_ = Logger.getLogger(TestMathEngineXML.class.getName());

    /**
     * test paring a specific theory context element
     */
    public void testParseContextRls() {
        System.out.println("---BEGIN isac.util.parser.TestMathEngineXML");
        logger_.debug("---testParseRefFormula begin@@@@@@@@@@@@@@@@@@@@@@@@@");
        xml_parser_digest_ = new XMLParserDigest("isac/src/xml/isac.dtd");
        xml_parser_digest_.setValidating(true);
        String xmlString = "<REFFORMULA>" // ........................
                + "  <CALCID> 1 </CALCID>" // .......................
                + "  <CALCFORMULA>" // ..............................
                + "    <POSITION>" // ...............................
                + "      <INTLIST>" // ..............................
                + "        <INT> 1 </INT>" // .......................
                + "      </INTLIST>" // .............................
                + "      <POS> Frm </POS>" // .......................
                + "    </POSITION>" // ..............................
                + "    <FORMULA>" // ................................
                + "      <MATHML>" // ...............................
                + "        <ISA> x + 1 = 2 </ISA>" // ...............
                + "      </MATHML>" // ..............................
                + "    </FORMULA>" // ...............................
                + "  </CALCFORMULA>" // .............................
                + "</REFFORMULA>";
        wrapper_ = xml_parser_digest_.parse(xmlString);
        int ci = wrapper_.getCalcID();
        CalcFormula form = (CalcFormula) wrapper_.getResponse();
        assertEquals(ci, 1);
        assertEquals(form.getPosition().toSMLString(), "([1],Frm)");
        assertEquals(form.toSMLString(), "x + 1 = 2");
        logger_.debug("---testParseRefFormula end@@@@@@@@@@@@@@@@@@@@@@@@@");
    }
    
    public void testParseTheoryContext() {
        logger_.debug("---testParseTheoryContext begin@@@@@@@@@@@@@@@@@@@@@@@@@");
        xml_parser_digest_ = new XMLParserDigest("isac/src/xml/isac.dtd");
        xml_parser_digest_.setValidating(true);
        String xmlString = "<CONTEXTTHY>" //.........................
                + "  <GUH> thy_Integration-thm-integral_mult </GUH>"
                + "  <SUBSLIST>" // .................................
                + "    <MATHML>" // .................................
                + "      <ISA> [(bdv,x), (bdv_2,y)] </ISA>" // ......
                + "    </MATHML>" // ................................
                + "  </SUBSLIST>" // ................................
                + "  <INSTANTIATED>" // .............................
                + "    <MATHML>" // .................................
                + "      <ISA> not (?b contains x) => Integral ?a * ?b D x = ?a * Integral ?b D x </ISA>"
                + "    </MATHML>" // ................................
                + "  </INSTANTIATED>" // ............................
                + "  <APPLTO>" // ...................................
                + "    <MATHML>" // .................................
                + "      <ISA> Integral 1 D x + Integral 2*x D x + Integral x^2 D x </ISA>"
                + "    </MATHML>" // ................................
                + "  </APPLTO>" // ..................................
                + "  <APPLAT>" // ...................................
                + "    <MATHML>" // .................................
                + "      <ISA> Integral 1 D x + Integral ?a * ?b D x + Integral x^2 D x </ISA>"
                + "    </MATHML>" // ................................
                + "  </APPLAT>" // ..................................
                + "  <ORDER>" // ....................................
                //+ "    <THEREF>" net yet implemented // .............
                //+ "      <TAG> Rewrite Order </TAG>" not yet implemented
                + "    <ID> dummy_order </ID>" // .....................
                //+ "      <GUH> thy_Atools-ord-dummy_ord </GUH>" not yet implemented
                //+ "    </THEREF>" // ................................
                + "  </ORDER>" // ...................................
                + "  <ASMSEVAL>" // .................................
                + "    <ASMEVALUATED>" // ...........................
                + "      <ASM>" // ..................................
      	        + "        <MATHML>" // .............................
                + "          <ISA> not (2 contains x) </ISA>" // ....
                + "        </MATHML>" // ............................
    	        + "      </ASM>" // .................................
    	        + "      <VALUE>"  // ...............................
      	        + "        <MATHML>" // .............................
                + "          <ISA> True </ISA>" // ..................
                + "        </MATHML>" // ............................
    	        + "      </VALUE>" // ...............................
                + "    </ASMEVALUATED>" // ..........................
                + "    <ASMEVALUATED>" // ...........................
                + "      <ASM>"  // .................................
    	        + "        <MATHML>" // .............................
                + "          <ISA> not (2 contains y) </ISA>" // ....
                + "        </MATHML>" // ............................
    	        + "      </ASM>" // .................................
      	        + "      <VALUE>"  // ...............................
      	        + "        <MATHML>" // .............................
                + "          <ISA> False </ISA>" // .................
                + "        </MATHML>" // ............................
    	        + "      </VALUE>" // ...............................
                + "    </ASMEVALUATED>" // ..........................
                + "  </ASMSEVAL>" // ................................
                + "  <LHS>" // ......................................
                + "    <MATHML>" // .................................
                + "      <ISA> Integral ?a * ?b D x </ISA>" // ......
                + "    </MATHML>" // ................................
                + "  </LHS>" // .....................................
                + "  <LHSINST>" // ..................................
                + "    <MATHML>" // .................................
                + "      <ISA> Integral 2 * x D x </ISA>" // ........
                + "    </MATHML>" // ................................
                + "  </LHSINST>" // .................................
                + "  <RHS>" // ......................................
                + "    <MATHML>" // .................................
                + "      <ISA> ?a * Integral ?b D x </ISA>" // ......
                + "    </MATHML>" // ................................
                + "  </RHS>" // .....................................
                + "  <RHSINST>" // ..................................
                + "    <MATHML>" // .................................
                + "      <ISA> 2 * Integral x D x </ISA>" // ........
                + "    </MATHML>" // ................................
                + "  </RHSINST>" // .................................
                + "  <RESULT>" // ...................................
                + "    <MATHML>" // .................................
                + "      <ISA> Integral 1 D x + 2 * Integral x D x + Integral x^2 D x </ISA>"
                + "    </MATHML>" // ................................
                + "  </RESULT>" // ..................................
                + "  <ASSUMPTIONS> something </ASSUMPTIONS>" // .....
                + "  <EVALRLS>" // ..................................
                + "    <KESTOREREF>"  // ................................
                + "      <TAG> Ruleset </TAG>" // ...................
                + "      <ID> eval_rls </ID>" // ....................
                + "      <GUH> thy_Atools-rls-eval_rls </GUH>" // ...
                + "    </KESTOREREF>"  // ...............................
                + "  </EVALRLS>" // .................................
                + "</CONTEXTTHY>";
        wrapper_ = xml_parser_digest_.parse(xmlString);
        ContextTheory thy_context = (ContextTheory) wrapper_.getResponse();
        assertEquals(thy_context.getKEStoreKey().toString(), "thy_Integration-thm-integral_mult");
        assertEquals(thy_context.getSubsList().toString(), "[(bdv,x), (bdv_2,y)]");
        assertEquals(thy_context.getInstanciatedTo().toString(), "not (?b contains x) => Integral ?a * ?b D x = ?a * Integral ?b D x");
        assertEquals(thy_context.getAppliedTo().toString(), "Integral 1 D x + Integral 2*x D x + Integral x^2 D x");
        assertEquals(thy_context.getAppliedAt().toString(), "Integral 1 D x + Integral ?a * ?b D x + Integral x^2 D x");
        assertEquals(thy_context.getOrder(), "dummy_order");
        assertEquals(thy_context.getEvalAssumtion(0).getAssumtion().toSMLString(), "not (2 contains x)");
        assertEquals(thy_context.getEvalAssumtion(0).getValue().toString(), "true");
        assertEquals(thy_context.getEvalAssumtion(1).getAssumtion().toSMLString(), "not (2 contains y)");
        assertEquals(thy_context.getEvalAssumtion(1).getValue().toString(), "false");
        assertEquals(thy_context.getLHS().toString(), "Integral ?a * ?b D x");
        assertEquals(thy_context.getLHSInstanciated().toString(), "Integral 2 * x D x");
        assertEquals(thy_context.getRHS().toString(), "?a * Integral ?b D x");
        assertEquals(thy_context.getRHSInstanciated().toString(), "2 * Integral x D x");
        assertEquals(thy_context.getResult().toString(), "Integral 1 D x + 2 * Integral x D x + Integral x^2 D x");
        assertEquals(thy_context.getAssumtions().toString(), "something");
        assertEquals(thy_context.getEvalRLS().getTag(), "Ruleset");
        assertEquals(thy_context.getEvalRLS().getID(), "eval_rls");
        assertEquals(thy_context.getEvalRLS().getKEStoreKey().toString(), "thy_Atools-rls-eval_rls");
        
        logger_.debug("---testParseTheoryContext end@@@@@@@@@@@@@@@@@@@@@@@@@");
    }
    
    public void testParseProblemContext() {
        logger_.debug("---testParseProblemContext begin@@@@@@@@@@@@@@@@@@@@@@@@@");
        xml_parser_digest_ = new XMLParserDigest("isac/src/xml/isac.dtd");
        xml_parser_digest_.setValidating(true);
        String xmlString = "<CONTEXTPBL>" // ........................
                + "  <GUH> pbl_equation_univar </GUH>" // ...........
                + "  <STATUS> correct </STATUS>" // .................
                + "  <HEAD>" // .....................................
                + "    <MATHML>" // .................................
                + "      <ISA> solve(x ^ 2 + 4 * x + 5 = 2, x) </ISA>"
                + "    </MATHML>" // ................................
                + "  </HEAD>" // ....................................
                + "  <MODEL>" // ....................................
                + "    <GIVEN>" // ..................................
                + "      <ITEM status=\"correct\">" // ..............
                + "        <MATHML>" // .............................
                + "          <ISA> equality (x ^ 2 + 4 * x + 5 = 2) </ISA>"
                + "        </MATHML>" // ............................
                + "      </ITEM>" // ................................
                + "      <ITEM status=\"correct\">" // ..............
                + "        <MATHML>" // .............................
                + "          <ISA> solveFor x </ISA>" // ............
                + "        </MATHML>" // ............................
                + "      </ITEM>" // ................................
                + "    </GIVEN>" // .................................
                + "    <WHERE>" // ..................................
                + "      <ITEM status=\"correct\">" // ..............
                + "        <MATHML>" // .............................
                + "          <ISA> matches (?a = ?b) (x ^ 2 + 4 * x + 5 = 2) </ISA>"
                + "        </MATHML>" // ............................
                + "      </ITEM>" // ................................
                + "    </WHERE>" // .................................
                + "    <FIND>" // ...................................
                + "      <ITEM status=\"correct\">" // ..............
                + "        <MATHML>" // .............................
                + "          <ISA> solutions L </ISA>" // ...........
                + "        </MATHML>" // ............................
                + "      </ITEM>" // ................................
                + "    </FIND>" // ..................................
                + "    <RELATE>  </RELATE>" // ......................
                + "  </MODEL>" // ...................................
                + "</CONTEXTPBL>";
        wrapper_ = xml_parser_digest_.parse(xmlString);
        ContextProblem pbl_context = (ContextProblem) wrapper_.getResponse();
        assertEquals(pbl_context.getKEStoreKey().toString(), "pbl_equation_univar");
        assertEquals(pbl_context.getStatus(), "correct");
        assertEquals(pbl_context.getHeadLine().toString(), "solve(x ^ 2 + 4 * x + 5 = 2, x)");
        assertEquals(((ModelItem) pbl_context.getModel().getGiven().getItems().get(0)).getItemStatus(), "correct");
        assertEquals(((ModelItem) pbl_context.getModel().getGiven().getItems().get(0)).getFormula().toString(), "equality (x ^ 2 + 4 * x + 5 = 2)");
        assertEquals(((ModelItem) pbl_context.getModel().getGiven().getItems().get(1)).getItemStatus(), "correct");
        assertEquals(((ModelItem) pbl_context.getModel().getGiven().getItems().get(1)).getFormula().toString(), "solveFor x");
        assertEquals(((ModelItem) pbl_context.getModel().getWhere().getItems().get(0)).getItemStatus().toString(), "correct");
        assertEquals(((ModelItem) pbl_context.getModel().getWhere().getItems().get(0)).getFormula().toString(), "matches (?a = ?b) (x ^ 2 + 4 * x + 5 = 2)");
        assertEquals(((ModelItem) pbl_context.getModel().getFind().getItems().get(0)).getItemStatus().toString(), "correct");
        assertEquals(((ModelItem) pbl_context.getModel().getFind().getItems().get(0)).getFormula().toString(), "solutions L");
        assertEquals(pbl_context.getModel().getRelate().getNumberOfItems(), 0);
        
        logger_.debug("---testParseProblemContext end@@@@@@@@@@@@@@@@@@@@@@@@@");
    }
    
    public void testParseMethodContext() {
        logger_.debug("---testParseMethodContext begin@@@@@@@@@@@@@@@@@@@@@@@@@");
        xml_parser_digest_ = new XMLParserDigest("isac/src/xml/isac.dtd");
        xml_parser_digest_.setValidating(true);
        String xmlString = "<CONTEXTMET>"
                + "  <GUH> pbl_equation_univar </GUH>" // ...........
                + "  <SCRIPT>" // ...................................
                + "    <MATHML>" // .................................
                + "      <ISA> Script BiegelinieScript l_ q_ v_ b_ rb_ rm_ = </ISA>"
                + "    </MATHML>" // ................................
                + "  </SCRIPT>" // ..................................
                + "  <STATUS> correct </STATUS>" // .................
                + "  <MODEL>" // ....................................
                + "    <GIVEN>" // ..................................
                + "      <ITEM status=\"correct\">" // ..............
                + "        <MATHML>" // .............................
                + "          <ISA> equality (x ^ 2 + 4 * x + 5 = 2) </ISA>"
                + "        </MATHML>" // ............................
                + "      </ITEM>" // ................................
                + "      <ITEM status=\"correct\">" // ..............
                + "        <MATHML>" // .............................
                + "          <ISA> solveFor x </ISA>" // ............
                + "        </MATHML>" // ............................
                + "      </ITEM>" // ................................
                + "    </GIVEN>" // .................................
                + "    <WHERE>" // ..................................
                + "      <ITEM status=\"correct\">" // ..............
                + "        <MATHML>" // .............................
                + "          <ISA> matches (?a = ?b) (x ^ 2 + 4 * x + 5 = 2) </ISA>"
                + "        </MATHML>" // ............................
                + "      </ITEM>" // ................................
                + "    </WHERE>" // .................................
                + "    <FIND>" // ...................................
                + "      <ITEM status=\"correct\">" // ..............
                + "        <MATHML>" // .............................
                + "          <ISA> solutions L </ISA>" // ...........
                + "        </MATHML>" // ............................
                + "      </ITEM>" // ................................
                + "    </FIND>" // ..................................
                + "    <RELATE>  </RELATE>" // ......................
                + "  </MODEL>" // ...................................
                + "</CONTEXTMET>";
        wrapper_ = xml_parser_digest_.parse(xmlString);
        ContextMethod met_context = (ContextMethod) wrapper_.getResponse();
        assertEquals(met_context.getKEStoreKey().toString(), "pbl_equation_univar");
        assertEquals(met_context.getStatus(), "correct");
        assertEquals(met_context.getScript().toString(), "Script BiegelinieScript l_ q_ v_ b_ rb_ rm_ =");
        assertEquals(((ModelItem) met_context.getModel().getGiven().getItems().get(0)).getItemStatus(), "correct");
        assertEquals(((ModelItem) met_context.getModel().getGiven().getItems().get(0)).getFormula().toString(), "equality (x ^ 2 + 4 * x + 5 = 2)");
        assertEquals(((ModelItem) met_context.getModel().getGiven().getItems().get(1)).getItemStatus(), "correct");
        assertEquals(((ModelItem) met_context.getModel().getGiven().getItems().get(1)).getFormula().toString(), "solveFor x");
        assertEquals(((ModelItem) met_context.getModel().getWhere().getItems().get(0)).getItemStatus().toString(), "correct");
        assertEquals(((ModelItem) met_context.getModel().getWhere().getItems().get(0)).getFormula().toString(), "matches (?a = ?b) (x ^ 2 + 4 * x + 5 = 2)");
        assertEquals(((ModelItem) met_context.getModel().getFind().getItems().get(0)).getItemStatus().toString(), "correct");
        assertEquals(((ModelItem) met_context.getModel().getFind().getItems().get(0)).getFormula().toString(), "solutions L");
        assertEquals(met_context.getModel().getRelate().getNumberOfItems(), 0);
        
        logger_.debug("---testParseMethodContext end@@@@@@@@@@@@@@@@@@@@@@@@@");
    }
    
    public void testParseMessage() {
        logger_.debug("---testParseMessage begin@@@@@@@@@@@@@@@@@@@@@@@@@");
        xml_parser_digest_ = new XMLParserDigest("isac/src/xml/isac.dtd");
        xml_parser_digest_.setValidating(true);
        String xmlString = "<MESSAGE>"
                + "  <CALCID> 2 </CALCID>"
                + "  <STRING> thy-context not to calchead </STRING>"
                + "</MESSAGE>";
        wrapper_ = xml_parser_digest_.parse(xmlString);
        assertEquals(wrapper_.getResponse(), null);
        
        logger_.debug("---testParseMessage end@@@@@@@@@@@@@@@@@@@@@@@@@");
    }
}
