/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2005 by Walther Neuper 
 * created Apr 15, 2005 8:26:54 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.util.users;

import isac.session.IObjectManager;
import isac.users.IUserManager;
import isac.users.IUserSettings;
import isac.users.User;
import isac.users.UserSettings;

import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;

import junit.framework.TestCase;

/**
 * @author neuper
 * 
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */
public class TestUser extends TestCase {

    protected void setUp() throws Exception {
        super.setUp();
    }

    /**
     * set up a minimal testbed for getting UserSettings of a Learner
     * 
     * @exception RemoteException
     *                if XSessionManager=ObjectManager NOT launched
     *                (ObjectManager launchedd after KEStore, BridgeMain)
     * @see isac.session.TestUserManager#testLogin
     */
    public void testGetUserSettings() {
        System.out
                .println("---BEGIN isac.util.user.TestUser#testGetUserSettings");

		String name = "//localhost/isac-ObjectManager";
		UserSettings user_settings = null;
		User user = null;
		
		try {
			Object lookup = Naming.lookup(name);
			IUserManager user_manager = ((IObjectManager) lookup)
					.getUserManager();
		
			user_manager.login("x", "x");

			user = user_manager.getUser("0");

			String usern = user.getUsername();
			assertEquals("username", usern, "x");

		} catch (RemoteException exc) {
			System.err.println("no rmiregistry runing on server " + name
					+ " no connection possible --- try later");
		} catch (NotBoundException exc) {
			System.err.println("servercom: notboundexc: " + exc.getMessage());
			System.err.println("server not found - try to connect to Proxy");
		} catch (java.net.MalformedURLException exc) {
			System.err.println("servercom: MalformedURLException: "
					+ exc.getMessage());
			exc.printStackTrace();
		}
		
        //this stores the whole UserSettings in the local 'user_settings'
        user_settings = user.getUserSettings();

        // in isac/xmldata/users/x_settings.txt we have:
        // key = IUserSettings.start_worksheet
        // val = IUserSettings.SKIP_SPECIFY_START_SOLVE
        int val = user_settings.getValue(IUserSettings.KEY1_START_SPECIFY);
        assertEquals("getValue(IUserSettings.KEY1_START_SPECIFY)", val,
                IUserSettings.VAL1_SKIP_SPECIFY_TO_START_SOLVE);

        val = user_settings.getValue(IUserSettings.KEY3_STATUS);
        assertEquals("getValue(IUserSettings.KEY3_STATUS)", val,
                IUserSettings.VAL3_LEARNER);

        System.out
                .println("---END isac.util.user.TestUser#testGetUserSettings");
    }

}