/*
 * Created on Aug 21, 2003
 */
package isac.bridge;

/**
 * Simple Logging utility class used by the bridge components
 * 
 * @version 1.0
 * @author Richard Gradischnegg RG
 */
import java.io.FileWriter;
import java.io.IOException;
import java.util.Date;

public class BridgeLogger {

    //The logging text will be stored in this file
    FileWriter general_log_;

    //minimum severity level of a message to be logged
    //0 == log everything
    int min_log_level_ = 0;

    /**
     * Create a new BridgeLogger which writes data to a file
     * 
     * @param path
     *            The path to the file which should be logged into
     */
    public BridgeLogger(String path) {
        try {
            general_log_ = new FileWriter(path + "../sml/BridgeLog/general.txt");
            log(1, "--------------------------");
            log(1, "Server started on " + new Date());
            log(1, "--------------------------");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Log general messages of bridge components in one file
     * 
     * @param level
     *            Severity level
     * @param msg
     *            The message to be logged
     */
    public void log(int level, String msg) {
        if (level > min_log_level_) {
            try {
                general_log_.write(msg + "\n");
                general_log_.flush();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Close the logging file
     */
    public void close() {
        try {
            general_log_.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public int getMinLogLevel() {
        return min_log_level_;
    }

    public void setMinLogLevel(int i) {
        min_log_level_ = i;
    }
}