/**
 * Created on Aug 13, 2003
 * @author Richard Gradischnegg RG
 */
package isac.bridge;

/**
 * @author rgradisc
 */
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.Vector;

/**
 * This class handles the output of the clients
 * 
 * @author Richard Gradischnegg RG
 *  
 */
class ClientOutputWorker implements Runnable {

    private Socket client_;

    private int id_;

    private ClientList client_list_;

    private BridgeMain bridge_;

    private BufferedReader client_in_;

    private PrintWriter client_out_;

    //private PrintWriter smlWriter;
    private Vector time_out_times_;

    //---------------------------------------------------------------
    public ClientOutputWorker(int id, Socket client, BridgeMain bridge) {
        this.id_ = id;
        this.client_ = client;
        this.bridge_ = bridge;
        this.time_out_times_ = bridge.getTimeOutTimes();
        this.client_list_ = bridge.getClientList();
        //this.smlWriter = bridge_.getSmlWriter();
        try {
            client_in_ = new BufferedReader(new InputStreamReader(client
                    .getInputStream()));
            client_out_ = new PrintWriter(client.getOutputStream(), true);
            bridge.log(1, ">>> new client_ in : " + client_in_.toString());
            bridge.log(1, ">>> new client_ out: " + client_out_.toString());
        } catch (IOException e) {
            System.out.println("in or out failed");
            System.exit(-1);
        }
        //register client_ in client_ list
        synchronized (client_list_) {
            client_list_.addClient(id, client_out_);
        }
    }

    //---------------------------------------------------------------
    public void run() {
        String line;
        while (true) {
            try {

                line = client_in_.readLine();
                if (line == null) {
                    client_list_.removeClient(id_);
                    break; // exit mainloop -> exit thread
                }
                sendData(line);
            } catch (IOException e) {
                bridge_.log(1, "Read failed");
                break;
                //System.exit(-1);
            }
        }
        bridge_.log(1, "[[from client_" + id_ + ": connection closed]]");
        bridge_.getClientList().removeClient(id_);
        try {
            client_.shutdownInput();
            client_.shutdownOutput();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    //---------------------------------------------------------------
    private synchronized void sendData(String line) {
        //so far: only one kernel
        bridge_.log(1, "[[from client_" + id_ + ": " + line);
        //bridge_.bufferInput(1, id_, line); // (kernelID, clientID, line)
        time_out_times_.add(new Long(System.currentTimeMillis())); // getTimeOutTimes(kernelID)
        bridge_.getSmlWriter().println(line); // getSmlWriter(kernelID)

    }
}