/**
 * @author Richard Gradischnegg RG
 * Created on Aug 13, 2003
 */

package isac.bridge;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;

import org.apache.log4j.Logger;

/**
 * This class runs in its own thread, fetches the output from the kernel, and
 * sends it back to the clients
 * 
 * @author richard
 */
public class Kernel2ClientsServer implements Runnable {

    //Start and end sequences to mark XML parts in SML output
    private static final String start_seq_ = "@@@@@begin@@@@@";

    private static final String end_seq_ = "@@@@@end@@@@@";

    private ClientList client_list_;

    private BridgeMain bridge_;

    private Vector time_out_times_;

    static Logger logger_ = Logger.getLogger(Kernel2ClientsServer.class
            .getName());

    //---------------------------------------------------------------
    public Kernel2ClientsServer(BridgeMain bridge) {
        logger_.warn("Kernel2ClientsServer: bridge_=" + bridge);
        //this.smlReader = bridge_.getSmlReader();
        this.client_list_ = bridge.getClientList();
        this.time_out_times_ = bridge.getTimeOutTimes();
        this.bridge_ = bridge;
    }

    //---------------------------------------------------------------
    public void run() {
        String line;
        boolean startedXML = false;
        int indexStart;
        int indexEnd;
        int currID = 0;
        StringBuffer sb = new StringBuffer();
        while (true) {
            try {
                line = bridge_.getSmlReader().readLine();
                if (bridge_.ignore_output_ == 0) {
                    bridge_.log(1, "[[from sml: " + line);
                }
                if (startedXML) {
                    indexEnd = line.indexOf(end_seq_);
                    if (indexEnd != -1) {
                        sb.append(line.substring(0, indexEnd));
                        if (time_out_times_.size() > 0) {
                            time_out_times_.removeElementAt(0);
                        }
                        // remove first (oldest) element from time_out_times_

                        if (bridge_.ignore_output_ == 0) {
                            // send XML-string (response of sml-kernel) to
                            // client
                            String s = new String(sb);
                            //TODO: FIXXME: removes the id in response:
                            s = s.substring(s.indexOf("<"));
                            //TODO: FIXXME: only one client
                            //currID = XMLParser.parseCalcID(s);
                            //TODO fetch id from bridge_
                            //so far: only one bridgeRMI
                            currID = 1;
                            bridge_.log(1, "found CalcID: " + currID);
                            PrintWriter pw = client_list_
                                    .getPrintWriterOfClient(currID);
                            if (pw != null) {
                                pw.println(s);
                                bridge_.log(1, ">> Response sent to user "
                                        + pw.toString());
                            }
                            //bridge_.responseSent();
                        } else {
                            bridge_.ignore_output_--;
                            bridge_.log(1,
                                    "ignored output (restore kernel state)");
                        }
                        startedXML = false;
                    } else {
                        sb.append(line);
                    }
                } else {
                    indexStart = line.indexOf(start_seq_);
                    if (indexStart != -1) {
                        sb = new StringBuffer();
                        startedXML = true;
                    }
                }
            } catch (IOException e) {
                bridge_.log(1, "Read from sml-kernel failed, waiting for kernel to restart" + e.getMessage() );
            } catch (NullPointerException e) {
                bridge_.log(1, "Read from sml failed, waiting for restart...");
            } catch (IndexOutOfBoundsException e) {
                bridge_.log(1, "Kernel2Clients: Index out of bound");
                e.printStackTrace();
            } catch (NumberFormatException e) {
                bridge_.log(1, "Couldn't parse user ID: NumberformatException");
            } catch (Exception e ) {
            	bridge_.log(1, "Unknown exception" + e.getMessage() );
            }
        }
    }
}