/**
 * @author Richard Gradischnegg RG
 * Created on Aug 13, 2003
 */
package isac.bridge;

/**
 * The sml-kernel is running in its own thread. This Class is used for setting
 * up and starting the thread. The communication with the bridge_ is done via
 * Input/Output streams
 * 
 * @author Richard Gradischnegg RG
 */
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.AccessControlException;

public class SMLThread implements Runnable {

    private Process process_;

    public boolean exited_;

    protected String execution_commandos[] = new String[2];

    private InputStream sml_input_;

    private OutputStream sml_output_;

    private BridgeMain bridge_;

    public SMLThread(BridgeMain bridge, String kernelExec, String kernelArgs) {
        execution_commandos[0] = kernelExec;
        execution_commandos[1] = kernelArgs;
        bridge.log(1, kernelExec);
        bridge.log(1, kernelArgs);

        this.bridge_ = bridge;
        initProcess();
        exited_ = false;
    }

    //---------------------------------------------------------------
    /**
     * returns the InputStream from the Kernel
     */
    public synchronized InputStream getInputStream() {
        try {
            sml_input_ = process_.getInputStream();
        } catch (java.lang.NullPointerException null_pointer_exception) {
            bridge_.log(1, "NullpointerException" + null_pointer_exception);
            try {
                wait(20);
            } catch (InterruptedException e) {
                System.out.println("getInputStream: " + e); // DEBUG
            }
            sml_input_ = getInputStream(); // recurse until stream ready
        }
        return sml_input_;
    }

    //---------------------------------------------------------------
    /**
     * returns the OutputStream from the Kernel
     *  
     */
    public synchronized OutputStream getOutputStream() {
        try {
            sml_output_ = process_.getOutputStream();
        } catch (java.lang.NullPointerException null_pointer_exception) {
            try {
                wait(20);
            } catch (InterruptedException e) {
                System.out.println("getOutputStream: " + e); // DEBUG
            }
            sml_output_ = getOutputStream(); // recurse until stream ready
        }
        return sml_output_;
    }

    /**
     * Initialize the process_ and execute the sml process_ with the execution
     * commandos
     */
    public synchronized void initProcess() {
        Runtime runtime = Runtime.getRuntime();
        // start sml
        try {
            try {
                process_ = runtime.exec(execution_commandos);
            } catch (AccessControlException e) {
                bridge_.log(1, "AccessControlExecption:\n" + e.toString());
                e.printStackTrace();
            }
        } catch (IOException e) {
            e.printStackTrace();
            exited_ = true;
            bridge_.log(1, "SMLThread: could not execute process_");
        }
    }

    //---------------------------------------------------------------
    /**
     * starts the ISAC - Kernel
     *  
     */
    public void run() {
        // wait until sml has terminated
        while (!exited_) {
            try {
                process_.waitFor();
            } catch (InterruptedException e) {
                bridge_.log(1, "SMLThread INTERRUPTED");
                break;
            }
            exited_ = true;
            process_.destroy(); // Destroy process_ and I/O streams
        }
        try {
            sml_input_.close();
            sml_output_.close();
        } catch (IOException e) {
            bridge_.log(1, "Cannot close SML Streams");
            e.printStackTrace();

        }
        bridge_.log(1, "SML Thread: Streams closed");
        //process_.destroy(); // Destroy process_ and I/O streams
        //bridge_.log(1, "SML Thread: Process returned:
        // "+process_.exitValue());
    }
}