package isac.gui.browser;

import java.awt.Dimension;

import isac.gui.IGuiConstants;
import isac.gui.browser.minibrowser.IMiniBrowser;
import isac.gui.browser.minibrowser.MiniBrowser;
import isac.gui.browser.panels.HierarchyPanel;

import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;

/**
 *
 * The BrowserPanel is a graphical container for the HierarchyPanel and MiniBrowser
 * It contains an adjustable split panel where these components are added.
 *
 */
public class BrowserPanel extends JPanel {

	static final long serialVersionUID = -5077134100354345800L;
	
	/**
	 * Split panel that splits the layout vertically in two parts:
	 * left for the hierarchy
	 * right for the content
	 */
	protected JSplitPane 	horizontal_split_pane_;
	
	/** 
	 * 
	 * The graphical representation of the tree structure 
	 * that comes from the KEStore
	 * 
	 */
  	protected HierarchyPanel 		hierarchy_panel_;
	
	/**
	 * 
	 * The HTML Viewer for the content
	 * 
	 */
	protected IMiniBrowser 	mini_browser_;
	
	/***
	 * 
	 * Link to the BrowserFrame that holds this Panel
	 * used for message dispatching.
	 * 
	 */
	protected BrowserFrame  parent_;
	
	/**
	 * The constructor
	 * 
	 * @param parent - The parent frame, which handles the communication with the BrowserDialog
	 * @param hierarchy - the hierarchy from KEStore
	 */
	BrowserPanel( BrowserFrame parent ) {
		super();
		
		/// save the parent
		parent_ = parent;
		
		/// set up the tree structure for the KEStore data
		hierarchy_panel_ = new HierarchyPanel( parent_ );
		
//		hierarchy_panel_.setMinimumSize(new Dimension(IGuiConstants.EXAMPLE_BROWSER_MIN_SIZE_X,
//        		IGuiConstants.EXAMPLE_BROWSER_MIN_SIZE_Y));
//		hierarchy_panel_.setPreferredSize(new Dimension(IGuiConstants.EXAMPLE_BROWSER_MIN_SIZE_X,
//        		IGuiConstants.EXAMPLE_BROWSER_MIN_SIZE_Y));
        
		/// Set up the MiniBrowser used for HTML representation of the data
		/// TODO: Change the MiniBrowser
		mini_browser_ = new MiniBrowser( parent_ );
		
		/// use a split pane to separate these two graphical components
		horizontal_split_pane_ = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, false);
		
		horizontal_split_pane_.setLeftComponent( hierarchy_panel_ );
        horizontal_split_pane_.setRightComponent( new JScrollPane( (MiniBrowser) mini_browser_ ) );
        
        horizontal_split_pane_.setOneTouchExpandable(true);

        setLayout(new BoxLayout(this, BoxLayout.X_AXIS));        
        add( horizontal_split_pane_ );
	}
	
	/**
	 * Get method for HierarchyPanel. Called by BrowserFrame when it 
	 * wants something from the Hierarchy.
	 * 
	 * @return the hierarchy panel
	 */
	public HierarchyPanel getHierarchyPanel() {
		System.out.println("In BrowserPanel::getHierarchyPanel()");
		return hierarchy_panel_;
	}

	/**
	 * Get method for MiniBrowser. Called by BrowserFrame when it 
	 * wants something from the MiniBrowser.
	 * 
	 * @return the minibrowser
	 */
	public IMiniBrowser getMiniBrowser() {
		System.out.println("In BrowserPanel::getMiniBrowser()");
		return mini_browser_;
	}	
}
