package isac.gui.browser.panels;

import isac.gui.browser.BrowserFrame;
import isac.gui.util.HierarchyNode;

import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.UserActionOnMathContext;

import isac.util.formulae.Context;
import isac.util.formulae.HierarchyKey;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JTree;
import javax.swing.tree.TreePath;

public class HierarchyPanelListener extends MouseAdapter {

	protected BrowserFrame  parent_;
	
	public HierarchyPanelListener( BrowserFrame parent ) {
		parent_ = parent;
	}
	
    /***
     * 
     * Tree events 
     * 
     */

	/**
	 * 
	 * Inherited from MouseListener
	 * The MouseEvents come from the HierarchyPanel
	 * 
	 *   If the user selected an item from the Tree in HierarchyPanel,
	 *   the BrowserFrame notifies the BrowserDialog of this
	 *   action with UI_BROWSER_CHANGE_CONTEXT. The BrowserDialog decides what to do next.
	 *   
	 *   If the user has clicked on the item twice then the
	 *   UI_BROWSER_EXECUTE_CONTEXT is sent instead.  
	 * 
	 */						
	public void mousePressed(MouseEvent mouse_event) {
		
		System.out.println( "In BrowserFrame::mousePressed() ");
				
		Object event_source = mouse_event.getSource();
			
		if( !(event_source instanceof JTree) )
			return;
		
		JTree    view_tree = (JTree) event_source;
		TreePath sel_path   = view_tree.getPathForLocation(mouse_event.getX(), mouse_event.getY());
        
        if( sel_path == null ) // nothing is chosen
        	return;
        
        if( mouse_event.getButton() == MouseEvent.BUTTON1 ) {
        	System.out.println( "Mouse pressed in hierarchy!!! Tree path : " + sel_path );
        	
        	HierarchyKey  h_key 		= generateHierarchyKeyForPath( sel_path );
        	HierarchyNode h_last_node 	= (HierarchyNode) sel_path.getLastPathComponent();

        	Context context_to_send  	= new Context();
        	context_to_send.setKEStoreKey(h_last_node.getKEStoreKey());
        	
        	context_to_send.setHierarchyKey( h_key );
   
        	EUIElement actionID = EUIElement.UI_BROWSER_CHANGE_CONTEXT;
        	    	
        	UserActionOnMathContext action_to_send = new UserActionOnMathContext( 
        			actionID,
        			EUIContext.UI_CONTEXT_HIERARCHY,
        			context_to_send 
        			);
        	parent_.notifyBrowserDialog( action_to_send );
        	
        }
	}
	
	private HierarchyKey generateHierarchyKeyForPath( TreePath sel_path ) {
		
		HierarchyKey the_key = new HierarchyKey();
		
		Object[] path = sel_path.getPath();
		
		for (int i = path.length - 1; i > 0; i--) {
			the_key.addString(((HierarchyNode) path[i]).getId());
		}
		
		return the_key;
	}	

}
