/*
 * Created on Jul 19, 2005
 * 
 *  
 */
package isac.gui.calcheadviews;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.CaretListener;

/**
 * @author Manuel Koschuch
 * 
 * class representing the single items in a calcheadpanel's Model
 */
public class ModelItemsPanel extends JPanel {

	/**
	 * For serialization, see corresponding API documentation
	 */
	private static final long serialVersionUID = 1L;

	private Vector fields_;

	private int current_position_;

	public ModelItemsPanel() {

		fields_ = new Vector();
		current_position_ = 0;
	}

	public void addToVector(JTextField text_field_to_add) {
		fields_.add(text_field_to_add);

	}

	public int getNumberOfFields() {
		return fields_.size();
	}

	public Vector getFields() {
		return fields_;
	}

	public Object getFieldAt(int position) {
		return fields_.get(position);
	}

	public Object getLastField() {
		return fields_.lastElement();
	}

	public Component add(Component c) {
		fields_.add(c);
		return super.add(c);
	}

	public void remove(Component c) {
		fields_.remove(c);

		CaretListener[] listeners = ((JTextField) c).getCaretListeners();
		for (int i = 0; i < listeners.length; i++) {
			((JTextField) c).removeCaretListener(listeners[i]);
		}
		super.remove(c);
	}

	public void clearTextFields() {
		current_position_ = 0;
		for (int i = 0; i < fields_.size(); i++) {
			JTextField current_field = (JTextField) fields_.get(i);
			current_field.setText("");
		}
		deleteAllEmptyFields();
	}

	/**
	 * Deletes all empty fields, except the last and the current
	 * 
	 * @param field
	 *            TextPane the event was triggered in
	 * @return true, when Field has been deleted
	 */
	public boolean deleteEmptyFields(JTextField field) {
		boolean deleted = false;
		JTextField last_field = (JTextField) fields_.lastElement();
		for (int i = 0; i < fields_.size(); i++) {
			JTextField current_field = (JTextField) fields_.get(i);
			if ((current_field != last_field) && (current_field != field)
					&& (current_field.getText().length() == 0)) {
				this.remove(current_field);
				deleted = true;
			}
		}
		return deleted;
	}

	/**
	 * Deletes all empty fields, including the last excluding the first
	 * 
	 * @param field
	 *            TextPane the event was triggered in
	 * @return true, when Field has been deleted
	 */
	public boolean deleteAllEmptyFields() {
		boolean deleted = false;
		for (int i = 0; i < fields_.size() - 1; i++) {
			JTextField current_field = (JTextField) fields_.get(i);
			if (current_field.getText().length() == 0) {
				this.remove(current_field);

				deleted = true;
			}
		}
		return deleted;
	}

	public int getCurrentPosition() {
		return current_position_;
	}

	public void increaseCurrentPosition() {
		current_position_++;
	}

	/**
	 * Sets ModelItemsPanel's editable status
	 * 
	 * @param editable
	 *            if true, all fields are editable and have registered Listeners
	 */
	public void setEditable(boolean editable) {
		for (int i = 0; i < fields_.size(); i++) {
			JTextField current_field = (JTextField) fields_.get(i);
			current_field.setEditable(editable);
			if (!editable) {
				removeListeners(current_field);
			}
		}
	}

	/**
	 * Removes all Action- and CaretListeners added to a Field
	 * 
	 * @param field
	 *            field the listeners should be removed from
	 */
	private void removeListeners(JTextField field) {
		CaretListener[] caret_listeners = field.getCaretListeners();
		ActionListener[] action_listeners = field.getActionListeners();
		for (int i = 0; i < caret_listeners.length; i++) {
			field.removeCaretListener(caret_listeners[i]);
		}
		for (int i = 0; i < action_listeners.length; i++) {
			field.removeActionListener(action_listeners[i]);
		}
	}

}