package isac.gui.calcheadviews;

import isac.gui.util.SpringUtilities;
import isac.interfaces.IGuiToGui;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.IUIAction;
import isac.useractions.IUIElement;
import isac.useractions.UserAction;
import isac.util.formulae.CalcHead;
import isac.util.formulae.Match;
import isac.util.formulae.Model;
import isac.util.formulae.ModelItem;
import isac.util.formulae.ModelItemList;

import java.awt.Color;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.util.Iterator;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SpringLayout;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

/**
 * Class that represents the GUI representation of a Calchead's Model
 * 
 * @author Manuel Koschuch
 * 
 * @version $Revision$
 */

public class ModelPanel extends JPanel implements CaretListener, ActionListener {

	/**
	 * Constant used for serialization (@see corresponding API Documentation)
	 */
	private static final long serialVersionUID = 1L;

	private static final int GIVEN_PANEL_INDEX = 0;

	private static final int WHERE_PANEL_INDEX = 1;

	private static final int FIND_PANEL_INDEX = 2;

	private static final int RELATE_PANEL_INDEX = 3;

	private JLabel label_given_;

	private JLabel label_where_;

	private JLabel label_find_;

	private JLabel label_relate_;

	private JTextField txt_given_;

	private JTextField txt_where_;

	private JTextField txt_find_;

	private JTextField txt_relate_;

	private ModelItemsPanel panel_given_ = new ModelItemsPanel();

	private JScrollPane pane_given_ = new JScrollPane(panel_given_);

	private ModelItemsPanel panel_where_ = new ModelItemsPanel();

	private JScrollPane pane_where_ = new JScrollPane(panel_where_);

	private ModelItemsPanel panel_find_ = new ModelItemsPanel();

	private JScrollPane pane_find_ = new JScrollPane(panel_find_);

	private ModelItemsPanel panel_relate_ = new ModelItemsPanel();

	private JScrollPane pane_relate_ = new JScrollPane(panel_relate_);

	private SpringLayout layout_ = new SpringLayout();

	private ResourceBundle res_bundle_;

	private int current_panel_index_;

	private IGuiToGui focus_lost_handler_;

	public ModelPanel(Model model, ResourceBundle resource_bundle) {
		res_bundle_ = resource_bundle;
		current_panel_index_ = 0;
		focus_lost_handler_ = null;
		init(true);
	}

	public void addFocusLostHandler(IGuiToGui handler) {
		focus_lost_handler_ = handler;
	}

	public ModelPanel(Match match, ResourceBundle resource_bundle) {
		res_bundle_ = resource_bundle;
		current_panel_index_ = 0;
		init(false);
	}

	private void init(boolean editable) {
		buildTextFields(editable);
		buildLabels();
		layoutComponents();
	}

	private void layoutComponents() {
		this.setLayout(layout_);

		setupPane(pane_given_, panel_given_, txt_given_, label_given_);
		setupPane(pane_where_, panel_where_, txt_where_, label_where_);
		setupPane(pane_find_, panel_find_, txt_find_, label_find_);
		setupPane(pane_relate_, panel_relate_, txt_relate_, label_relate_);

		formatContainer(panel_given_);
		formatContainer(panel_where_);
		formatContainer(panel_find_);
		formatContainer(panel_relate_);

		SpringUtilities.makeCompactGrid(this, 4, 2, // rows, cols
				5, 5, // initialX, initialY
				5, 5);// xPad, yPad

	}

	/**
	 * @param correct
	 *            If true then colorize the label green otherwise red.
	 * @param labelGiven
	 *            Label that should be colored
	 */
	private void colorizeLabel(boolean correct, JLabel label) {
		if (correct)
			label.setForeground(CalcHeadView.MODEL_ITEM_CORRECT_COLOR);
		else
			label.setForeground(CalcHeadView.MODEL_ITEM_FALSE_COLOR);
	}

	/**
	 * Fills a ModelList with text from the Textfields
	 * 
	 * @param list
	 *            ItemList to be filled
	 * @param items
	 *            Vector of Textfields, the list is filled from
	 * 
	 */

	private void insertTextIntoCalcheadModel(ModelItemList list, Vector items) {
		for (int i = 0; i < items.size(); i++) {
			ModelItem currentItem = new ModelItem();
			JTextField current_field = (JTextField) items.get(i);
			if (!(current_field.getText().equals(""))) {
				currentItem.setText(((JTextField) items.get(i)).getText());
				list.addItem(currentItem);
			}
		}
	}

	/**
	 * Used to format the single Panes (given, where, find and relate)
	 * 
	 * @param pane
	 *            the Scrollpane containing the Panels
	 * @param panel
	 *            the Panel containing the TextFields
	 * @param text_field
	 *            the first Textfield in the panel
	 * @param label
	 *            the label for the panel
	 * 
	 */

	private void setupPane(JScrollPane pane, JPanel panel,
			JTextField text_field, JLabel label) {
		pane
				.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		pane.getVerticalScrollBar().setUnitIncrement(3);
		pane.setBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY, 1));
		panel.setLayout(new SpringLayout());
		panel.add(text_field);
		this.add(label);
		this.add(pane);
		pane.setViewportView(panel);
	}

	/**
	 * Used to format the single Panes, when there is only one Field added
	 * 
	 */

	private void formatContainer(Container c) {
		SpringUtilities.makeCompactGrid(c, 1, 1, // rows, cols
				5, 5, // initialX, initialY
				5, 5);// xPad, yPad
	}

	/**
	 * Used to react to user inputs into one of the TextFields
	 * 
	 */

	public void caretUpdate(CaretEvent e) {

		ModelItemsPanel panel = (ModelItemsPanel) ((JTextField) e.getSource())
				.getParent();

		// last field isn't empty->create new field
		if (((JTextField) panel.getLastField()).getText().length() != 0) {
			addFieldToPanel(panel);

			// reformat Panel
			SpringUtilities.makeCompactGrid(panel, panel.getNumberOfFields(),
					1, // rows, cols
					5, 5, // initialX, initialY
					5, 5);// xPad, yPad

			// request Focus again for current TextField
			((JTextField) e.getSource()).requestFocusInWindow();
		}

		// empty field exists and is neither current nor last field
		if (panel.deleteEmptyFields((JTextField) e.getSource())) {
			SpringUtilities.makeCompactGrid(panel, panel.getNumberOfFields(),
					1, // rows, cols
					5, 5, // initialX, initialY
					5, 5);// xPad, yPad

			panel.repaint();

			// FIXME MK find solution to get JScrollPane independent from GUI
			// components
			((JScrollPane) panel.getParent().getParent())
					.setViewportView(panel);
		}
	}

	/**
	 * adds a field to the given Panel
	 * 
	 * @param panel
	 *            the panel to add the field to
	 */

	private void addFieldToPanel(ModelItemsPanel panel) {

		JTextField new_field = new JTextField();
		new_field.setBorder(BorderFactory.createEtchedBorder());
		new_field.setEditable(false);

		if (panel != panel_where_) {
			new_field.setEditable(true);
			new_field.addCaretListener(this);
			new_field.addActionListener(this);
		}

		panel.add(new_field);

		// FIXME MK find solution to get JScrollPane independent from GUI
		// components
		((JScrollPane) panel.getParent().getParent()).setViewportView(panel);

		SpringUtilities.makeCompactGrid(panel,
				1 + panel.getNumberOfFields() - 1, 1, // rows, cols
				5, 5, // initialX, initialY
				5, 5);// xPad, yPad
	}

	/**
	 * Colorizes the Items in the individual fields of a given panel
	 * 
	 * @param it
	 *            Iterator through Model to check
	 * @param panel
	 *            panel, that holds the text fields
	 * @param editable
	 *            if false, no TextField ist editable
	 * @return true, when all items are either correct or superflous
	 */

	private boolean checkAndFormatModelItems(Iterator it,
			ModelItemsPanel panel, boolean editable) {

		boolean correct = true;
		while (it.hasNext()) {
			JTextField txt = (JTextField) panel.getLastField();
			Object o = it.next();
			if (o instanceof ModelItem) {
				ModelItem chi = (ModelItem) o;
				if(!colorizeSingleModelItem(chi, txt)){
					correct = false;
				}
				txt.removeCaretListener(this);
				txt.setText(chi.toSMLString());
				if (editable && panel != panel_where_) {
					txt.addCaretListener(this);
				}
				if (panel != panel_where_) {
					txt.setEditable(editable);
				}
			} else if (o instanceof String) {
				String s = (String) o;
				txt.setText(txt.getText() + s);
				txt.setEditable(editable);
			}
			if (it.hasNext())
				this.addFieldToPanel(panel);
		}
		return correct;
	}

	/**
	 * Colorizes a single ModelItem
	 * 
	 * @param chi
	 *            the ModelItem
	 * @param txt
	 *            the Textfield that is filled with the content of the ModelItem
	 * @return true, when ModelItem status is correct, false otherwise
	 */
	private boolean colorizeSingleModelItem(ModelItem chi, JTextField txt) {
		boolean correct = true;
		switch (chi.getStatus()) {
		case CalcHead.MODEL_ITEM_CORRECT: {
			txt.setForeground(CalcHeadView.MODEL_ITEM_CORRECT_COLOR);
			correct = true;
			break;
		}
		case CalcHead.MODEL_ITEM_FALSE: {
			txt.setForeground(CalcHeadView.MODEL_ITEM_FALSE_COLOR);
			correct = false;
			break;
		}
		case CalcHead.MODEL_ITEM_SYNTAXERROR: {
			txt.setForeground(CalcHeadView.MODEL_ITEM_SYNTAXERROR_COLOR);
			correct = false;
			break;
		}
		case CalcHead.MODEL_ITEM_INCORRECT: {
			txt.setForeground(CalcHeadView.MODEL_ITEM_INCORRECT_COLOR);
			correct = false;
			break;
		}
		case CalcHead.MODEL_ITEM_HELP_ME: {
			txt.setForeground(CalcHeadView.MODEL_ITEM_HELP_ME_COLOR);
			correct = true;
			break;
		}
		case CalcHead.MODEL_ITEM_INCOMPLETE: {
			txt.setForeground(CalcHeadView.MODEL_ITEM_INCOMPLETE_COLOR);
			correct = false;
			break;
		}
		case CalcHead.MODEL_ITEM_SUPERFLUOUS: {
			txt.setForeground(CalcHeadView.MODEL_ITEM_SUPERFLUOUS_COLOR);
			correct = true;
			break;
		}
		case CalcHead.MODEL_ITEM_MISSING: {
			txt.setForeground(CalcHeadView.MODEL_ITEM_MISSING_COLOR);
			correct = false;
			break;
		}
		}
		return correct;
	}

	/**
	 * Used in Initialization, to build the first TextField in every Panel
	 * 
	 * @param editable
	 *            indicates, if fields should be editable
	 */

	private void buildTextFields(boolean editable) {
		txt_given_ = new JTextField(CalcHeadView.FIELD_SIZE);
		txt_given_.setBorder(BorderFactory.createEtchedBorder());
		txt_given_.setEditable(editable);
		txt_where_ = new JTextField(CalcHeadView.FIELD_SIZE);
		txt_where_.setBorder(BorderFactory.createEtchedBorder());
		txt_where_.setEditable(false);
		txt_find_ = new JTextField(CalcHeadView.FIELD_SIZE);
		txt_find_.setBorder(BorderFactory.createEtchedBorder());
		txt_find_.setEditable(editable);
		txt_relate_ = new JTextField(CalcHeadView.FIELD_SIZE);
		txt_relate_.setBorder(BorderFactory.createEtchedBorder());
		txt_relate_.setEditable(editable);
		if (editable) {
			txt_given_.addCaretListener(this);
			txt_find_.addCaretListener(this);
			txt_relate_.addCaretListener(this);
			txt_given_.addActionListener(this);
			txt_find_.addActionListener(this);
			txt_relate_.addActionListener(this);

		}
	}

	/**
	 * Used in Initializtion to build the Labels for every Panel
	 */

	private void buildLabels() {
		label_given_ = new JLabel(res_bundle_.getString("given"));
		label_where_ = new JLabel(res_bundle_.getString("where"));
		label_find_ = new JLabel(res_bundle_.getString("find"));
		label_relate_ = new JLabel(res_bundle_.getString("relate"));
	}

	/**
	 * Fill all Textfields from the supplied Model
	 * 
	 * @param model
	 *            model to fill TextField from
	 * @param editable
	 *            false, when TextFields shouldn't be editable (used when
	 *            creating ModelPanel from a Match for display in Browser)
	 */
	public void fillTextFieldsFromModel(Model model, boolean editable) {

		panel_given_.clearTextFields();
		panel_where_.clearTextFields();
		panel_find_.clearTextFields();
		panel_relate_.clearTextFields();

		boolean correct = true;

		correct = this.checkAndFormatModelItems(model.getGiven().getItems()
				.iterator(), panel_given_, editable);
		this.colorizeLabel(correct, label_given_);
		correct = this.checkAndFormatModelItems(model.getFind().getItems()
				.iterator(), panel_find_, editable);
		this.colorizeLabel(correct, label_find_);
		correct = this.checkAndFormatModelItems(model.getWhere().getItems()
				.iterator(), panel_where_, editable);
		this.colorizeLabel(correct, label_where_);
		correct = this.checkAndFormatModelItems(model.getRelate().getItems()
				.iterator(), panel_relate_, editable);
		this.colorizeLabel(correct, label_relate_);

	}

	/**
	 * Fills a given Model with the contents of the TextFields
	 * 
	 * @param model
	 *            Model that is filled from the TextFields
	 */
	public void fillModelFromTextFields(Model model) {

		insertTextIntoCalcheadModel(model.getGiven(), panel_given_.getFields());
		insertTextIntoCalcheadModel(model.getWhere(), panel_where_.getFields());
		insertTextIntoCalcheadModel(model.getFind(), panel_find_.getFields());
		insertTextIntoCalcheadModel(model.getRelate(), panel_relate_
				.getFields());

	}

	/**
	 * Clears all textFields
	 */

	public void reset() {

		current_panel_index_ = 0;
		panel_given_.clearTextFields();
		panel_where_.clearTextFields();
		panel_find_.clearTextFields();
		panel_relate_.clearTextFields();

		formatContainer(panel_given_);
		formatContainer(panel_where_);
		formatContainer(panel_find_);
		formatContainer(panel_relate_);

	}

	/**
	 * Returns number of all Fields in the ModelPanel
	 * 
	 * @return number of Fields
	 */

	public int getTotalNumberOfFields() {
		int number = 0;
		number += panel_given_.getNumberOfFields();
		number += panel_where_.getNumberOfFields();
		number += panel_find_.getNumberOfFields();
		number += panel_relate_.getNumberOfFields();
		return number;
	}

	/**
	 * Fills Textfields in a model stepwise
	 * 
	 * @param model
	 *            Model to fill TextFields from
	 * @return false, when Textfields left to fill, true otherwise
	 * @deprecated was misunderstanding
	 */

	public boolean next(Model model) {
		switch (current_panel_index_) {
		case GIVEN_PANEL_INDEX:
			if (next(model.getGiven(), panel_given_, label_given_)) {
				current_panel_index_++;
				return false;
			}
			return false;

		case WHERE_PANEL_INDEX:
			if (next(model.getWhere(), panel_where_, label_where_)) {
				current_panel_index_++;
				return false;
			}
			return false;

		case FIND_PANEL_INDEX:
			if (next(model.getFind(), panel_find_, label_find_)) {
				current_panel_index_++;
				return false;
			}
			return false;

		case RELATE_PANEL_INDEX:
			if (next(model.getRelate(), panel_relate_, label_relate_)) {
				current_panel_index_++;
				return true;
			}
			return false;

		default:
			return true;

		}
	}

	/**
	 * Fills single ModelItemsPanel stepwise.
	 * 
	 * @param item_list
	 *            list Containing ModelItems
	 * @param panel
	 *            panel, to be filled
	 * @return true, when Panel completely filled, false otherwise
	 * 
	 * @deprecated was misunderstanding
	 * 
	 */

	private boolean next(ModelItemList item_list, ModelItemsPanel panel,
			JLabel label) {
		Vector items = item_list.getItems();
		int number_of_items = items.size();
		int current_position = panel.getCurrentPosition();
		boolean correct = true;
		JTextField current_field = (JTextField) panel.getLastField();

		if (current_position < number_of_items) {
			correct = colorizeSingleModelItem((ModelItem) items
					.get(current_position), current_field);
			current_field.removeCaretListener(this);
			current_field.setText(((ModelItem) items.get(current_position))
					.toSMLString());
			if (panel != panel_where_) {
				current_field.addCaretListener(this);
			}
			panel.increaseCurrentPosition();
			colorizeLabel(correct, label);
			// not last field?
			if (current_position != number_of_items - 1) {
				addFieldToPanel(panel);
				return false;
			}
		}
		return true;
	}

	/**
	 * Fills TextFields from a Match (used when displaying ModePanel in Browser
	 * 
	 * @param match
	 *            Match to fill ModelPanel from
	 */
	public void fillTextFieldsFromMatch(Match match) {
		fillTextFieldsFromModel(match.getModel(), false);
	}

	public void actionPerformed(ActionEvent e) {
		if (focus_lost_handler_ != null) {
			focus_lost_handler_.notifyUserAction(new UserAction(
					EUIElement.UI_SPECIFY_EDIT_ACTIVE_FORMULA_COMPLETE,
					EUIContext.UI_CONTEXT_CALCHEAD));
		}
	}

	/**
	 * Sets ModelPanel's editable status
	 * 
	 * @param editable
	 *            if false, all fields are read only and there are no listeners
	 *            attached to them any more
	 * 
	 */
	public void setEditable(boolean editable) {
		panel_given_.setEditable(editable);
		panel_where_.setEditable(editable);
		panel_find_.setEditable(editable);
		panel_relate_.setEditable(editable);

	}

}
