package isac.gui.calcheadviews;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.ResourceBundle;

import isac.gui.util.SpringUtilities;
import isac.interfaces.IGuiToGui;
import isac.useractions.EUIContext;
import isac.useractions.EUIElement;
import isac.useractions.IUIAction;
import isac.useractions.IUIElement;
import isac.useractions.UserAction;
import isac.util.formulae.CalcHead;
import isac.util.formulae.CalcHeadSimpleID;
import isac.util.formulae.HierarchyKey;
import isac.util.formulae.ModelItem;
import isac.util.formulae.Specification;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.SpringLayout;

/**
 * Class for the graphical Representation of the Specifiaction in a CalcHead
 * 
 * @author Manuel Koschuch
 * @version $Revision$
 */

public class SpecificationPanel extends JPanel implements ActionListener {

	/**
	 * Constant used for serialization (@see corresponding API Documentation)
	 */
	private static final long serialVersionUID = 1L;

	private JLabel label_theory_;

	private JLabel label_method_;

	private JLabel label_problem_;

	private JLabel txt_theory_;

	private JLabel txt_method_;

	private JLabel txt_problem_;

	private ResourceBundle res_bundle_;

	static final int LABEL_METHOD = 1;

	static final int LABEL_PROBLEM = 2;

	static final int LABEL_THEORY = 3;

	private static final int POSITION_THEORY = 0;

	private static final int POSITION_PROBLEM = 1;

	private static final int POSITION_METHOD = 2;

	private int current_position_;

	private JRadioButton problem_button_;

	private JRadioButton method_button_;

	private IGuiToGui radio_button_handler_;

	public SpecificationPanel(Specification specification,
			ResourceBundle resource_bundle) {
		res_bundle_ = resource_bundle;
		current_position_ = 0;
		init();
	}

	private void init() {
		buildButtons();
		buildTextLabels();
		buildLabels();
		layoutComponents();
	}

	private void buildButtons() {
		problem_button_ = new JRadioButton();
		problem_button_.addActionListener(this);
		method_button_ = new JRadioButton();
		method_button_.addActionListener(this);
		ButtonGroup group = new ButtonGroup();
		group.add(problem_button_);
		group.add(method_button_);
	}

	/**
	 * Builds the TextFields and labels them
	 * 
	 */

	private void layoutComponents() {
		this.setLayout(new SpringLayout());
		this.add(new JLabel(""));
		this.add(label_theory_);
		this.add(txt_theory_);
		this.add(problem_button_);
		this.add(label_problem_);
		this.add(txt_problem_);
		this.add(method_button_);
		this.add(label_method_);
		this.add(txt_method_);

		SpringUtilities.makeCompactGrid(this, 3, 3, // rows, cols
				5, 5, // initialX, initialY
				5, 5);// xPad, yPad

	}

	private void buildTextLabels() {
		txt_theory_ = new JLabel();
		txt_method_ = new JLabel();
		txt_problem_ = new JLabel();
	}

	/**
	 * Clears all TextFields
	 */

	private void clearTexts() {
		if (txt_theory_ != null)
			txt_theory_.setText("");
		if (txt_problem_ != null)
			txt_problem_.setText("");
		if (txt_method_ != null)
			txt_method_.setText("");
	}

	/**
	 * parse the Text Field txtProblem which can have multiple mathTexts
	 * seperated by a seperator char(a comma at te moment) so that as a result
	 * those multiple mathTexts are in a String[] where every element is one
	 * mathText
	 * 
	 * @param seperator:
	 *            the char that seperates multiple mathTexts
	 * @param text
	 *            the TextField to parse
	 * @return String[] which contains all mathTexts of the text Field
	 * @author Alois Kirchsteiger 10.09.2004 ~11
	 * @author Manuel Koschuch
	 */
	private String[] parseTxtToStringArray(char seperator, JLabel text) {
		String text_to_split = text.getText();
		String seperatorString = String.valueOf(seperator);
		return text_to_split.split(seperatorString);
	}

	/**
	 * parses Text from a Lable to a HierarchyKey
	 * 
	 * @param text_to_parse
	 *            the text to parse
	 * 
	 * @return the HierarchyKey
	 * 
	 */
	private HierarchyKey parseTxtToKEStoreID(JLabel text_to_parse) {
		String[] text = parseTxtToStringArray(',', text_to_parse);
		HierarchyKey calc_compound_ID = new HierarchyKey();
		for (int i = 0; i < text.length; i++) {
			calc_compound_ID.addString(text[i]);
		}
		return calc_compound_ID;
	}

	/**
	 * insert the mathTexts from the TextField txtTheory in the CalcHead
	 * 
	 * @param calcHead
	 */
	private void insertTxtTheoryInCalcHeadSpecification(
			Specification specification) {
		String theory = txt_theory_.getText();
		CalcHeadSimpleID theoryCalcHeadSimpleID = new CalcHeadSimpleID();
		theoryCalcHeadSimpleID.setID(theory);
		specification.setTheory(theoryCalcHeadSimpleID);
	}

	/**
	 * inserts the whole specification (theory, method, problem) in the calcHead
	 * 
	 * @param calcHead
	 * @author Alois Kirchsteiger 10.09.2004 13:56:25
	 */

	public void fillSpecificationFromTextFields(Specification spec) {
		insertTxtTheoryInCalcHeadSpecification(spec);
		spec.setMethod(parseTxtToKEStoreID(txt_method_));
		spec.setProblem(parseTxtToKEStoreID(txt_problem_));

	}

	/**
	 * Fills the TextFields from the given Specification
	 * 
	 * @param spec
	 *            Specification to fill the TextFields from
	 * 
	 */

	public void fillTextFieldsFromSpecification(Specification spec,
			String belongs_to) {
		clearTexts();
		if (spec.getMethod() != null)
			this.checkAndFormatSpecificationItems(spec.getMethod().getID()
					.iterator(), txt_method_);
		if (spec.getProblem() != null)
			this.checkAndFormatSpecificationItems(spec.getProblem().getID()
					.iterator(), txt_problem_);
		if (spec.getProblem() != null)
			txt_theory_.setText(spec.getTheory().getID());
		if (belongs_to.equals("Pbl")) {
			problem_button_.setSelected(true);
		} else if (belongs_to.equals("Met")) {
			method_button_.setSelected(true);
		}
	}

	private void buildLabels() {
		label_theory_ = new JLabel(res_bundle_.getString("theory") + ":");
		label_method_ = new JLabel(res_bundle_.getString("method") + ":");
		label_problem_ = new JLabel(res_bundle_.getString("problem") + ":");
	}

	private boolean checkAndFormatSpecificationItems(Iterator it, JLabel txt) {
		boolean correct = true;
		while (it.hasNext()) {
			Object o = it.next();
			if (o instanceof ModelItem) {
				ModelItem chi = (ModelItem) o;
				if (chi.getStatus() != CalcHead.MODEL_ITEM_CORRECT) {
					txt.setForeground(CalcHeadView.MODEL_ITEM_INCORRECT_COLOR);
					correct = false;
				}
			} else if (o instanceof String) {
				String s = (String) o;
				txt.setText(txt.getText() + s);
			}
			if (it.hasNext())
				txt.setText(txt.getText() + ", ");
		}
		return correct;
	}

	/**
	 * Draws Border around given Label, to indicate whether a Model or a Guard
	 * is displayed
	 * 
	 * @param label
	 *            Label to highlight
	 * 
	 */

	public void highlightLabel(int label) {
		label_method_.setBorder(null);
		label_problem_.setBorder(null);
		label_theory_.setBorder(null);

		switch (label) {
		case LABEL_METHOD:
			label_method_.setBorder(BorderFactory.createEtchedBorder());
			break;
		case LABEL_PROBLEM:
			label_problem_.setBorder(BorderFactory.createEtchedBorder());
			break;
		case LABEL_THEORY:
			label_theory_.setBorder(BorderFactory.createEtchedBorder());
			break;
		}
	}

	/**
	 * Clears all Texts
	 */

	public void reset() {
		current_position_ = 0;
		clearTexts();
	}

	/**
	 * Fills Textfields stepwise from Specifiaction
	 * 
	 * @param spec
	 *            Specifiaction to fill TextFields from
	 * @deprecated was misunderstanding
	 * 
	 */

	public boolean next(Specification spec) {
		switch (current_position_) {
		case POSITION_THEORY:
			txt_theory_.setText(spec.getTheory().getID());
			current_position_++;
			return false;
		case POSITION_PROBLEM:
			this.checkAndFormatSpecificationItems(spec.getProblem().getID()
					.iterator(), txt_problem_);
			current_position_++;
			return false;
		case POSITION_METHOD:
			this.checkAndFormatSpecificationItems(spec.getMethod().getID()
					.iterator(), txt_method_);
			current_position_++;
			return true;
		default:
			return true;
		}

	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == problem_button_) {
			highlightLabel(LABEL_PROBLEM);
		} else {
			highlightLabel(LABEL_METHOD);
		}
		if (radio_button_handler_ != null) {
			radio_button_handler_.notifyUserAction(new UserAction(
					EUIElement.UI_SPECIFY_TOGGLE_METHOD_PROBLEM,
					EUIContext.UI_CONTEXT_CALCHEAD));
		}
	}

	public String getBelongsTo() {
		if (method_button_.isSelected()) {
			return "Met";
		} else {
			return "Pbl";
		}
	}

	public void addRadioButtonHandler(IGuiToGui radio_button_handler) {
		radio_button_handler_ = radio_button_handler;
	}

}