package isac.gui.treetable;

import isac.gui.Worksheet;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import java.util.ArrayList;
import java.util.EventObject;
import java.util.Iterator;

import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.border.LineBorder;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.tree.TreeCellRenderer;

public class CustomTreeCellEditor implements javax.swing.tree.TreeCellEditor, KeyListener {

	protected ArrayList<CellEditorListener> listeners_;
	protected Worksheet worksheet_;
	
	protected Object editedNode_;
	protected TreeCellRenderer renderer_;
	
	protected JTextField formulaText_;
	protected JTextField tacticText_;
	
	protected JPanel editorPanel_;
	
	private int subCell_;
	
	public static final int SUBCELL_FORMULA = 1;
	public static final int SUBCELL_TACTIC = 2;
	
	public CustomTreeCellEditor( TreeCellRenderer renderer, Worksheet ws ) {
		listeners_ = new ArrayList<CellEditorListener>();
		editedNode_ = null;
		subCell_ = 0;
		renderer_ = renderer;
		
		formulaText_ = new JTextField();
		tacticText_ = new JTextField();
		
		formulaText_.addKeyListener( this );
		tacticText_.addKeyListener( this );
		
		editorPanel_ = new JPanel();
		
		editorPanel_.setOpaque(true);
		editorPanel_.setBackground(UIManager.getColor("Tree.background"));
	    
		GridBagLayout gbl = new GridBagLayout();
		editorPanel_.setLayout(gbl);
	    
	    GridBagConstraints gbc = new GridBagConstraints();
	    gbc.gridwidth = GridBagConstraints.REMAINDER;
	    gbc.fill = GridBagConstraints.BOTH;
	    gbc.weightx = 1.0;
	    gbc.weighty = 1.0;
	        
	    gbc.anchor = GridBagConstraints.WEST;
	    gbl.setConstraints( formulaText_, gbc );
	    editorPanel_.add(formulaText_);
	    
	    gbc.anchor = GridBagConstraints.EAST;
	    gbc.insets = new Insets( 0, 30, 0, 0 );
	    gbl.setConstraints( tacticText_, gbc );
	    editorPanel_.add(tacticText_);
		
	    worksheet_ = ws;
	}
	
	public void activateSubCell( int subcell ) {
		subCell_ = subcell;
	}
	
	public int getSubCell() {
		return subCell_;
	}
	
	public void addCellEditorListener(CellEditorListener listener) {
		if( !listeners_.contains( listener ) )
			listeners_.add( listener ); 
		System.out.println("Listeners : " + listeners_.size() );
		Iterator it = listeners_.iterator();
		while(it.hasNext())
			System.out.println( it.next().toString() );
	}
	
	public void notifyWorksheet() {
		if( worksheet_ != null )
			worksheet_.editingStopped( new ChangeEvent( this ) );
	}

	public Object getCellEditorValue() {
		return editedNode_;
	}

	public boolean isCellEditable(EventObject e) {
		return true;
	}

	public void removeCellEditorListener(CellEditorListener listener) {
		listeners_.remove( listener ); 
	}

	public boolean shouldSelectCell(EventObject e) {
		return true;
	}

	public boolean stopCellEditing() {
		for( int i = 0; i < listeners_.size(); i++ )
			((CellEditorListener)(listeners_.get(i))).editingStopped( new ChangeEvent( this ) );
		return true;
	}

	public void cancelCellEditing() {
		for( int i = 0; i < listeners_.size(); i++ )
			((CellEditorListener)(listeners_.get(i))).editingCanceled( new ChangeEvent( this ) );
	}	
		
	public Component getTreeCellEditorComponent(JTree tree, Object value, 
			boolean isSelected, boolean expanded, boolean leaf, int row) {

		if( !isSelected ) {
			return renderer_.getTreeCellRendererComponent(tree, value, true, expanded, leaf, row, true);
		}
		
		editedNode_ = value;
		if( !(value instanceof CalcModelNode ) )
			return null; 
    
		CalcModelNode current_node = (CalcModelNode) value;
		
		formulaText_.setText( current_node.toString() );
		
		try {
			tacticText_.setText(((CalcModelNode)value).getSecondaryString() + "  ");
		} 
		catch (Exception e) {
			tacticText_.setText("");
		}
			    		
		if( subCell_ == SUBCELL_TACTIC ) {
			formulaText_.setEditable( false );
			formulaText_.setEnabled( false );
			tacticText_.setBorder(new LineBorder( Color.BLACK  ) );			
			tacticText_.setEditable( true );
			tacticText_.setEnabled( true );
			tacticText_.requestFocus();
		}
		
		if( subCell_ == SUBCELL_FORMULA ) {
			tacticText_.setEditable( false );
			tacticText_.setEnabled( false );
			formulaText_.setBorder(new LineBorder( Color.BLACK  ) );
			formulaText_.setEditable( true );
			formulaText_.setEnabled( true );
			formulaText_.requestFocus();
		}		
		
		Dimension panel_dimension = editorPanel_.getPreferredSize();
		Rectangle worksheet_rect = this.worksheet_.getBounds();
		editorPanel_.setPreferredSize(new Dimension(worksheet_rect.width - 50, panel_dimension.height));
		
		return editorPanel_;
	}

	public void keyPressed(KeyEvent arg0) {

	}

	public void keyReleased(KeyEvent arg0) {
		if( arg0.getKeyCode() == KeyEvent.VK_ENTER ) {
			editedNode_ = new String( formulaText_.getText() );
			notifyWorksheet();
		}
	}

	public void keyTyped(KeyEvent arg0) {

	}

}
