/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.gui.util;

import isac.util.formulae.HierarchyKey;
import isac.util.formulae.KEStoreKey;

import java.io.Serializable;
import java.util.Vector;

/**
 * Abstract class for nodes in the hierarchy (ExampleHierarchy, ...).
 * 
 * @author Mario Hochreiter, JL
 * @version 1.0
 *  
 */
public class HierarchyNode implements Serializable {

    static final long serialVersionUID = 438029646987696684L;
    
	protected String id_;
    protected KEStoreKey ke_store_key_;
    
    protected HierarchyNode parent_;
    protected Vector<HierarchyNode> children_;

    public HierarchyNode( ) {
    	parent_ = null;
    	children_ = new Vector<HierarchyNode>();
    }
    
    public boolean add( HierarchyNode child ) {
    	child.parent_ = this;
    	return children_.add( child );
    }
    
    /**
     * 
     * @param child
     * @param index
     * @return true is successful, false if node could not be added at the
     *         specuified position
     */
    public boolean add(int index, HierarchyNode child) {
        try {
            children_.add(index, child);
            child.parent_ = this;
            return true;
        } catch (ArrayIndexOutOfBoundsException e) {
            return false;
        }
    }
    
    /**
     * @return the children of this node
     */
    public Vector<HierarchyNode> getChildren() {
        return children_;
    }
    
    /**
     * @param index
     * @return the child at the specified index
     */
    public HierarchyNode getChild(int index) {
    	if( index >= 0 && index < children_.size() )
    		return children_.get(index);
    	else
    		throw new IllegalArgumentException("in HierarchyNode::getChild() index is out of range"); 
    }
    
    public void setId(String id) {
        this.id_ = id;
    }

    public KEStoreKey getKEStoreKey() {
        return ke_store_key_;
    }
    
	/**
	 * @return
	 */
	public HierarchyKey getHierarchyKey() {
		HierarchyKey key = new HierarchyKey();
		HierarchyNode node = this;
		while( node.parent_ != null ) {
			key.addString( node.getId() );
			node = node.getParent();
		}
		
		return key;
	}    

    public String getId() {
        return id_;
    }

    public void setKEStoreKey( KEStoreKey contentref ) {
        this.ke_store_key_ = contentref;
    }

    public String toString() {
        return id_;
    }

	/**
	 * @return Returns the parent_.
	 */
	public HierarchyNode getParent() {
		return parent_;
	}

	/**
	 * @param parent_ The parent_ to set.
	 */
	public void setParent(HierarchyNode parent) {
		parent_ = parent;
	}
	
	public boolean equals( HierarchyNode o ) {
		System.out.println("Equals called for " + this + " and " + o );
		System.out.println("id " + id_ + " and " + o.id_ );
		System.out.println("keStoreKey " + ke_store_key_ + " and " + o.ke_store_key_ );
		return ( id_ == o.id_ && ke_store_key_ == o.ke_store_key_  );
	}
}