/**
 * @author Alan Krempler
 * Created on Dec 11, 2003
 */
package isac.interfaces;

import java.rmi.Remote;
import java.rmi.RemoteException;

import isac.useractions.IUIAction;
import isac.useractions.IUserAction;

/**
 * Interface for requesting actions from the worksheet user-interface.
 * Distinction between this interface and the interface IToUser: while the
 * interface IToUser is (among other things) used to notify the worksheet from
 * changes in the undelying structur of the calculation, the IToGUI interface
 * enables the dialog to change the behaviour of the gui
 * 
 * @author Alan Krempler
 *  
 */
public interface IToGUI extends Remote {

    /**
     * Set the active/inactive status of a UI object
     * 
     * @param id
     *            ID of a UI object
     * @param active
     *            Desired status (true means active)
     *            
     *            Removed because no one uses this method
     */
//    public void activateUIObject(int id, boolean active) throws RemoteException;

    /**
     * first general idea for action through this interface
     * 
     * @deprecated should be replaced by doUIAction(IUIAction)
     */
    public boolean doUIAction(IUserAction action) throws RemoteException;

    /**
     * inform the listeners, i.e. the user-interface, that an action on is
     * requested, as soon as the worksheet is ready with rendering.
     */
    public void doUIAction(IUIAction action) throws RemoteException;

    /**
     * is called from the WorksheetDialog; adds an UIElement to the GUI; the GUI
     * decides where and how to show this UIElement depending on the context
     * 
     * @param action:
     *            the action contains the context and the action id
     */
    public void addUIElement(IUIAction action) throws RemoteException;

    /**
     * removes an UIELement specified by an action
     */
    public boolean removeUIElement(IUIAction action) throws RemoteException;

    /***
     * 
     */
    public boolean isUIRunning() throws RemoteException;
}