/**
 * @author Andreas Griesmayer
 * Created on Aug 19, 2003
 */
package isac.session;

import java.lang.reflect.Method;
import java.util.Vector;

import org.apache.xmlrpc.XmlRpcHandler;

abstract public class Handler implements XmlRpcHandler {

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.xmlrpc.XmlRpcHandler#execute(java.lang.String,
     *      java.util.Vector)
     */
    /**
     * sends the <code>message</code> with the <code>parameter</code> s to
     * the <code>target</code> using java-reflection. Message-passing to an
     * object complies with the object-orientated paradigm and corresponds to an
     * method call. Is used to call methods on the target which are previously
     * not known.
     * 
     * @param target
     *            Object to send the message to
     * @param message
     *            name of the method to call
     * @param params
     *            parameters of the method to call
     */
    protected Object methodCall(Object target, String message, Vector params)
            throws Exception {
        Class params_type[] = getTypes(params);
        Object params_obj[] = params.toArray();

        Class target_class = target.getClass();
        Method method = target_class.getDeclaredMethod(message, params_type);
        return method.invoke(target, params_obj);
    }

    private Class[] getTypes(Vector params) {
        Class[] types = new Class[params.size()];
        
        for (int index = 0; index < params.size(); index++)
            types[index] = params.elementAt(index).getClass();
        
        return types;
    }
}