/*
 * @author Robert Koenighofer, member of the ISAC-team, 
 * Copyright (c) 2006 by Robert Koenighofer
 * created Jul 11, 2006 10:09:13 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */

package isac.users;

import java.rmi.RemoteException;
import java.rmi.Remote;
import isac.browserdialog.IBrowserDialog;

/**
 * This is an interface for the UserManager. The UserManager is responsible
 * for the administration of the users.
 * 
 * @author Robert Koenighofer
 * @see isac.user.UserManager
 */
public interface IUserManager extends Remote{

	/**
	 * Checks by use of {@link isac.users.Accounts} if username and 
	 * password is correct. If this is true an new Session is created by 
	 * use of the {@link isac.session.SessionManager}. An interface to the
	 * created Session object is returned. If the login fails, null
	 * is returned.
	 * 
	 * @param username the name of the user who wants to login
	 * @param pass the password the user has entered
	 * @return an interface to the Session which was created for the user or null if login fails
	 * @throws RemoteException if remote object is not available any more
	 * @see isac.user.UserManager#login(String, String)
	 */
	public IBrowserDialog login(String username, String pass) throws RemoteException;

	/**
	 * Returns the User object of the user belonging to the session
	 * with the specified ID. If no Session with this ID is
	 * known, null is returned.
	 * 
	 * @param session_id
	 * @return the User who owns the session or null if session_id is unknown
	 * @throws RemoteException if remote object is not available any more
	 * @see isac.user.UserManager#getUser(String)
	 */
	public User getUser(String session_id) throws RemoteException;

	/**
	 * Returns the username of the user belonging to the session
	 * with the specified ID as String. If no Session with this ID is
	 * known, null is returned.
	 * 
	 * @param session_id
	 * @return the username who owns the session or null if session_id is unknown
	 * @throws RemoteException if remote object is not available any more
	 * @see isac.user.UserManager#getUsername(String)
	 */
	public String getUsername(String session_id) throws RemoteException;

}
