/*
 * @author Alan Krempler
 * Created on Feb 22, 2005
 */
package isac.users;

import isac.util.formulae.CalcElement;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.TreeMap;
import java.util.Vector;

/**
 * @author Alan Krempler
 * 
 * The UserModel is a class to gather statistics about one user's interaction
 * with the system. Statistis are gathered per user and per interactional
 * context.
 */
public class UserModel extends Properties {

	//	special value for retrieving the statistics regardless of the dialog atom
	// used in the interaction
	public static final int DIALOG_ATOM_ANY = 0;

	// counter for identifying pending interactions
	private static int interaction_nr_ = 1;

	private String userpath_;
	private String username_;
	
    /**
     * successInfos stores the successInfos for CalcElements. that means the key
     * for this TreeMap is a calcElement, the value is the relating SuccessInfo
     */
    private TreeMap success_infos_;

	public UserModel (String userpath, String username){
		userpath_=userpath;
		username_=username;
		//TODO.WN050311
	}

	
	/**
	 * @param item_id
	 *            ID of the item being queried, a tactic for example
	 * @param dialog_atom
	 *            ID of the interactional context use DIALOG_ATOM_ANY if asking
	 *            for statistics for all dialog atoms
	 * @return count of previously recorded interactions with this item
	 */
	public int getTouchedCount(int item_id, int dialog_atom) {
		// TODO.AK0502: returning dummy value in absence of valid statistics
		return 0;
	}

	/**
	 * @param item_id
	 *            ID of the item being queried, a tactic for example
	 * @param dialog_atom
	 *            ID of the interactional context use DIALOG_ATOM_ANY if asking
	 *            for statistics for all dialog atoms
	 * @return count of previously recorded successful interactions with this
	 *         item
	 */
	public int getSuccessCount(int item_id, int dialog_atom) {
		// TODO.AK0502: returning dummy value in absence of valid statistics
		return 0;
	}

	/**
	 * @param item_id
	 *            ID of the item being queried, a tactic for example
	 * @param dialog_atom
	 *            ID of the interactional context use DIALOG_ATOM_ANY if asking
	 *            for statistics for all dialog atoms
	 * @return time of last recorded interaction with this item
	 */
	public java.util.Date getTouchedLast(int item_id, int dialog_atom) {
		// TODO.AK0502: returning dummy value in absence of valid statistics
		return new java.util.Date();
	}

	/**
	 * @param item_id
	 *            ID of the item being queried, a tactic for example
	 * @param dialog_atom
	 *            ID of the interactional context use DIALOG_ATOM_ANY if asking
	 *            for statistics for all dialog atoms
	 * @return time of last recorded successful interaction with this item
	 */
	public java.util.Date getSuccessLast(int item_id, int dialog_atom) {
		// TODO.AK0502: returning dummy value in absence of valid statistics
		return new java.util.Date();
	}

	/**
	 * @param item_id
	 *            ID of the item being queried, a tactic for example
	 * @param dialog_atom
	 *            ID of the interactional context use DIALOG_ATOM_ANY if asking
	 *            for statistics for all dialog atoms
	 * @return average time (seconds) spent in interactions with this item
	 */
	public int getTimeSpentAvg(int item_id, int dialog_atom) {
		// TODO.AK0502: returning dummy value in absence of valid statistics
		return 0;
	}

	/**
	 * @param item_id
	 *            ID of the item being queried, a tactic for example
	 * @param dialog_atom
	 *            ID of the interactional context use DIALOG_ATOM_ANY if asking
	 *            for statistics for all dialog atoms
	 * @return maximum time (seconds) spent in interactions with this item
	 */
	public int getTimeSpentMax(int item_id, int dialog_atom) {
		// TODO.AK0502: returning dummy value in absence of valid statistics
		return 0;
	}

	/**
	 * @param item_id
	 *            ID of the item being queried, a tactic for example
	 * @param dialog_atom
	 *            ID of the interactional context use DIALOG_ATOM_ANY if asking
	 *            for statistics for all dialog atoms
	 * @return standard deviation of time spent in interactions with this item
	 */
	public float getTimeSpentStddev(int item_id, int dialog_atom) {
		// TODO.AK0502: returning dummy value in absence of valid statistics
		return 0;
	}

	/**
	 * Mark the beginning of an interaction with an item.
	 * 
	 * @param item_id
	 *            ID of the item being acted upon, a tactic for example
	 * @param dialog_atom
	 *            ID of the interactional context
	 * @return unique number to indentify the end of the transaction
	 */
	public int startInteraction(int item_id, int dialog_atom) {
		// TODO.AK0502: insert code to record statistics

		// record timestamp
		// new java.util.Date(); ...

		// save in static vector of currently open interactions
		// ...

		return interaction_nr_++;
	}

	/**
	 * Mark the end of an interaction with an item.
	 * 
	 * @param interaction_id
	 *            ID of the interaction returned from the call to
	 *            {@link # startInteraction() }
	 * @param success
	 *            information about the success of the interaction
	 * @return false in case of failure - no matching start could be found, true
	 *         in case of success
	 */
	public boolean endInteraction(int interaction_id, int success) {
		// TODO.AK0502: insert code to record statistics

		// record end timestamp
		// new java.util.Date(); ...

		// add to statistics

		// delete from static vector of currently open interactions
		// ...

		return true;
	}

	//WN050311 just for creation of the first file
	//beware of the fix path !
	public static void main(String[] args) {
		UserModel a = new UserModel("isac/xmldata/users","x");
		a.setProperty("todo-key", "todo-value");

		String filename = "isac/xmldata/users/x_model.txt";
		try {
			FileOutputStream propOutFile = new FileOutputStream(filename);
			a.store(propOutFile, "dynamic dialog data of user 'x'");

		} catch (IOException e) {
			System.err.println("I/O failed.");
		}
	}
	
    /**
     * gets the info for each CalcElement of the parameter calcElements
     * 
     * @param calcElements
     *            this Vector must not contain anything else than CalcElements
     * @return a TreeMap with the SuccessInfo for each CalcElement in the
     *         calcElements Vector. Those CalcElements are the index in the
     *         HashMap, the relating SuccessInfos are the values
     */
    public Map getSuccessInfoFor(Vector calcElements) {
        TreeMap result = new TreeMap();
        Iterator iterator = calcElements.iterator();
        while (iterator.hasNext()) {
            CalcElement calcElement = (CalcElement) iterator.next();
            if (success_infos_.containsKey(calcElement)) {
                result.put(calcElement, success_infos_.get(calcElement));
            } else {
                // no successInfo available
                result.put(calcElement, null);
            }
        }
        //TODO
        return result;
    }

    /**
     * @param calcElement
     * @return SuccessInfo or null if no SuccessInfo available
     */
    public SuccessInfo getSuccessInfoFor(CalcElement calcElement) {
        return (SuccessInfo) success_infos_.get(calcElement);
    }

    /**
     * @param calcElementsSuccesses,
     *            a TreeMap with a CalcElement as key and the relating success
     *            (Float between 0 and 1) as value
     */
    public void addSuccessInfoFor(Map calcElementsSuccesses)
            throws ClassCastException {
        Set set = calcElementsSuccesses.keySet();
        Iterator iterator = set.iterator();
        while (iterator.hasNext()) {
            try {
                CalcElement calcElement = (CalcElement) iterator.next();
                Float success = (Float) calcElementsSuccesses.get(calcElement);
                addSuccessInfoFor(calcElement, success.floatValue());
            } catch (ClassCastException e) {
                throw new ClassCastException(
                        "the parameter for the method "
                                + "void addSuccessInfoFor(TreeMap calcElementsSuccesses) "
                                + "must have CalcElements as keys and Floats as value!!");
            }
        }
    }

    public void addSuccessInfoFor(CalcElement calcElement, float success) {
        SuccessInfo successInfo;
        if (success_infos_.containsKey(calcElement)) {
            successInfo = (SuccessInfo) success_infos_.get(calcElement);
            successInfo.addSuccess(success);
            success_infos_.put(calcElement, successInfo);
        } else {
            successInfo = new SuccessInfo(success);
            success_infos_.put(calcElement, successInfo);
        }
    }

}