/*
 * Created on Feb 22, 2005
 *
 * @author Alan Krempler
 */

package isac.users;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Properties;

/**
 * Class for storing/retrieving user settings, i.e. settings which can be set
 * manually by the user as opposed to the
 * {@link isac.util.usersettings.UserModel#}, which is an abstraction of the
 * system's expeience with a user and canot be altered manually. As there is no
 * fixed set of user settings, you can regard this class as a simple storage
 * backend for key/value pairs.
 * 
 * @author Alan Krempler
 *  
 */
public class UserSettings extends Properties implements IUserSettings {

    private HashMap<String, Integer> storage_;

    String userpath_;

    String username_;

    public static HashMap<String, Integer> valstr_valint_;

    /**
     * connect the String-representation of a UserSettings-value with the
     * respective Integer-representation
     * 
     * @see isac.uti.users.IUserSettings
     */
    private HashMap<String, Integer> createValstrValint() {
        HashMap<String, Integer> m = new HashMap<String, Integer>();
        m.put("DUMMY", new Integer(IUserSettings.DUMMY));
        m.put("VAL1_SKIP_SPECIFY_TO_START_SOLVE", new Integer(
                IUserSettings.VAL1_SKIP_SPECIFY_TO_START_SOLVE));
        m.put("VAL1_POP_CALCHEAD_WITH_DESCRIPTIONS", new Integer(
                IUserSettings.VAL1_POP_CALCHEAD_WITH_DESCRIPTIONS));
        
        m.put("VAL2_FORMULAE_ONLY", new Integer(
                IUserSettings.VAL2_FORMULAE_ONLY));
        m.put("VAL2_FORMULA_TACTIC_FORMULA_", new Integer(
                IUserSettings.VAL2_FORMULA_TACTIC_FORMULA_));
        
        m.put("VAL3_LEARNER", new Integer(IUserSettings.VAL3_LEARNER));
        m.put("VAL3_AUTHOR", new Integer(IUserSettings.VAL3_AUTHOR));
        
        m.put("VAL4_ENGLISH",
        		new Integer(IUserSettings.VAL4_ENGLISH));
        m.put("VAL4_DEUTSCH",
        		new Integer(IUserSettings.VAL4_DEUTSCH));
        //m.put("",
        //		new Integer(IUserSettings.));
        return m;
    }

    public UserSettings(String userpath, String username) {
        userpath_ = userpath;
        username_ = username;

        //load Properties (keystr, valstr) temporarily
        Properties keystr_valstr = new Properties();
        String filename = userpath_ + username + "_settings.txt";
        try {
            FileInputStream propInFile = new FileInputStream(filename);
            keystr_valstr.load(propInFile);
            keystr_valstr.list(System.out);
        } catch (FileNotFoundException e) {
            System.err.println("Can?t find " + filename);
        } catch (IOException e) {
            System.err.println("I/O failed.");
        }

        //create HashMap (valstr, valint) temporarily
        HashMap<String, Integer> valstr_valint = new HashMap<String, Integer>();
        valstr_valint = this.createValstrValint();

        //create HashMap (keystr, \\valstr\\ valint) and store it
        storage_ = new HashMap<String, Integer>();
        Enumeration keystrs = keystr_valstr.propertyNames();//WN050311 how
        // better ?
        String keystr = null;
        String valstr = null;
        Integer valint = null;
        while (keystrs.hasMoreElements()) {
            keystr = (String) keystrs.nextElement();
            valstr = keystr_valstr.getProperty(keystr);
            valint = valstr_valint.get(valstr);
            storage_.put(keystr, valint);
        }
    }

    /**
     * @param key
     *            A string key
     * @return The string value associated with the key parameter; null, if
     *         nothing is stored under this key
     */
    public int getValue(String key) {
        return ((Integer) storage_.get(key)).intValue();
    }

    /**
     * @param key
     *            The key under which "value" is to be stored. Existing data
     *            unde the same key will be replaced.
     * @param value
     *            The value to be stored under "key"
     * @return true in case of success, false in case of error, for example if
     *         trying to replace stored values without proper permissions
     */
    public boolean setValue(String key, int value) {
        //TODO.WN050311 storage_.put(key, value);
        return true;
    }

    //WN050311 just for creation of the first file
    //beware of the fix path !
    public static void main(String[] args) {
        UserSettings s = new UserSettings("isac/xmldata/users/", "x");
        s.setProperty("start_worksheet", "SKIP_SPECIFY_START_SOLVE");
        s.setProperty("enter_specify", "SKIP_SPECIFY_START_SOLVE");
        s.setProperty("next_button", "FORMULAE_ONLY");

        String filename = "isac/xmldata/users/x_settings.txt";
        try {
            FileOutputStream propOutFile = new FileOutputStream(filename);
            s.store(propOutFile, "static dialog data of user 'x'");

        } catch (IOException e) {
            System.err.println("I/O failed.");
        }
    }

}

