package isac.util;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;


/**
 * paths for setup of the ObjectManager; for RMI and for data.
 * 
 * @author Manuel Koschuch (c) Manuel Koschuch 2006
 */
public class ObjectManagerPaths {

	/**
	 * RMI to mathengine (handled through several constructors)
	 */
	public static String MATH_ENGINE_PATH;

	// localhost

	/**
	 * RMI to the ObjectManager
	 */
	public static String OBJECT_MANAGER_PATH;

	// "//localhost/isac-ObjectManager"

	/**
	 * port to the RMI registry
	 */
	public static int OBJECT_MANAGER_PORT;
	
	
	/**
	 * port to the RMI object manager object
	 */
	public static int OBJECT_MANAGER_RMI_PORT;

	// 1099

	/**
	 * path to the user data
	 */
	public static String SESSION_MANAGER_USER_PATH;

	// "..../isac/xmldata/users/"

	/**
	 * RMI to the XBrowserDialog
	 */
	public static String BD_DIALOG_PATH;

	// "http://localhost:1050"

	/**
	 * RMI to the WorksheetDialog
	 */
	public static String WORKSHEET_DIALOG_PATH;

	// "//localhost/isac-WorksheetDialog"

	/**
	 * RMI to KEStore
	 */
	public static String KESTORE_PATH;
	// "http://localhost:1030"
	
	/**
	 * Root directory of html files for the XMiniBrowser
	 */
	public static String WEB_PATH;
	// "http://www.ist.tugraz.at/projects/isac/www/kbase-new/"
	
	public static String loadFromResource(String resname)
	{
		InputStream is = ObjectManagerPaths.class.getResourceAsStream(resname);
		
		if (is == null)
			return "ObjectManager: Initialisation file not found";
		
		return loadFromStream(is);
	}
	
	public static String loadFromFile(String filename)
	{
		FileInputStream fis = null;
	
		try
		{
			fis = new FileInputStream(filename);
		}
		catch( FileNotFoundException fnfex )
		{
			System.out.println(fnfex.getMessage());
			return "\nObjectManager: Initialisation file not found in resources";
		}
		
		return loadFromStream(fis);
	}
	
	private static String loadFromStream(InputStream is)
	{
		Properties p = new Properties();
		
		try
		{
			p.load(is);
		}
		catch( IOException ioex)
		{
			System.out.println(ioex.getMessage());
			return "ObjectManager: Initialisation file is not a Property file";
		}
		
		if ((MATH_ENGINE_PATH = p.getProperty("MATH_ENGINE_PATH")) == null)
			return "ObjectManager: MATH_ENGINE_PATH is required parameter";
		
		if ((OBJECT_MANAGER_PATH = p.getProperty("OBJECT_MANAGER_PATH")) == null)
			return "ObjectManager: OBJECT_MANAGER_PATH is required parameter";
		
		if (p.getProperty("OBJECT_MANAGER_PORT") == null)
			return "ObjectManager: OBJECT_MANAGER_PORT is required parameter";
		
		if (p.getProperty("OBJECT_MANAGER_RMI_PORT") == null)
			return "ObjectManager: OBJECT_MANAGER_RMI_PORT is required parameter";
			
		
		try
		{
			OBJECT_MANAGER_RMI_PORT = Integer.parseInt(p.getProperty("OBJECT_MANAGER_RMI_PORT"));
		}
		catch(NumberFormatException nfe)
		{
			System.out.println(nfe.getMessage());
			return "ObjectManager: OBJECT_MANAGER_RMI_PORT is not a number";
		}
		
		try
		{
			OBJECT_MANAGER_PORT = Integer.parseInt(p.getProperty("OBJECT_MANAGER_PORT"));
		}
		catch(NumberFormatException nfe)
		{
			System.out.println(nfe.getMessage());
			return "ObjectManager: OBJECT_MANAGER_PORT is not a number";
		}
		
		if ((SESSION_MANAGER_USER_PATH = p.getProperty("SESSION_MANAGER_USER_PATH")) == null)
			return "ObjectManager: SESSION_MANAGER_USER_PATH is required parameter";
		
		if ((BD_DIALOG_PATH = p.getProperty("BD_DIALOG_PATH")) == null)
			return "ObjectManager: BD_DIALOG_PATH is required parameter";
		
		if ((WORKSHEET_DIALOG_PATH = p.getProperty("WORKSHEET_DIALOG_PATH")) == null)
			return "ObjectManager: WORKSHEET_DIALOG_PATH is required parameter";
		
		if ((KESTORE_PATH = p.getProperty("KESTORE_PATH")) == null)
			return "ObjectManager: KESTORE_PATH is required parameter";
		
		if ((WEB_PATH = p.getProperty("WEB_PATH")) == null)
			return "ObjectManager: WEB_PATH is required parameter";
		
		return null;
	}

}
