/*
 * @author Robert Koenighofer, member of the ISAC-team, 
 * Copyright (c) 2006 by Robert Koenighofer
 * created Jul 11, 2006 10:09:13 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */

package isac.util.formulae;

import java.util.Vector;

/**
 * FIXXME: what contains this class in detail?
 * 
 * @author Robert Koenighofer
 */
public class ContextTheory extends Context {

	/**
	 * For serialization, see corresponding API Documentation; value generated
	 * by eclipse to avoid a warning
	 */
	private static final long serialVersionUID = -7725863587815147098L;

	/** should be a list of substitutions */
	private Formula subslist_;

	private Formula instanciated_to_;

	private Formula applied_to_;

	private Formula applied_at_;

	// TODO GK060717 has to be changed to a reference object
	private String order_;

	private Vector<FormulaPair> eval_assumtions_ = new Vector<FormulaPair>();

	private Formula lhs_;

	private Formula lhs_instanciated_;

	private Formula rhs_;

	private Formula rhs_instanciated_;

	private Formula result_;

	private String assumtion_;

	private Reference eval_rls_;

	public Formula getSubsList() {
		return subslist_;
	}

	public Formula getInstanciatedTo() {
		return instanciated_to_;
	}

	public Formula getAppliedTo() {
		return applied_to_;
	}

	public Formula getAppliedAt() {
		return applied_at_;
	}

	public String getOrder() {
		return order_;
	}

	public FormulaPair getEvalAssumtion(int position) {
		if (position < eval_assumtions_.size()) {
			return eval_assumtions_.get(position);
		}
		// TODO GK060718 if the position can not be found in the vector do
		// something...
		return null;
	}

	public Formula getLHS() {
		return lhs_;
	}

	public Formula getLHSInstanciated() {
		return lhs_instanciated_;
	}

	public Formula getRHS() {
		return rhs_;
	}

	public Formula getRHSInstanciated() {
		return rhs_instanciated_;
	}

	public Formula getResult() {
		return result_;
	}

	public String getAssumtions() {
		return assumtion_;
	}

	public Reference getEvalRLS() {
		return eval_rls_;
	}

	public void setSubsList(Formula subslist) {
		this.subslist_ = subslist;
	}

	public void setInstanciatedTo(Formula instanciated_to) {
		this.instanciated_to_ = instanciated_to;
	}

	public void setAppliedTo(Formula applied_to) {
		this.applied_to_ = applied_to;
	}

	public void setAppliedAt(Formula applied_at) {
		this.applied_at_ = applied_at;
	}

	public void setOrder(String order) {
		this.order_ = order;
	}

	public void setEvalAssumtion(FormulaPair formula_pair) {
		this.eval_assumtions_.add(formula_pair);
	}

	public void setLHS(Formula lhs) {
		this.lhs_ = lhs;
	}

	public void setLHSInstanciated(Formula lhs_inst) {
		this.lhs_instanciated_ = lhs_inst;
	}

	public void setRHS(Formula rhs) {
		this.rhs_ = rhs;
	}

	public void setRHSInstanciated(Formula rhs_inst) {
		this.rhs_instanciated_ = rhs_inst;
	}

	public void setResult(Formula result) {
		this.result_ = result;
	}

	public void setAssumtions(String assumtion) {
		this.assumtion_ = assumtion;
	}

	public void setEvalRLS(Reference eval_rls) {
		this.eval_rls_ = eval_rls;
	}

	/**
	 * Transforms the content of the object into a string representation.
	 * 
	 * @return the string representation of the Context
	 */
	/*
	 * public String toString() { // TODO RK060710 implement me // System.out //
	 * .println("TODO: isac.util.formulae.ContextTheory#toString(): // implement
	 * me!"); // System.exit(-1); return null; }
	 */

	/**
	 * Transforms the content of the object into a HTML representation.
	 * 
	 * @return the HTML representation of the Context
	 */
	public String toHTML() {

		String result = "<div class=\"context\">\n";
		result += "		<table style=\"width: 100%; height: 124px;\">\n";
		result += "		  <tbody>\n";
		if (subslist_ != null) {
			result += "		    <tr>\n";
			result += "		      <td width=\"112\">instantiated by:</td>\n";
			result += "		      <td width=\"609\"><em>\n";
			result += "		" + subslist_.toString() + "\n";
			result += "		</em></td>\n";
			result += "		    </tr>\n";
		}
		if (instanciated_to_ != null) {
			result += "		    <tr>\n";
			result += "		      <td>to:</td>\n";
			result += "		      <td><em>\n";
			result += "               " + instanciated_to_.toString() + "\n";
			result += "\n";
			result += "		</em></td>\n";
			result += "		    </tr>\n";
		}
		if (applied_to_ != null) {
			result += "		    <tr>\n";
			result += "	      <td><strong><font color=\"#00cc33\">applied to:</font></strong></td>\n";
			result += "		      <td style=\"font-weight: bold;\"><em>\n";
			result += "\n";
			result += applied_to_.toString() + "\n";
			result += "\n";
			result += "		</em></td>\n";
			result += "		    </tr>\n";
		}
		if (applied_at_ != null) {
			result += "		    <tr>\n";
			result += "		      <td>at:</td>\n";
			result += "		      <td><em>\n";
			result += "\n";
			result += applied_at_.toString() + "\n";
			result += "\n";
			result += "		</em></td>\n";
			result += "		    </tr>\n";
		}
		if (order_ != null) {
			result += "		    <tr>\n";
			result += "		      <td>under order:</td>\n";
			result += "		      <td>\n";
			result += "\n";
			result += order_ + "\n";

			result += "\n";
			result += "		</td>\n";
			result += "		    </tr>\n";
		}
		result += "		  </tbody>\n";
		result += "		</table>\n";

		result += "<table style=\"text-align: left; width: 100%;\" border=\"0\"\n";
		result += "			 cellspacing=\"2\" cellpadding=\"2\">\n";
		result += "			  <tbody>\n";
		result += "\n";
		if (eval_assumtions_ != null) {
			for (int count = 0; count < eval_assumtions_.size(); count++) {
				result += "			    <tr>\n";
				result += "			      <td style=\"vertical-align: top; text-align: center;\"><em>\n";
				result += "			      \n";
				result += eval_assumtions_.get(count).getAssumtion().toString();
				result += "\n";
				result += "			</em></td>\n";
				result += "			      <td style=\"vertical-align: top; text-align: center;\">..evaluates\n";
				result += "			to..<br></br>\n";
				result += "			      </td>\n";
				result += "			      <td style=\"vertical-align: top; text-align: center;\"><span\n";
				result += "			 style=\"font-style: italic;\">\n";
				result += "			 \n";
				result += eval_assumtions_.get(count).getValue().toString();
				result += "			 \n";
				result += "			 </span><br></br>\n";
				result += "			      </td>\n";
				result += "			    </tr>\n";
			}
		}
		result += "\n";
		result += "\n";
		if (lhs_ != null && lhs_instanciated_ != null) {
			result += "			<tr>\n";
			result += "			      <td style=\"vertical-align: top; text-align: center;\"><em><font\n";
			result += "			 color=\"#00cc33\">\n";
			result += "\n";

			result += lhs_.toString();

			result += "\n";
			result += "\n";
			result += "			</font></em></td>\n";
			result += "			      <td style=\"vertical-align: top;\">\n";
			result += "			      <div style=\"text-align: center;\"><span style=\"font-weight: bold;\">..matches..</span></div>\n";
			result += "			      </td>\n";
			result += "			      <td style=\"vertical-align: top; text-align: center;\"><em>\n";
			result += "			      \n";
			result += lhs_instanciated_.toString();
			result += "\n";
			result += "			</em></td>\n";
			result += "			    </tr>\n";
		}
		if (rhs_ != null && rhs_instanciated_ != null) {
			result += "			<tr>\n";
			result += "			      <td style=\"vertical-align: top; text-align: center;\"><em>\n";
			result += "\n";
			result += rhs_.toString();
			result += "\n";
			result += "			<br></br>\n";
			result += "			      </em></td>\n";
			result += "			      <td style=\"vertical-align: top;\">\n";
			result += "			      <div style=\"text-align: center;\"><span style=\"font-weight: bold;\">..instantiates\n";
			result += "			to..</span></div>\n";
			result += "			      </td>\n";
			result += "			      <td style=\"vertical-align: top; text-align: center;\"><em><font color=\"#00cc33\">\n";
			result += "			 \n";
			result += rhs_instanciated_.toString();
			result += "\n";
			result += "			</font></em><em></em></td>\n";
			result += "			    </tr>\n";
		}
		result += "			  </tbody>\n";
		result += "			</table>\n";

		result += "		<table style=\"text-align: left; width: 100%;\" border=\"0\" cellspacing=\"2\"\n";
		result += "			 cellpadding=\"2\">\n";
		result += "			  <tbody>\n";
		if (result_ != null) {
			result += "			    <tr>\n";
			result += "			      <td style=\"vertical-align: top;\"> <strong>results in:</strong></td>\n";
			result += "			      <td style=\"vertical-align: top; font-weight: bold;\"><em>\n";
			result += "\n";
			result += "\n";
			result += result_.toString();
			result += "\n";
			result += "			 </em></td>\n";
			result += "			    </tr>\n";
		}
		if (assumtion_ != null) {
			result += "			    <tr>\n";
			result += "			      <td style=\"vertical-align: top;\">assuming:<br></br>\n";
			result += "			      </td>\n";
			result += "			      <td style=\"vertical-align: top;\">\n";
			result += "\n";
			result += assumtion_;
			result += "\n";
			result += "			 <br></br>\n";
			result += "			      </td>\n";
			result += "			    </tr>\n";
		}
		if (eval_rls_ != null) {
			result += "			    <tr>\n";
			result += "			      <td style=\"vertical-align: top;\">evaluated by:<br></br>\n";
			result += "			      </td>\n";
			result += "			      <td style=\"vertical-align: top;\">\n";
			result += "\n";

			String kestore_key = eval_rls_.getKEStoreKey().toString();
			if (kestore_key.endsWith("xml"))
				kestore_key = kestore_key.replaceFirst(".xml", "");
			String link_target = "../" + kestore_key.substring(0, 3) + "/"
					+ kestore_key + ".html";

			result += "<a href=\"" + link_target + "\">" + eval_rls_.getID()
					+ "</a>";

			result += "			      <br></br>\n";
			result += "			      </td>\n";
			result += "			    </tr>\n";
		}
		result += "			  </tbody>\n";
		result += "			</table></div>\n";

		return result;
	}

}
