/*
 * @author Walther Neuper, member of the ISAC-team, Copyright (c) 2005 by
 * Walther Neuper created Jun 24, 2005 11:00:52 AM Institute for
 * Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.util.formulae;

import java.io.Serializable;

/**
 * container for meth-engines response to tryMatch and tryRefine
 * 
 * @author Walther Neuper, JL
 */
public class Match implements Serializable {

    static final long serialVersionUID = -7147348031620160985L;

	/**
     * pointer to the problem or method, the model of which has been matched
     */
    HierarchyKey kestore_id_;

    /**
     * was the match correct ?
     * 
     * @see isac.util.formulae.CalcHead#status_string_ where the value is copied
     *      from: is either "correct" or "incorrect"
     */
    String status_;

    /**
     * Constants for the status_ member.
     * TODO ALL: change all constant strings to this static constants 
     */
    public static String CORRECT = "correct";
    public static String INCORRECT = "incorrect";

    /**
     * the headline
     * 
     * @see isac.util.formulae.CalcHead#headline_ and
     *      isac.util.genhtml.templates.pbl_node_content.xsl and
     *      isac.util.genhtml.templates.met_node_content.xsl
     */
    Formula head_line_;

    /**
     * the fields Given, Where, Find, Relate
     * 
     * @see isac.util.formulae.CalcHead#model_ where the value is copied from
     */
    Model model_;

    /**
     * the container is always filled from a CalcHead; it is read only.
     */
    public Match(HierarchyKey kestore_id, String status, Formula head_line,
            Model model) {
        kestore_id_ = kestore_id;
        status_ = status;
        head_line_ = head_line;
        model_ = model;
    }

    public HierarchyKey getKEStoreID() {
        return kestore_id_;
    }

    public String getStatus() {
        return status_;
    }

    public Formula getHeadline() {
        return head_line_;
    }

    public Model getModel() {
        return model_;
    }

}