/*
 * @author Richard Gradischnegg
 * Created on Oct 27, 2003
 */

package isac.util.formulae;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Vector;

/**
 * Position of a passive calculation element (Formula or CalcHead)
 * 
 * @author Richard Gradischnegg
 */

public class Position implements Serializable, Comparable {

    static final long serialVersionUID = -1380318634655939503L;

	private Vector<Integer> int_list_;

    private String kind_;

    public Position() {
        int_list_ = new Vector<Integer>();
        kind_ = "Pbl";
    }

    public String toString() {
        return this.toSMLString();
    }

    public void addInt(String intStr) {
        Integer i = Integer.valueOf(intStr);
        int_list_.add(i);
    }

    public void setKind(String string) {
        kind_ = string;
    }

    public String getKind() {
        return kind_;
    }

    public Vector getIntList() {
        return int_list_;
    }

    public String toSMLString() {
        StringBuffer sb = new StringBuffer();
        Iterator it = int_list_.iterator();
        sb.append("([");
        while (it.hasNext()) {
            sb.append(it.next());
            if (it.hasNext()) sb.append(",");
        }
        sb.append("],");
        sb.append(kind_);
        sb.append(")");
        return sb.toString();
    }

    // WN040909 equivalence classes: {Res} | {Frm, Pbl, Met, Und}
    public static int compareKind(String k1, String k2) {
        if (k1.equals("Res")) {
            if (k2.equals("Res"))
                return 0;
            else
                return 1;
        } else {
            if (k2.equals("Res"))
                return -1;
            else
                return 0;
        }
    }

    public boolean equals(Position other) {
        if (this.int_list_.size() != other.int_list_.size()) return false;
        Iterator it1 = this.int_list_.iterator();
        Iterator it2 = other.int_list_.iterator();
        while (it1.hasNext()) {
            if (!it1.next().equals(it2.next())) return false;
        }
        // WN040909 equivalence classes: {Res} | {Frm, Pbl, Met, Und}
        if (0 == compareKind(this.kind_, other.kind_))
            return true;
        else
            return false;
    }

    /*
     * WN040909 surprising complicated order for isac's tree-structure as impl.
     * in SML
     * 
     * @see java.lang.Comparable#compareTo(java.lang.Object) Position is used as
     *      key in ...
     * @see CalcStructure + see 'fun posless' in SML
     */
    public int compareTo(Object other) {
        if (this.int_list_.size() == 0) {//([],*)..
            if (((Position) other).int_list_.size() == 0) {
                return compareKind(this.kind_, ((Position) other).kind_);//([],*)..([],*)
            } else {
                if (this.kind_.equals("Res"))
                    return 1;//([],Res) > *
                else
                    return -1;//([],Frm|Pbl|Met) < *
            }
        } else {
            if (((Position) other).int_list_.size() == 0) {
                if (((Position) other).kind_.equals("Res"))
                    return -1;// * < ([],Res)
                else
                    return 1;// * > ([],Frm|Pbl|Met)
            } else {// scan to unequal listElements OR to end at min.
                int l1 = this.int_list_.size();
                int l2 = ((Position) other).int_list_.size();
                Iterator itr1 = this.getIntList().iterator();
                Iterator itr2 = ((Position) other).getIntList().iterator();
                Iterator min = itr1;//min driven by itr1
                if (l1 > l2) min = itr2;//min driven by itr2
                Integer i1 = (Integer) itr1.next();
                Integer i2 = (Integer) itr2.next();
                while (i1.equals(i2) && min.hasNext()) {//min driven by itr1
                    // EXOR itr2
                    i1 = (Integer) itr1.next();
                    i2 = (Integer) itr2.next();
                }
                //logger_.debug("-----------------------------");
                //logger_.debug("i1= " + i1 + ", i2= " + i2);
                if (i1.equals(i2)) {
                    if (l1 == l2)
                        return compareKind(this.kind_, ((Position) other).kind_);//([1,2],*)..([1,2],*)
                    else {
                        if (l1 < l2) {
                            if (this.kind_.equals("Res"))
                                return 1;//([1,2],Res) > ([1,2,*],*)
                            else
                                return -1;//([1,2],Frm|Pbl|Met) < ([1,2,*],*)
                        } else {
                            if (((Position) other).kind_.equals("Res"))
                                return -1;//([1,2,*],*) < ([1,2],Res)
                            else
                                return 1;//([1,2,*],*) > ([1,2],Frm|Pbl|Met)
                        }
                    }
                } else
                    return i1.compareTo(i2);//([#,m],*)<([#,n],*) iff m<n
            }
        }
    }

    //WN040909
    public boolean less(Position p) {
        if (this.compareTo(p) < 0)
            return true;
        else
            return false;
    }
    
    /**
     * The current level of nesting into subcalculations.
     * 
     * @return An int representing level of nesting (higher == more detailed)
     *         <p>
     *         Special values:
     *         <p>
     *         0: outmost level: only first and last step of calculation are on
     *         this level
     */
    public int getLevel() {
        return int_list_.size();
    }
    

    public Object clone() {
    	Position p = new Position();
    	p.setKind(getKind());
    	Iterator intlistit = this.int_list_.iterator();
    	while(intlistit.hasNext())
    		p.addInt(intlistit.next().toString());
    	
    	return p;
    }

    public boolean onEndOfCalculation() {
        if (getLevel() == 0 && getKind().equals("Res"))
            return true;
        else
            return false;
    }

}