/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.util.parser;

import isac.gui.util.ExampleNode;
import isac.gui.util.Hierarchy;
import isac.util.formulae.KEStoreKey;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

/**
 * Parses the XML representation of the example hierarchy_ and converts it to a
 * object representation. Each NODE Tag is converted to a
 * <code>ExampleNode</code> object. The example nodes are managed by the
 * <code>Hierarchy</code>.
 * 
 * @author Mario Hochreiter
 * @version 1.0
 */
public class XMLExampleParser extends XMLHierarchyParser {

    protected static final String TYPE_EXAMPLE_ = "EXAMPLE";

    public XMLExampleParser(String xml) {
        super(xml);
        System.out.println("XMLExampleParser::DON'T USE ME!!!");
        System.exit(-1);
    }

    private void createExampleNodesHierarchy(Node node, ExampleNode root,
            String txtType) {
        ExampleNode exNode = null;
        String elemName = null;
        // is there anything to do?
        if (node == null) { return; }

        int type = node.getNodeType();
        switch (type) {
        case Node.DOCUMENT_NODE:
            {
                Document document = (Document) node;
                createNodesHierarchy(document.getDocumentElement());
                break;
            }

        case Node.ELEMENT_NODE:
            {
                String nodeName = node.getNodeName().trim();
                if (nodeName.equals(TAG_NODE_)) {
                    exNode = new ExampleNode();
                    hierarchy_.addNode(root, exNode);
                } else if (nodeName.equals(TYPE_ID_)) {
                    elemName = TYPE_ID_;
                } else if (nodeName.equals(TYPE_CONTENTREF_)) {
                    elemName = TYPE_CONTENTREF_;
                } else if (nodeName.equals(TYPE_EXAMPLE_)) {
                    elemName = TYPE_EXAMPLE_;
                }

                // drop through to entity reference
            }

        case Node.ENTITY_REFERENCE_NODE:
            {
                Node child = node.getFirstChild();
                while (child != null) {
                    // elemName is set -> next node is of type text
                    if (elemName != null)
                            createExampleNodesHierarchy(child, root, elemName);
                    createExampleNodesHierarchy(child, exNode, txtType);
                    child = child.getNextSibling();
                }
                break;

            }

        case Node.CDATA_SECTION_NODE:
            {
                break;
            }

        case Node.TEXT_NODE:
            {
                // unknown elemType or whitespace is ignored
                if (txtType == null) return;

                Text text = (Text) node;
                String txt = text.getNodeValue().trim();

                if (txtType.equals(TYPE_ID_)) {
                    root.setId(txt);
                } else if (txtType.equals(TYPE_CONTENTREF_)) {
                    root.setKEStoreKey( new KEStoreKey(txt));
                } else if (txtType.equals(TYPE_EXAMPLE_)) {
                    root.setExample(txt);
                }
                break;
            }
        }
    }

    public Hierarchy createNodesHierarchy(Node node) {
        this.createExampleNodesHierarchy(node, null, null);
        return hierarchy_;
    }
}