/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.util.parser;

import isac.gui.util.Hierarchy;
import isac.gui.util.HierarchyNode;
import isac.util.formulae.KEStoreKey;

import java.io.StringReader;

import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.xml.sax.InputSource;

/**
 * Abstract base class for all XMLHierarchy parser (like ExampleParser, ...).
 * 
 * @author Mario Hochreiter, JL
 * @version 1.0
 *  
 */
public class XMLHierarchyParser {

    protected static final String TAG_NODE_ = "NODE";

    // the different tags a node tag can have
    protected static final String TYPE_ID_ = "ID";

    protected static final String TYPE_CONTENTREF_ = "CONTENTREF";
    
    protected static final String TYPE_EXAMPLE_ = "EXAMPLE";
    
    protected static final String TYPE_NO_ = "NO";

    /** XML in String representation */
    protected String xml_;

    /** XML as document object */
    protected Document document_;

    /** Hierarchy that manages the different nodes (example, problem, method) */
    protected Hierarchy hierarchy_;

    public XMLHierarchyParser(String xmlString) {
        this.xml_ = xmlString;
        this.document_ = this.parseXMLStringDOM();
        this.hierarchy_ = new Hierarchy();
    }

    private Document parseXMLStringDOM() {
        InputSource is = new InputSource(new StringReader(xml_));
        DOMParser parser = new DOMParser();
        try {
            parser.parse(is);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return parser.getDocument();
    }

    public Node getRoot() {
        return document_.getDocumentElement();
    }
    
    public Hierarchy createNodesHierarchy(Node node) {
        this.createNodesHierarchy(node, null, null);
        return hierarchy_;
	}
    
    private void createNodesHierarchy(Node node, HierarchyNode root,
            String txtType) {
        HierarchyNode hierarchy_node = null;
        String elemName = null;
        // is there anything to do?
        if (node == null) { return; }

        int type = node.getNodeType();
        switch (type) {
        case Node.DOCUMENT_NODE:
            {
                Document document = (Document) node;
                createNodesHierarchy(document.getDocumentElement());
                break;
            }

        case Node.ELEMENT_NODE:
            {
                String nodeName = node.getNodeName().trim();
                if (nodeName.equals(TAG_NODE_)) {
                    hierarchy_node = new HierarchyNode();
                    hierarchy_.addNode(root, hierarchy_node);
                } else if (nodeName.equals(TYPE_ID_)) {
                    elemName = TYPE_ID_;
                } else if (nodeName.equals(TYPE_CONTENTREF_)) {
                    elemName = TYPE_CONTENTREF_;
                } else if (nodeName.equals(TYPE_EXAMPLE_)) {
                    elemName = TYPE_EXAMPLE_;
                } else if (nodeName.equals(TYPE_NO_)) {
                    elemName = TYPE_NO_;
                }

                // drop through to entity reference
            }

        case Node.ENTITY_REFERENCE_NODE:
            {
                Node child = node.getFirstChild();
                while (child != null) {
                    // elemName is set -> next node is of type text
                    if (elemName != null)
                            createNodesHierarchy(child, root, elemName);
                    createNodesHierarchy(child, hierarchy_node, txtType);
                    child = child.getNextSibling();
                }
                break;

            }

        case Node.CDATA_SECTION_NODE:
            {
                break;
            }

        case Node.TEXT_NODE:
            {
                // unknown elemType or whitespace is ignored
                if (txtType == null) return;

                Text text = (Text) node;
                String txt = text.getNodeValue().trim();

                if (txtType.equals(TYPE_ID_)) {
                    root.setId(txt);
                } else if (txtType.equals(TYPE_CONTENTREF_)) {
                    root.setKEStoreKey( new KEStoreKey( txt ) );
                }
                break;
            }
        }
    }
}