/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.util.parser;

import isac.gui.util.Hierarchy;
import isac.gui.util.MethodNode;
import isac.util.formulae.KEStoreKey;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

/**
 * Parses the XML representation of the method hierarchy_ and converts it to a
 * object representation. Each NODE Tag is converted to a
 * <code>MethodNode</code> object. The method nodes are managed by the
 * <code>Hierarchy</code>.
 * 
 * @author Mario Hochreiter, JL
 * @version 1.0
 *  
 */
public class XMLMethodParser extends XMLHierarchyParser {

    protected static final String TYPE_NO_ = "NO";

    public XMLMethodParser(String xml) {
        super(xml);
        System.out.println("XMLMethodParser::DON'T USE ME!!!");
        System.exit(-1);
    }

    private void createMethodNodesHierarchy(Node node, MethodNode root,
            String txtType) {
        MethodNode methodNode = null;
        String elemName = null;
        // is there anything to do?
        if (node == null) { return; }

        int type = node.getNodeType();
        switch (type) {
        case Node.DOCUMENT_NODE:
            {
                Document document = (Document) node;
                createNodesHierarchy(document.getDocumentElement());
                break;
            }

        case Node.ELEMENT_NODE:
            {
                String nodeName = node.getNodeName().trim();
                if (nodeName.equals(TAG_NODE_)) {
                    methodNode = new MethodNode();
                    hierarchy_.addNode(root, methodNode);
                } else if (nodeName.equals(TYPE_ID_)) {
                    elemName = TYPE_ID_;
                } else if (nodeName.equals(TYPE_CONTENTREF_)) {
                    elemName = TYPE_CONTENTREF_;
                } else if (nodeName.equals(TYPE_NO_)) {
                    elemName = TYPE_NO_;
                }
                // drop through to entity reference
            }

        case Node.ENTITY_REFERENCE_NODE:
            {
                Node child = node.getFirstChild();
                while (child != null) {
                    // elemName is set -> next node is of type text
                    if (elemName != null)
                            createMethodNodesHierarchy(child, root, elemName);
                    createMethodNodesHierarchy(child, methodNode, txtType);
                    child = child.getNextSibling();
                }
                break;

            }

        case Node.CDATA_SECTION_NODE:
            {
                break;
            }

        case Node.TEXT_NODE:
            {
                // unknown elemType or whitespace is ignored
                if (txtType == null) return;

                Text text = (Text) node;
                String txt = text.getNodeValue().trim();

                if (txtType.equals(TYPE_ID_)) {
                    root.setId(txt);
                } else if (txtType.equals(TYPE_CONTENTREF_)) {
                    root.setKEStoreKey( new KEStoreKey( txt ) );
                } else if (txtType.endsWith(TYPE_NO_)) {
                    root.setNo(txt);
                }
                break;
            }
        }
    }

    public Hierarchy createNodesHierarchy(Node node) {
        this.createMethodNodesHierarchy(node, null, null);
        return hierarchy_;
    }
}