/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2005 by Walther Neuper
 * created Aug 8, 2005 3:41:05 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.wsdialog;

import java.rmi.RemoteException;

import isac.util.formulae.Match;
import isac.util.formulae.MethodID;
import isac.util.formulae.ProblemID;

/**
 * a consciously sketchy interface between the two components of the dialog
 * guide, the XBrowserDialog and the WorksheetDialog.
 * 
 * @author Walther Neuper
 */
public interface IBrowserDialogToWorksheetDialog {

    /**
     * Match the problem with the active CalcHead on the Worksheet.
     * FIXXXME.WN051125 THE WORKSHEETDIALOG DOES NOT KNOW ABOUT AN OPEN
     * CALCHEADPANEL -- SHOULD HE ???... If on the Worksheet there is no
     * CalcHead active, return null. ...??? Fixes the WorksheetDialog until the
     * MathEngine returns the Match.
     * 
     * @return Match calculated by the MathEngine or null if on the Worksheet
     *         there is no CalcHead active
     * 
     * @see isac.interfaces.ICalcIterator#initMatchProblem()
     */
    public Match initMatchProblem();

    /**
     * Match the problem with the active CalcHead on the Worksheet.
     * FIXXXME.WN051125 THE WORKSHEETDIALOG DOES NOT KNOW ABOUT AN OPEN
     * CALCHEADPANEL -- SHOULD HE ???... If on the Worksheet there is no
     * CalcHead active, return null. ...??? Fixes the WorksheetDialog until the
     * MathEngine returns the Match.
     * 
     * @param id
     *            referencing the problem in the KEStore, which is mirrored in
     *            an SML structure.
     * @return Match calculated by the MathEngine or null if on the Worksheet
     *         there is no CalcHead active
     */
    public Match tryMatchProblem(ProblemID id);

    /**
     * Starts the refinement method of the match engine and returns a match
     * 
     * @param id
     * @return Match calculated by MatchEngine or null
     */
    public Match tryRefineProblem(ProblemID id);

    /**
     * Match the guide of the method with the active CalcHead on the Worksheet.
     * If on the Worksheet there is no CalcHead active, return null. Fixes the
     * WorksheetDialog until the MathEngine returns the Match.
     * 
     * @param id
     *            referencing the guide of the method in the KEStore, which is
     *            mirrored in an SML structure.
     * @return Match calculated by the MathEngine or null if on the Worksheet
     *         there is no CalcHead active
     */
    public Match tryMatchMethod(MethodID id);

    /**
     * Transfer the Problem selected on the ProblemBrowser (after hitting
     * <To Worksheet>) to the WorkSheet by fetching the CalcHead from the
     * MathEngine (which sets the 'ModelItem status' for feedback)
     * 
     * @param problem_id
     * @throws RemoteException
     */
	public void doProblemToWorksheet(ProblemID problem_id) throws RemoteException;
}