/*
 * @author Robert Koenighofer, member of the ISAC-team, 
 * Copyright (c) 2006 by Robert Koenighofer
 * created Jul 11, 2006 10:09:13 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */

package isac.wsdialog;

import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * This is an interface for the WSDialogManger. It is used by the window
 * application to perform actions with the WorksheetDialoges
 * 
 * @see isac.gui.WindowApplication
 * @see isac.wsdialog.WSDialogManager
 * @author Robert Koenighofer
 */
public interface IWSDialogManager extends Remote{
	
    /**
     * This method creates a new WorksheetDialog and returns
     * the ID of the created Object. The new Dialog is added into
     * the map and is set as active Worksheet dialog.
     * 
     * @return the ID of the created Worksheet dialog.
     * @throws RemoteException if remote object is not available
     * @see isac.wsdialog.WSDialogManager#openWSDialog()
     */
	 public String openWSDialog() throws RemoteException;
	 
    /**
	 * This method returns an interface to the WorksheetDialog with the desired
	 * ID. If the ID is unknown to the WSDialogManager, null is returned.
	 * 
	 * @return an interface to the WorksheetDialog with the desired ID or null
	 *         if it does not exist.
	 * @throws RemoteException
	 *             if remote object is not available
	 * @see isac.wsdialog.WSDialogManager#getWSDialog(String)
	 */
	 public IWorksheetDialog getWSDialog(String wsd_id) throws RemoteException;
	 
    /**
     * This method closes the WorksheetDialog with the specified ID. If
     * the ID is unknown to the WSDialogManager nothing will happen.
     * If the worksheet to close is the active worksheet, the worksheet
     * with the highest ID will become the new active worksheet. If there
     * is no worksheet left, the active worksheet will be set to null.
     * 
     * @param wsd_id the ID of the worksheet to close 
     * @throws RemoteException if remote object is not available
     * @see isac.wsdialog.WSDialogManager#closeWSDialog(String)
     */	 
	 public void closeWSDialog(String wsd_id) throws RemoteException;
	 
    /**
     * This method returns the active WorksheetDialog. If there
     * is no active WorksheetDialog (e.g. if there is no WorksheetDialog
     * at all), null is returned.
     * 
     * @return The active WorksheetDialog or null if there is non.
     * @throws RemoteException if remote object is not available
     * @see isac.wsdialog.WSDialogManager#numberOpenedWSDialoges()
     */	 
	 public int numberOpenedWSDialoges() throws RemoteException;
	 
    /**
     * This method returns the an interface to the active WorksheetDialog. 
     * If there is no active WorksheetDialog (e.g. if there is no 
     * WorksheetDialog at all), null is returned.
     * 
     * @return The active WorksheetDialog or null if there is non.
     * @throws RemoteException if remote object is not available
     * @see isac.wsdialog.WSDialogManager#getActiveWSDialog()
     */
    public IWorksheetDialog getActiveWSDialog() throws RemoteException;
}
