(*.calculate in rationals: gcd, lcm, etc.
   (c) Stefan Karnel 2002
   Institute for Mathematics D and Institute for Software Technology, 
   TU-Graz SS 2002 
   Use is subject to license terms.

use"IsacKnowledge/Rational.ML";
use"Rational.ML";

remove_thy"Rational";
use_thy"IsacKnowledge/Isac";
****************************************************************.*)

(*.*****************************************************************
  Remark on notions in the documentation below:
    referring to the remark on 'polynomials' in Poly.sml we use
    [2] 'polynomial' normalform (Polynom)
    [3] 'expanded_term' normalform (Ausmultiplizierter Term),
    where normalform [2] is a special case of [3], i.e. [3] implies [2].
    Instead of 
      'fraction with numerator and nominator both in normalform [2]'
      'fraction with numerator and nominator both in normalform [3]' 
    we say: 
      'fraction in normalform [2]'
      'fraction in normalform [3]' 
    or
      'fraction [2]'
      'fraction [3]'.
    a 'simple fraction' is a term with '/' as outmost operator and
    numerator and nominator in normalform [2] or [3].
****************************************************************.*)

signature RATIONALI =
sig
  type mv_monom
  type mv_poly 
  val add_fraction_ : theory -> term -> (term * term list) option      
  val add_fraction_p_ : theory -> term -> (term * term list) option       
  val calculate_Rational : rls
  val calc_rat_erls:rls
  val cancel : rls
  val cancel_ : theory -> term -> (term * term list) option    
  val cancel_p : rls   
  val cancel_p_ : theory -> term -> (term * term list) option
  val common_nominator : rls              
  val common_nominator_ : theory -> term -> (term * term list) option
  val common_nominator_p : rls              
  val common_nominator_p_ : theory -> term -> (term * term list) option
  val eval_is_expanded : string -> 'a -> term -> theory -> 
			 (string * term) option                    
  val expanded2polynomial : term -> term option
  val factout_ : theory -> term -> (term * term list) option
  val factout_p_ : theory -> term -> (term * term list) option
  val is_expanded : term -> bool
  val is_polynomial : term -> bool

  val mv_gcd : (int * int list) list -> mv_poly -> mv_poly
  val mv_lcm : mv_poly -> mv_poly -> mv_poly

  val norm_expanded_rat_ : theory -> term -> (term * term list) option
(*WN0602.2.6.pull into struct !!!
  val norm_Rational : rls(*.normalizes an arbitrary rational term without
                           roots into a simple and canceled fraction
                           with normalform [2].*)
*)
(*val norm_rational_p : 19.10.02 missing FIXXXXXXXXXXXXME
      rls               (*.normalizes an rational term [2] without
                           roots into a simple and canceled fraction
                           with normalform [2].*)
*)
  val norm_rational_ : theory -> term -> (term * term list) option
  val polynomial2expanded : term -> term option
  val rational_erls : 
      rls             (*.evaluates an arbitrary rational term with numerals.*)

(*WN0210???SK: fehlen Funktionen, die exportiert werden sollen ? *)
end

(*.**************************************************************************
survey on the functions
~~~~~~~~~~~~~~~~~~~~~~~
 [2] 'polynomial'   :rls               | [3]'expanded_term':rls
--------------------:------------------+-------------------:-----------------
 factout_p_         :                  | factout_          :
 cancel_p_          :                  | cancel_           :
                    :cancel_p          |                   :cancel
--------------------:------------------+-------------------:-----------------
 common_nominator_p_:                  | common_nominator_ :
                    :common_nominator_p|                   :common_nominator
 add_fraction_p_    :                  | add_fraction_     :
--------------------:------------------+-------------------:-----------------
???SK                 :norm_rational_p   |                   :norm_rational

This survey shows only the principal functions for reuse, and the identifiers 
of the rls exported. The list below shows some more useful functions.


conversion from Isabelle-term to internal representation
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

... BITTE FORTSETZEN ...

polynomial2expanded = ...
expanded2polynomial = ...

remark: polynomial2expanded o expanded2polynomial = I, 
        where 'o' is function chaining, and 'I' is identity WN0210???SK

functions for greatest common divisor and canceling
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
mv_gcd
factout_
factout_p_
cancel_
cancel_p_

functions for least common multiple and addition of fractions
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
mv_lcm
common_nominator_
common_nominator_p_
add_fraction_       (*.add 2 or more fractions.*)
add_fraction_p_     (*.add 2 or more fractions.*)

functions for normalform of rationals
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
WN0210???SK interne Funktionen f"ur norm_rational: 
          schaffen diese SML-Funktionen wirklich ganz allgemeine Terme

norm_rational_
norm_expanded_rat_

**************************************************************************.*)


(*##*)
structure RationalI : RATIONALI = 
struct 
(*##*)

(*. gcd of integers .*)
(* die gcd Funktion von Isabelle funktioniert nicht richtig !!! *)
fun gcd_int a b = if b=0 then a
		  else  gcd_int b (a mod b);

(*. univariate polynomials (uv) .*)
(*. univariate polynomials are represented as a list of the coefficent in reverse maximum degree order .*)
(*. 5 * x^5 + 4 * x^3 + 2 * x^2 + x + 19 => [19,1,2,4,0,5] .*)
type uv_poly = int list;

(*. adds two uv polynomials .*)
fun uv_mod_add_poly ([]:uv_poly,p2:uv_poly) = p2:uv_poly 
  | uv_mod_add_poly (p1,[]) = p1
  | uv_mod_add_poly (x::p1,y::p2) = (x+y)::(uv_mod_add_poly(p1,p2)); 

(*. multiplies a uv polynomial with a skalar s .*)
fun uv_mod_smul_poly ([]:uv_poly,s:int) = []:uv_poly 
  | uv_mod_smul_poly (x::p,s) = (x*s)::(uv_mod_smul_poly(p,s)); 

(*. calculates the remainder of a polynomial divided by a skalar s .*)
fun uv_mod_rem_poly ([]:uv_poly,s) = []:uv_poly 
  | uv_mod_rem_poly (x::p,s) = (x mod s)::(uv_mod_smul_poly(p,s)); 

(*. calculates the degree of a uv polynomial .*)
fun uv_mod_deg ([]:uv_poly) = 0  
  | uv_mod_deg p = length(p)-1;

(*. calculates the remainder of x/p and represents it as value between -p/2 and p/2 .*)
fun uv_mod_mod2(x,p)=
    let
	val y=(x mod p);
    in
	if (y)>(p div 2) then (y)-p else 
	    (
	     if (y)<(~p div 2) then p+(y) else (y)
	     )
    end;

(*.calculates the remainder for each element of a integer list divided by p.*)  
fun uv_mod_list_modp [] p = [] 
  | uv_mod_list_modp (x::xs) p = (uv_mod_mod2(x,p))::(uv_mod_list_modp xs p);

(*. appends an integer at the end of a integer list .*)
fun uv_mod_null (p1:int list,0) = p1 
  | uv_mod_null (p1:int list,n1:int) = uv_mod_null(p1,n1-1) @ [0];

(*. uv polynomial division, result is (quotient, remainder) .*)
(*. only for uv_mod_divides .*)
(* FIXME: Division von x^9+x^5+1 durch x-1000 funktioniert nicht integer zu klein  *)
fun uv_mod_pdiv (p1:uv_poly) ([]:uv_poly) = raise error ("RATIONALS_UV_MOD_PDIV_EXCEPTION: division by zero")
  | uv_mod_pdiv p1 [x] = 
    let
	val xs=ref [];
    in
	if x<>0 then 
	    (
	     xs:=(uv_mod_rem_poly(p1,x));
	     while length(!xs)>0 andalso hd(!xs)=0 do xs:=tl(!xs)
	     )
	else raise error ("RATIONALS_UV_MOD_PDIV_EXCEPTION: division by zero");
	([]:uv_poly,!xs:uv_poly)
    end
  | uv_mod_pdiv p1 p2 =  
    let
	val n= uv_mod_deg(p2);
	val m= ref (uv_mod_deg(p1));
	val p1'=ref (rev(p1));
	val p2'=(rev(p2));
	val lc2=hd(p2');
	val q=ref [];
	val c=ref 0;
	val output=ref ([],[]);
    in
	(
	 if (!m)=0 orelse p2=[0] then raise error ("RATIONALS_UV_MOD_PDIV_EXCEPTION: Division by zero") 
	 else
	     (
	      if (!m)<n then 
		  (
		   output:=([0],p1) 
		   ) 
	      else
		  (
		   while (!m)>=n do
		       (
			c:=hd(!p1') div hd(p2');
			if !c<>0 then
			    (
			     p1':=uv_mod_add_poly(!p1',uv_mod_null(uv_mod_smul_poly(p2',~(!c)),!m-n));
			     while length(!p1')>0 andalso hd(!p1')=0  do p1':= tl(!p1');
			     m:=uv_mod_deg(!p1')
			     )
			else m:=0
			);
    		   output:=(rev(!q),rev(!p1'))
		   )
	      );
	     !output
	 )
    end;

(*. divides p1 by p2 in Zp .*)
fun uv_mod_pdivp (p1:uv_poly) (p2:uv_poly) p =  
    let
	val n=uv_mod_deg(p2);
	val m=ref (uv_mod_deg(uv_mod_list_modp p1 p));
	val p1'=ref (rev(p1));
	val p2'=(rev(uv_mod_list_modp p2 p));
	val lc2=hd(p2');
	val q=ref [];
	val c=ref 0;
	val output=ref ([],[]);
    in
	(
	 if (!m)=0 orelse p2=[0] then raise error ("RATIONALS_UV_MOD_PDIVP_EXCEPTION: Division by zero") 
	 else
	     (
	      if (!m)<n then 
		  (
		   output:=([0],p1) 
		   ) 
	      else
		  (
		   while (!m)>=n do
		       (
			c:=uv_mod_mod2(hd(!p1')*(power lc2 1), p);
			q:=(!c)::(!q);
			p1':=uv_mod_list_modp(tl(uv_mod_add_poly(uv_mod_smul_poly(!p1',lc2),
								  uv_mod_smul_poly(uv_mod_smul_poly(p2',hd(!p1')),~1)))) p;
			m:=(!m)-1
			);
		   
		   while !p1'<>[] andalso hd(!p1')=0 do
		       (
			p1':=tl(!p1')
			);

    		   output:=(rev(uv_mod_list_modp (!q) (p)),rev(!p1'))
		   )
	      );
	     !output:uv_poly * uv_poly
	 )
    end;

(*. calculates the remainder of p1/p2 .*)
fun uv_mod_prest (p1:uv_poly) ([]:uv_poly) = raise error("UV_MOD_PREST_EXCEPTION: Division by zero") 
  | uv_mod_prest [] p2 = []:uv_poly
  | uv_mod_prest p1 p2 = (#2(uv_mod_pdiv p1 p2));

(*. calculates the remainder of p1/p2 in Zp .*)
fun uv_mod_prestp (p1:uv_poly) ([]:uv_poly) p= raise error("UV_MOD_PRESTP_EXCEPTION: Division by zero") 
  | uv_mod_prestp [] p2 p= []:uv_poly 
  | uv_mod_prestp p1 p2 p = #2(uv_mod_pdivp p1 p2 p); 

(*. calculates the content of a uv polynomial .*)
fun uv_mod_cont ([]:uv_poly) = 0  
  | uv_mod_cont (x::p)= gcd_int x (uv_mod_cont(p));

(*. divides each coefficient of a uv polynomial by y .*)
fun uv_mod_div_list (p:uv_poly,0) = raise error("UV_MOD_DIV_LIST_EXCEPTION: Division by zero") 
  | uv_mod_div_list ([],y)   = []:uv_poly
  | uv_mod_div_list (x::p,y) = (x div y)::uv_mod_div_list(p,y); 

(*. calculates the primitiv part of a uv polynomial .*)
fun uv_mod_pp ([]:uv_poly) = []:uv_poly
  | uv_mod_pp p =  
    let
	val c=ref 0;
    in
	(
	 c:=uv_mod_cont(p);
	 
	 if !c=0 then raise error ("RATIONALS_UV_MOD_PP_EXCEPTION: content is 0")
	 else uv_mod_div_list(p,!c)
	)
    end;

(*. gets the leading coefficient of a uv polynomial .*)
fun uv_mod_lc ([]:uv_poly) = 0 
  | uv_mod_lc p  = hd(rev(p)); 

(*. calculates the euklidean polynomial remainder sequence in Zp .*)
fun uv_mod_prs_euklid_p(p1:uv_poly,p2:uv_poly,p)= 
    let
	val f =ref [];
	val f'=ref p2;
	val fi=ref [];
    in
	( 
	 f:=p2::p1::[]; 
 	 while uv_mod_deg(!f')>0 do
	     (
	      f':=uv_mod_prestp (hd(tl(!f))) (hd(!f)) p;
	      if (!f')<>[] then 
		  (
		   fi:=(!f');
		   f:=(!fi)::(!f)
		   )
	      else ()
	      );
	      (!f)
	 
	 )
    end;

(*. calculates the gcd of p1 and p2 in Zp .*)
fun uv_mod_gcd_modp ([]:uv_poly) (p2:uv_poly) p = p2:uv_poly 
  | uv_mod_gcd_modp p1 [] p= p1
  | uv_mod_gcd_modp p1 p2 p=
    let
	val p1'=ref[];
	val p2'=ref[];
	val pc=ref[];
	val g=ref [];
	val d=ref 0;
	val prs=ref [];
    in
	(
	 if uv_mod_deg(p1)>=uv_mod_deg(p2) then
	     (
	      p1':=uv_mod_list_modp (uv_mod_pp(p1)) p;
	      p2':=uv_mod_list_modp (uv_mod_pp(p2)) p
	      )
	 else 
	     (
	      p1':=uv_mod_list_modp (uv_mod_pp(p2)) p;
	      p2':=uv_mod_list_modp (uv_mod_pp(p1)) p
	      );
	 d:=uv_mod_mod2((gcd_int (uv_mod_cont(p1))) (uv_mod_cont(p2)), p) ;
	 if !d>(p div 2) then d:=(!d)-p else ();
	 
	 prs:=uv_mod_prs_euklid_p(!p1',!p2',p);

	 if hd(!prs)=[] then pc:=hd(tl(!prs))
	 else pc:=hd(!prs);

	 g:=uv_mod_smul_poly(uv_mod_pp(!pc),!d);
	 !g
	 )
    end;

(*. calculates the minimum of two real values x and y .*)
fun uv_mod_r_min(x,y):BasisLibrary.Real.real = if x>y then y else x;

(*. calculates the minimum of two integer values x and y .*)
fun uv_mod_min(x,y) = if x>y then y else x;

(*. adds the squared values of a integer list .*)
fun uv_mod_add_qu [] = 0.0 
  | uv_mod_add_qu (x::p) =  BasisLibrary.Real.fromInt(x)*BasisLibrary.Real.fromInt(x) + uv_mod_add_qu p;

(*. calculates the euklidean norm .*)
fun uv_mod_norm ([]:uv_poly) = 0.0
  | uv_mod_norm p = Math.sqrt(uv_mod_add_qu(p));

(*. multipies two values a and b .*)
fun uv_mod_multi a b = a * b;

(*. decides if x is a prim, the list contains all primes which are lower then x .*)
fun uv_mod_prim(x,[])= false 
  | uv_mod_prim(x,[y])=if ((x mod y) <> 0) then true
		else false
  | uv_mod_prim(x,y::ys) = if uv_mod_prim(x,[y])
			then 
			    if uv_mod_prim(x,ys) then true 
			    else false
		    else false;

(*. gets the first prime, which is greater than p and does not divide g .*)
fun uv_mod_nextprime(g,p)= 
    let
	val list=ref [2];
	val exit=ref 0;
	val i = ref 2
    in
	while (!i<p) do (* calculates the primes lower then p *)
	    (
	     if uv_mod_prim(!i,!list) then
		 (
		  if (p mod !i <> 0)
		      then
			  (
			   list:= (!i)::(!list);
			   i:= (!i)+1
			   )
		  else i:=(!i)+1
		  )
	     else i:= (!i)+1
		 );
	    i:=(p+1);
	    while (!exit=0) do   (* calculate next prime which does not divide g *)
	    (
	     if uv_mod_prim(!i,!list) then
		 (
		  if (g mod !i <> 0)
		      then
			  (
			   list:= (!i)::(!list);
			   exit:= (!i)
			   )
		  else i:=(!i)+1
		      )
	     else i:= (!i)+1
		 ); 
	    !exit
    end;

(*. decides if p1 is a factor of p2 in Zp .*)
fun uv_mod_dividesp ([]:uv_poly) (p2:uv_poly) p= raise error("UV_MOD_DIVIDESP: Division by zero") 
  | uv_mod_dividesp p1 p2 p= if uv_mod_prestp p2 p1 p = [] then true else false;

(*. decides if p1 is a factor of p2 .*)
fun uv_mod_divides ([]:uv_poly) (p2:uv_poly) = raise error("UV_MOD_DIVIDES: Division by zero")
  | uv_mod_divides p1 p2 = if uv_mod_prest p2 p1  = [] then true else false;

(*. chinese remainder algorithm .*)
fun uv_mod_cra2(r1,r2,m1,m2)=     
    let 
	val c=ref 0;
	val r1'=ref 0;
	val d=ref 0;
	val a=ref 0;
    in
	(
	 while (uv_mod_mod2((!c)*m1,m2))<>1 do 
	     (
	      c:=(!c)+1
	      );
	 r1':= uv_mod_mod2(r1,m1);
	 d:=uv_mod_mod2(((r2-(!r1'))*(!c)),m2);
	 !r1'+(!d)*m1    
	 )
    end;

(*. applies the chinese remainder algorithmen to the coefficients of x1 and x2 .*)
fun uv_mod_cra_2 ([],[],m1,m2) = [] 
  | uv_mod_cra_2 ([],x2,m1,m2) = raise error("UV_MOD_CRA_2_EXCEPTION: invalid call x1")
  | uv_mod_cra_2 (x1,[],m1,m2) = raise error("UV_MOD_CRA_2_EXCEPTION: invalid call x2")
  | uv_mod_cra_2 (x1::x1s,x2::x2s,m1,m2) = (uv_mod_cra2(x1,x2,m1,m2))::(uv_mod_cra_2(x1s,x2s,m1,m2));

(*. calculates the gcd of two uv polynomials p1' and p2' with the modular algorithm .*)
fun uv_mod_gcd (p1':uv_poly) (p2':uv_poly) =
    let 
	val p1=ref (uv_mod_pp(p1'));
	val p2=ref (uv_mod_pp(p2'));
	val c=gcd_int (uv_mod_cont(p1')) (uv_mod_cont(p2'));
	val temp=ref [];
	val cp=ref [];
	val qp=ref [];
	val q=ref[];
	val pn=ref 0;
	val d=ref 0;
	val g1=ref 0;
	val p=ref 0;    
	val m=ref 0;
	val exit=ref 0;
	val i=ref 1;
    in
	if length(!p1)>length(!p2) then ()
	else 
	    (
	     temp:= !p1;
	     p1:= !p2;
	     p2:= !temp
	     );

	 
	d:=gcd_int (uv_mod_lc(!p1)) (uv_mod_lc(!p2));
	g1:=uv_mod_lc(!p1)*uv_mod_lc(!p2);
	p:=4;
	
	m:=BasisLibrary.Real.ceil(2.0 *   
				  BasisLibrary.Real.fromInt(!d) *
				  BasisLibrary.Real.fromInt(power 2 (uv_mod_min(uv_mod_deg(!p2),uv_mod_deg(!p1)))) *  
				  BasisLibrary.Real.fromInt(!d) * 
				  uv_mod_r_min(uv_mod_norm(!p1) / BasisLibrary.Real.fromInt(abs(uv_mod_lc(!p1))),
					uv_mod_norm(!p2) / BasisLibrary.Real.fromInt(abs(uv_mod_lc(!p2))))); 

	while (!exit=0) do  
	    (
	     p:=uv_mod_nextprime(!d,!p);
	     cp:=(uv_mod_gcd_modp (uv_mod_list_modp(!p1) (!p)) (uv_mod_list_modp(!p2) (!p)) (!p)) ;
	     if abs(uv_mod_lc(!cp))<>1 then  (* leading coefficient = 1 ? *)
		 (
		  i:=1;
		  while (!i)<(!p) andalso (abs(uv_mod_mod2((uv_mod_lc(!cp)*(!i)),(!p)))<>1) do
		      (
		       i:=(!i)+1
		       );
		      cp:=uv_mod_list_modp (map (uv_mod_multi (!i)) (!cp)) (!p) 
		  )
	     else ();

	     qp:= ((map (uv_mod_multi (uv_mod_mod2(!d,!p)))) (!cp));

	     if uv_mod_deg(!qp)=0 then (q:=[1]; exit:=1) else ();

	     pn:=(!p);
	     q:=(!qp);

	     while !pn<= !m andalso !m>(!p) andalso !exit=0 do
		 (
		  p:=uv_mod_nextprime(!d,!p);
 		  cp:=(uv_mod_gcd_modp (uv_mod_list_modp(!p1) (!p)) (uv_mod_list_modp(!p2) (!p)) (!p)); 
 		  if uv_mod_lc(!cp)<>1 then  (* leading coefficient = 1 ? *)
 		      (
 		       i:=1;
 		       while (!i)<(!p) andalso ((uv_mod_mod2((uv_mod_lc(!q)*(!i)),(!p)))<>1) do
 			   (
 			    i:=(!i)+1
		           );
		       cp:=uv_mod_list_modp (map (uv_mod_multi (!i)) (!cp)) (!p)
 		      )
 		  else ();    
 		 
		  qp:=uv_mod_list_modp ((map (uv_mod_multi (uv_mod_mod2(!d,!p)))) (!cp)  ) (!p);
 		  if uv_mod_deg(!qp)>uv_mod_deg(!q) then
 		      (
 		       (*print("degree to high!!!\n")*)
 		       )
 		  else
 		      (
 		       if uv_mod_deg(!qp)=uv_mod_deg(!q) then
 			   (
 			    q:=uv_mod_cra_2(!q,!qp,!pn,!p);
			    pn:=(!pn) * !p;
			    q:=uv_mod_pp(uv_mod_list_modp (!q) (!pn)); (* found already gcd ? *)
			    if (uv_mod_divides (!q) (p1')) andalso (uv_mod_divides (!q) (p2')) then (exit:=1) else ()
		 	    )
		       else
			   (
			    if  uv_mod_deg(!qp)<uv_mod_deg(!q) then
				(
				 pn:= !p;
				 q:= !qp
				 )
			    else ()
			    )
		       )
		  );
 	     q:=uv_mod_pp(uv_mod_list_modp (!q) (!pn));
	     if (uv_mod_divides (!q) (p1')) andalso (uv_mod_divides (!q) (p2')) then exit:=1 else ()
	     );
	    uv_mod_smul_poly(!q,c):uv_poly
    end;

(*. multivariate polynomials .*)
(*. multivariate polynomials are represented as a list of the pairs, 
 first is the coefficent and the second is a list of the exponents .*)
(*. 5 * x^5 * y^3 + 4 * x^3 * z^2 + 2 * x^2 * y * z^3 - z - 19 
 => [(5,[5,3,0]),(4,[3,0,2]),(2,[2,1,3]),(~1,[0,0,1]),(~19,[0,0,0])] .*)

(*. global variables .*)
(*. order indicators .*)
val LEX_=0; (* lexicographical term order *)
val GGO_=1; (* greatest degree order *)

(*. datatypes for internal representation.*)
type mv_monom = (int *      (*.coefficient or the monom.*)
		 int list); (*.list of exponents)      .*)
fun mv_monom2str (i, is) = "("^ int2str i^"," ^ ints2str' is ^ ")";

type mv_poly = mv_monom list; 
fun mv_poly2str p = (strs2str' o (map mv_monom2str)) p;

(*. help function for monom_greater and geq .*)
fun mv_mg_hlp([]) = EQUAL 
  | mv_mg_hlp(x::list)=if x<0 then LESS
		    else if x>0 then GREATER
			 else mv_mg_hlp(list);

(*. adds a list of values .*)
fun mv_addlist([]) = 0
  | mv_addlist(p1) = hd(p1)+mv_addlist(tl(p1));
			   
(*. tests if the monomial M1 is greater as the monomial M2 and returns a boolean value .*)
(*. 2 orders are implemented LEX_/GGO_ (lexigraphical/greatest degree order) .*)
fun mv_monom_greater((M1x,M1l):mv_monom,(M2x,M2l):mv_monom,order)=
    if order=LEX_ then
	( 
	 if length(M1l)<>length(M2l) then raise error ("RATIONALS_MV_MONOM_GREATER_EXCEPTION: Order error")
	 else if (mv_mg_hlp((map op- (M1l~~M2l)))<>GREATER) then false else true
	     )
    else
	if order=GGO_ then
	    ( 
	     if length(M1l)<>length(M2l) then raise error ("RATIONALS_MV_MONOM_GREATER_EXCEPTION: Order error")
	     else 
		 if mv_addlist(M1l)=mv_addlist(M2l)  then if (mv_mg_hlp((map op- (M1l~~M2l)))<>GREATER) then false else true
		 else if mv_addlist(M1l)>mv_addlist(M2l) then true else false
	     )
	else raise error ("RATIONALS_MV_MONOM_GREATER_EXCEPTION: Wrong Order");
		   
(*. tests if the monomial X is greater as the monomial Y and returns a order value (GREATER,EQUAL,LESS) .*)
(*. 2 orders are implemented LEX_/GGO_ (lexigraphical/greatest degree order) .*)
fun mv_geq order ((x1,x):mv_monom,(x2,y):mv_monom) =
let 
    val temp=ref EQUAL;
in
    if order=LEX_ then
	(
	 if length(x)<>length(y) then 
	     raise error ("RATIONALS_MV_GEQ_EXCEPTION: Order error")
	 else 
	     (
	      temp:=mv_mg_hlp((map op- (x~~y)));
	      if !temp=EQUAL then 
		  ( if x1=x2 then EQUAL 
		    else if x1>x2 then GREATER
			 else LESS
			     )
	      else (!temp)
	      )
	     )
    else 
	if order=GGO_ then 
	    (
	     if length(x)<>length(y) then 
	      raise error ("RATIONALS_MV_GEQ_EXCEPTION: Order error")
	     else 
		 if mv_addlist(x)=mv_addlist(y) then 
		     (mv_mg_hlp((map op- (x~~y))))
		 else if mv_addlist(x)>mv_addlist(y) then GREATER else LESS
		     )
	else raise error ("RATIONALS_MV_GEQ_EXCEPTION: Wrong Order")
end;

(*. cuts the first variable from a polynomial .*)
fun mv_cut([]:mv_poly)=[]:mv_poly
  | mv_cut((x,[])::list) = raise error ("RATIONALS_MV_CUT_EXCEPTION: Invalid list ")
  | mv_cut((x,y::ys)::list)=(x,ys)::mv_cut(list);
	    
(*. leading power product .*)
fun mv_lpp([]:mv_poly,order)  = []
  | mv_lpp([(x,y)],order) = y
  | mv_lpp(p1,order)  = #2(hd(rev(sort (mv_geq order) p1)));
    
(*. leading monomial .*)
fun mv_lm([]:mv_poly,order)  = (0,[]):mv_monom
  | mv_lm([x],order) = x 
  | mv_lm(p1,order)  = hd(rev(sort (mv_geq order) p1));
    
(*. leading coefficient in term order .*)
fun mv_lc2([]:mv_poly,order)  = 0
  | mv_lc2([(x,y)],order) = x
  | mv_lc2(p1,order)  = #1(hd(rev(sort (mv_geq order) p1)));


(*. reverse the coefficients in mv polynomial .*)
fun mv_rev_to([]:mv_poly) = []:mv_poly
  | mv_rev_to((c,e)::xs) = (c,rev(e))::mv_rev_to(xs);

(*. leading coefficient in reverse term order .*)
fun mv_lc([]:mv_poly,order)  = []:mv_poly 
  | mv_lc([(x,y)],order) = mv_rev_to(mv_cut(mv_rev_to([(x,y)])))
  | mv_lc(p1,order)  = 
    let
	val p1o=ref (rev(sort (mv_geq order) (mv_rev_to(p1))));
	val lp=hd(#2(hd(!p1o)));
	val lc=ref [];
    in
	(
	 while (length(!p1o)>0 andalso hd(#2(hd(!p1o)))=lp) do
	     (
	      lc:=hd(mv_cut([hd(!p1o)]))::(!lc);
	      p1o:=tl(!p1o)
	      );
	 if !lc=[] then raise error ("RATIONALS_MV_LC_EXCEPTION: lc is empty") else ();
	 mv_rev_to(!lc)
	 )
    end;

(*. compares two powerproducts .*)
fun mv_monom_equal((_,xlist):mv_monom,(_,ylist):mv_monom) = (foldr and_) (((map op=) (xlist~~ylist)),true);
    
(*. help function for mv_add .*)
fun mv_madd([]:mv_poly,[]:mv_poly,order) = []:mv_poly
  | mv_madd([(0,_)],p2,order) = p2
  | mv_madd(p1,[(0,_)],order) = p1  
  | mv_madd([],p2,order) = p2
  | mv_madd(p1,[],order) = p1
  | mv_madd(p1,p2,order) = 
    (
     if mv_monom_greater(hd(p1),hd(p2),order) 
	 then hd(p1)::mv_madd(tl(p1),p2,order)
     else if mv_monom_equal(hd(p1),hd(p2)) 
	      then if mv_lc2(p1,order)+mv_lc2(p2,order)<>0 
		       then (mv_lc2(p1,order)+mv_lc2(p2,order),mv_lpp(p1,order))::mv_madd(tl(p1),tl(p2),order)
		   else mv_madd(tl(p1),tl(p2),order)
	  else hd(p2)::mv_madd(p1,tl(p2),order)
	      )
	      
(*. adds two multivariate polynomials .*)
fun mv_add([]:mv_poly,p2:mv_poly,order) = p2
  | mv_add(p1,[],order) = p1
  | mv_add(p1,p2,order) = mv_madd(rev(sort (mv_geq order) p1),rev(sort (mv_geq order) p2), order);

(*. monom multiplication .*)
fun mv_mmul((x1,y1):mv_monom,(x2,y2):mv_monom)=(x1*x2,(map op+) (y1~~y2)):mv_monom;

(*. deletes all monomials with coefficient 0 .*)
fun mv_shorten([]:mv_poly,order) = []:mv_poly
  | mv_shorten(x::xs,order)=mv_madd([x],mv_shorten(xs,order),order);

(*. zeros a list .*)
fun mv_null2([])=[]
  | mv_null2(x::l)=0::mv_null2(l);

(*. multiplies two multivariate polynomials .*)
fun mv_mul([]:mv_poly,[]:mv_poly,_) = []:mv_poly
  | mv_mul([],y::p2,_) = [(0,mv_null2(#2(y)))]
  | mv_mul(x::p1,[],_) = [(0,mv_null2(#2(x)))] 
  | mv_mul(x::p1,y::p2,order) = mv_shorten(rev(sort (mv_geq order) (mv_mmul(x,y) :: (mv_mul(p1,y::p2,order) @
									    mv_mul([x],p2,order)))),order);

(*. gets the maximum value of a list .*)
fun mv_getmax([])=0
  | mv_getmax(x::p1)= let 
		       val m=mv_getmax(p1);
		   in
		       if m>x then m
		       else x
		   end;
(*. calculates the maximum degree of an multivariate polynomial .*)
fun mv_grad([]:mv_poly) = 0 
  | mv_grad(p1:mv_poly)= mv_getmax((map mv_addlist) ((map #2) p1));

(*. converts the sign of a value .*)
fun mv_minus(x)=(~1) * x;

(*. converts the sign of all coefficients of a polynomial .*)
fun mv_minus2([]:mv_poly)=[]:mv_poly
  | mv_minus2(p1)=(mv_minus(#1(hd(p1))),#2(hd(p1)))::(mv_minus2(tl(p1)));

(*. searches for a negativ value in a list .*)
fun mv_is_negativ([])=false
  | mv_is_negativ(x::xs)=if x<0 then true else mv_is_negativ(xs);

(*. division of monomials .*)
fun mv_mdiv((0,[]):mv_monom,_:mv_monom)=(0,[]):mv_monom
  | mv_mdiv(_,(0,[]))= raise error ("RATIONALS_MV_MDIV_EXCEPTION Division by 0 ")
  | mv_mdiv(p1:mv_monom,p2:mv_monom)= 
    let
	val c=ref (#1(p2));
	val pp=ref [];
    in 
	(
	 if !c=0 then raise error("MV_MDIV_EXCEPTION Dividing by zero")
	 else c:=(#1(p1) div #1(p2));
	     if #1(p2)<>0 then 
		 (
		  pp:=(#2(mv_mmul((1,#2(p1)),(1,(map mv_minus) (#2(p2))))));
		  if mv_is_negativ(!pp) then (0,!pp)
		  else (!c,!pp) 
		      )
	     else raise error("MV_MDIV_EXCEPTION Dividing by empty Polynom")
		 )
    end;

(*. prints a polynom for (internal use only) .*)
fun mv_print_poly([]:mv_poly)=print("[]\n")
  | mv_print_poly((x,y)::[])= print("("^BasisLibrary.Int.toString(x)^","^ints2str(y)^")\n")
  | mv_print_poly((x,y)::p1) = (print("("^BasisLibrary.Int.toString(x)^","^ints2str(y)^"),");mv_print_poly(p1));


(*. division of two multivariate polynomials .*) 
fun mv_division([]:mv_poly,g:mv_poly,order)=([]:mv_poly,[]:mv_poly)
  | mv_division(f,[],order)= raise error ("RATIONALS_MV_DIVISION_EXCEPTION Division by zero")
  | mv_division(f,g,order)=
    let 
	val r=ref [];
	val q=ref [];
	val g'=ref [];
	val k=ref 0;
	val m=ref (0,[0]);
	val exit=ref 0;
    in
	r := rev(sort (mv_geq order) (mv_shorten(f,order)));
	g':= rev(sort (mv_geq order) (mv_shorten(g,order)));
	if #1(hd(!g'))=0 then raise error("RATIONALS_MV_DIVISION_EXCEPTION: Dividing by zero") else ();
	if  (mv_monom_greater (hd(!g'),hd(!r),order)) then ([(0,mv_null2(#2(hd(f))))],(!r))
	else
	    (
	     exit:=0;
	     while (if (!exit)=0 then not(mv_monom_greater (hd(!g'),hd(!r),order)) else false) do
		 (
		  if (#1(mv_lm(!g',order)))<>0 then m:=mv_mdiv(mv_lm(!r,order),mv_lm(!g',order))
		  else raise error ("RATIONALS_MV_DIVISION_EXCEPTION: Dividing by zero");	  
		  if #1(!m)<>0 then
		      ( 
		       q:=(!m)::(!q);
		       r:=mv_add((!r),mv_minus2(mv_mul(!g',[!m],order)),order)
		       )
		  else exit:=1;
		  if (if length(!r)<>0 then length(!g')<>0 else false) then ()
		  else (exit:=1)
		  );
		 (rev(!q),!r)
		 )
    end;

(*. multiplies a polynomial with an integer .*)
fun mv_skalar_mul([]:mv_poly,c) = []:mv_poly
  | mv_skalar_mul((x,y)::p1,c) = ((x * c),y)::mv_skalar_mul(p1,c); 

(*. inserts the a first variable into an polynomial with exponent v .*)
fun mv_correct([]:mv_poly,v:int)=[]:mv_poly
  | mv_correct((x,y)::list,v:int)=(x,v::y)::mv_correct(list,v);

(*. multivariate case .*)

(*. decides if x is a factor of y .*)
fun mv_divides([]:mv_poly,[]:mv_poly)=  raise error("RATIONALS_MV_DIVIDES_EXCEPTION: division by zero")
  | mv_divides(x,[]) =  raise error("RATIONALS_MV_DIVIDES_EXCEPTION: division by zero")
  | mv_divides(x:mv_poly,y:mv_poly) = #2(mv_division(y,x,LEX_))=[];

(*. gets the maximum of a and b .*)
fun mv_max(a,b) = if a>b then a else b;

(*. gets the maximum exponent of a mv polynomial in the lexicographic term order .*)
fun mv_deg([]:mv_poly) = 0  
  | mv_deg(p1)=
    let
	val p1'=mv_shorten(p1,LEX_);
    in
	if length(p1')=0 then 0 
	else mv_max(hd(#2(hd(p1'))),mv_deg(tl(p1')))
    end;

(*. gets the maximum exponent of a mv polynomial in the reverse lexicographic term order .*)
fun mv_deg2([]:mv_poly) = 0
  | mv_deg2(p1)=
    let
	val p1'=mv_shorten(p1,LEX_);
    in
	if length(p1')=0 then 0 
	else mv_max(hd(rev(#2(hd(p1')))),mv_deg2(tl(p1')))
    end;

(*. evaluates the mv polynomial at the value v of the main variable .*)
fun mv_subs([]:mv_poly,v) = []:mv_poly
  | mv_subs((c,e)::p1:mv_poly,v) = mv_skalar_mul(mv_cut([(c,e)]),power v (hd(e))) @ mv_subs(p1,v);

(*. calculates the content of a uv-polynomial in mv-representation .*)
fun uv_content2([]:mv_poly) = 0
  | uv_content2((c,e)::p1) = (gcd_int c (uv_content2(p1)));

(*. converts a uv-polynomial from mv-representation to  uv-representation .*)
fun uv_to_list ([]:mv_poly)=[]:uv_poly
  | uv_to_list ((c1,e1)::others) = 
    let
	val count=ref 0;
	val max=mv_grad((c1,e1)::others); 
	val help=ref ((c1,e1)::others);
	val list=ref [];
    in
	if length(e1)>1 then raise error ("RATIONALS_TO_LIST_EXCEPTION: not univariate")
	else if length(e1)=0 then [c1]
	     else
		 (
		  count:=0;
		  while (!count)<=max do
		      (
		       if length(!help)>0 andalso hd(#2(hd(!help)))=max-(!count) then 
			   (
			    list:=(#1(hd(!help)))::(!list);		       
			    help:=tl(!help) 
			    )
		       else 
			   (
			    list:= 0::(!list)
			    );
		       count := (!count) + 1
		       );
		      (!list)
		      )
    end;

(*. converts a uv-polynomial from uv-representation to mv-representation .*)
fun uv_to_poly ([]:uv_poly) = []:mv_poly
  | uv_to_poly p1 = 
    let
	val count=ref 0;
	val help=ref p1;
	val list=ref [];
    in
	while length(!help)>0 do
	    (
	     if hd(!help)=0 then ()
	     else list:=(hd(!help),[!count])::(!list);
	     count:=(!count)+1;
	     help:=tl(!help)
	     );
	    (!list)
    end;

(*. univariate gcd calculation from polynomials in multivariate representation .*)
fun uv_gcd ([]:mv_poly) p2 = p2
  | uv_gcd p1 ([]:mv_poly) = p1
  | uv_gcd p1 [(c,[e])] = 
    let 
	val list=ref (rev(sort (mv_geq LEX_) (mv_shorten(p1,LEX_))));
	val min=uv_mod_min(e,(hd(#2(hd(rev(!list))))));
    in
	[(gcd_int (uv_content2(p1)) c,[min])]
    end
  | uv_gcd [(c,[e])] p2 = 
    let 
	val list=ref (rev(sort (mv_geq LEX_) (mv_shorten(p2,LEX_))));
	val min=uv_mod_min(e,(hd(#2(hd(rev(!list))))));
    in
	[(gcd_int (uv_content2(p2)) c,[min])]
    end 
  | uv_gcd p11 p22 = uv_to_poly(uv_mod_gcd (uv_to_list(mv_shorten(p11,LEX_))) (uv_to_list(mv_shorten(p22,LEX_))));

(*. help function for the newton interpolation .*)
fun mv_newton_help ([]:mv_poly list,k:int) = []:mv_poly list
  | mv_newton_help (pl:mv_poly list,k) = 
    let
	val x=ref (rev(pl));
	val t=ref [];
	val y=ref [];
	val n=ref 1;
	val n1=ref[];
    in
	(
	 while length(!x)>1 do 
	     (
	      if length(hd(!x))>0 then n1:=mv_null2(#2(hd(hd(!x))))
	      else if length(hd(tl(!x)))>0 then n1:=mv_null2(#2(hd(hd(tl(!x)))))
		   else n1:=[]; 
	      t:= #1(mv_division(mv_add(hd(!x),mv_skalar_mul(hd(tl(!x)),~1),LEX_),[(k,!n1)],LEX_)); 
	      y:=(!t)::(!y);
	      x:=tl(!x)
	      );
	 (!y)
	 )
    end;
    
(*. help function for the newton interpolation .*)
fun mv_newton_add ([]:mv_poly list) t = []:mv_poly
  | mv_newton_add [x:mv_poly] t = x 
  | mv_newton_add (pl:mv_poly list) t = 
    let
	val expos=ref [];
	val pll=ref pl;
    in
	(

	 while length(!pll)>0 andalso hd(!pll)=[]  do 
	     ( 
	      pll:=tl(!pll)
	      ); 
	 if length(!pll)>0 then expos:= #2(hd(hd(!pll))) else expos:=[];
	 mv_add(hd(pl),
		mv_mul(
		       mv_add(mv_correct(mv_cut([(1,mv_null2(!expos))]),1),[(~t,mv_null2(!expos))],LEX_),
		       mv_newton_add (tl(pl)) (t+1),
		       LEX_
		       ),
		LEX_)
	 )
    end;

(*. calculates the newton interpolation with polynomial coefficients .*)
(*. step-depth is 1 and if the result is not an integerpolynomial .*)
(*. this function returns [] .*)
fun mv_newton ([]:(mv_poly) list) = []:mv_poly 
  | mv_newton ([mp]:(mv_poly) list) = mp:mv_poly
  | mv_newton pl =
    let
	val c=ref pl;
	val c1=ref [];
	val n=length(pl);
	val k=ref 1;
	val i=ref n;
	val ppl=ref [];
    in
	c1:=hd(pl)::[];
	c:=mv_newton_help(!c,!k);
	c1:=(hd(!c))::(!c1);
	while(length(!c)>1 andalso !k<n) do
	    (	 
	     k:=(!k)+1; 
	     while  length(!c)>0 andalso hd(!c)=[] do c:=tl(!c); 	  
	     if !c=[] then () else c:=mv_newton_help(!c,!k);
	     ppl:= !c;
	     if !c=[] then () else  c1:=(hd(!c))::(!c1)
	     );
	while  hd(!c1)=[] do c1:=tl(!c1);
	c1:=rev(!c1);
	ppl:= !c1;
	mv_newton_add (!c1) 1
    end;

(*. sets the exponents of the first variable to zero .*)
fun mv_null3([]:mv_poly)    = []:mv_poly
  | mv_null3((x,y)::xs) = (x,0::tl(y))::mv_null3(xs);

(*. calculates the minimum exponents of a multivariate polynomial .*)
fun mv_min_pp([]:mv_poly)=[]
  | mv_min_pp((c,e)::xs)=
    let
	val y=ref xs;
	val x=ref [];
    in
	(
	 x:=e;
	 while length(!y)>0 do
	     (
	      x:=(map uv_mod_min) ((!x) ~~ (#2(hd(!y))));
	      y:=tl(!y)
	      );
	 !x
	 )
    end;

(*. checks if all elements of the list have value zero .*)
fun list_is_null [] = true 
  | list_is_null (x::xs) = (x=0 andalso list_is_null(xs)); 

(* check if main variable is zero*)
fun main_zero (ms : mv_poly) = (list_is_null o (map (hd o #2))) ms;

(*. calculates the content of an polynomial .*)
fun mv_content([]:mv_poly) = []:mv_poly
  | mv_content(p1) = 
    let
	val list=ref (rev(sort (mv_geq LEX_) (mv_shorten(p1,LEX_))));
	val test=ref (hd(#2(hd(!list))));
	val result=ref []; 
	val min=(hd(#2(hd(rev(!list)))));
    in
	(
	 if length(!list)>1 then
	     (
	      while (if length(!list)>0 then (hd(#2(hd(!list)))=(!test)) else false) do
		  (
		   result:=(#1(hd(!list)),tl(#2(hd(!list))))::(!result);
		   
		   if length(!list)<1 then list:=[]
		   else list:=tl(!list) 
		       
		       );		  
		  if length(!list)>0 then  
		   ( 
		    list:=mv_gcd (!result) (mv_cut(mv_content(!list))) 
		    ) 
		  else list:=(!result); 
		  list:=mv_correct(!list,0);  
		  (!list) 
		  )
	 else
	     (
	      mv_null3(!list) 
	      )
	     )
    end

(*. calculates the primitiv part of a polynomial .*)
and mv_pp([]:mv_poly) = []:mv_poly
  | mv_pp(p1) = let
		    val cont=ref []; 
		    val pp=ref[];
		in
		    cont:=mv_content(p1);
		    pp:=(#1(mv_division(p1,!cont,LEX_)));
		    if !pp=[] 
			then raise error("RATIONALS_MV_PP_EXCEPTION: Invalid Content ")
		    else (!pp)
		end

(*. calculates the gcd of two multivariate polynomials with a modular approach .*)
and mv_gcd ([]:mv_poly) ([]:mv_poly) :mv_poly= []:mv_poly
  | mv_gcd ([]:mv_poly) (p2) :mv_poly= p2:mv_poly
  | mv_gcd (p1:mv_poly) ([]) :mv_poly= p1:mv_poly
  | mv_gcd ([(x,xs)]:mv_poly) ([(y,ys)]):mv_poly = 
     let
      val xpoly:mv_poly = [(x,xs)];
      val ypoly:mv_poly = [(y,ys)];
     in 
	(
	 if xs=ys then [((gcd_int x y),xs)]
	 else [((gcd_int x y),(map uv_mod_min)(xs~~ys))]:mv_poly
        )
    end 
  | mv_gcd (p1:mv_poly) ([(y,ys)]) :mv_poly= 
	(
	 [(gcd_int (uv_content2(p1)) (y),(map  uv_mod_min)(mv_min_pp(p1)~~ys))]:mv_poly
	)
  | mv_gcd ([(y,ys)]:mv_poly) (p2):mv_poly = 
	(
         [(gcd_int (uv_content2(p2)) (y),(map  uv_mod_min)(mv_min_pp(p2)~~ys))]:mv_poly
        )
  | mv_gcd (p1':mv_poly) (p2':mv_poly):mv_poly=
    let
	val vc=length(#2(hd(p1')));
	val cont = 
		  (
                   if main_zero(mv_content(p1')) andalso 
                     (main_zero(mv_content(p2'))) then
                     mv_correct((mv_gcd (mv_cut(mv_content(p1'))) (mv_cut(mv_content(p2')))),0)
                   else 
                     mv_gcd (mv_content(p1')) (mv_content(p2'))
                  );
	val p1= #1(mv_division(p1',cont,LEX_));
	val p2= #1(mv_division(p2',cont,LEX_)); 
	val gcd=ref [];
	val candidate=ref [];
	val interpolation_list=ref [];
	val delta=ref [];
        val p1r = ref [];
        val p2r = ref [];
        val p1r' = ref [];
        val p2r' = ref [];
	val factor=ref [];
	val r=ref 0;
	val gcd_r=ref [];
	val d=ref 0;
	val exit=ref 0;
	val current_degree=ref 99999; (*. FIXME: unlimited ! .*)
    in
	(
	 if vc<2 then (* areUnivariate(p1',p2') *)
	     (
	      gcd:=uv_gcd (mv_shorten(p1',LEX_)) (mv_shorten(p2',LEX_))
	      )
	 else
	     (
	      while !exit=0 do
		  (
		   r:=(!r)+1;
                   p1r := mv_lc(p1,LEX_);
		   p2r := mv_lc(p2,LEX_);
                   if main_zero(!p1r) andalso
                      main_zero(!p2r) 
                   then
                       (
                        delta := mv_correct((mv_gcd (mv_cut (!p1r)) (mv_cut (!p2r))),0)
                       )
                   else
                       (
		        delta := mv_gcd (!p1r) (!p2r)
                       );
		   if mv_shorten(mv_subs(!p1r,!r),LEX_)=[] andalso 
		      mv_shorten(mv_subs(!p2r,!r),LEX_)=[] 
                   then 
                       (
		       )
		   else 
		       (
			gcd_r:=mv_shorten(mv_gcd (mv_shorten(mv_subs(p1,!r),LEX_)) 
					         (mv_shorten(mv_subs(p2,!r),LEX_)) ,LEX_);
			gcd_r:= #1(mv_division(mv_mul(mv_correct(mv_subs(!delta,!r),0),!gcd_r,LEX_),
					       mv_correct(mv_lc(!gcd_r,LEX_),0),LEX_));
			d:=mv_deg2(!gcd_r); (* deg(gcd_r,z) *)
			if (!d < !current_degree) then 
			    (
			     current_degree:= !d;
			     interpolation_list:=[mv_correct(!gcd_r,0)]
			     )
			else
			    (
			     if (!d = !current_degree) then
				 (
				  interpolation_list:=mv_correct(!gcd_r,0)::(!interpolation_list)
				  )
			     else () 
				 )
			    );
		      if length(!interpolation_list)> uv_mod_min(mv_deg(p1),mv_deg(p2)) then 
			  (
			   candidate := mv_newton(rev(!interpolation_list));
			   if !candidate=[] then ()
			   else
			       (
				candidate:=mv_pp(!candidate);
				if mv_divides(!candidate,p1) andalso mv_divides(!candidate,p2) then
				    (
				     gcd:= mv_mul(!candidate,cont,LEX_);
				     exit:=1
				     )
				else ()
				    );
			       interpolation_list:=[mv_correct(!gcd_r,0)]
			       )
		      else ()
			  )
	     );
	     (!gcd):mv_poly
	     )
    end;	


(*. calculates the least common divisor of two polynomials .*)
fun mv_lcm (p1:mv_poly) (p2:mv_poly) :mv_poly = 
    (
     #1(mv_division(mv_mul(p1,p2,LEX_),mv_gcd p1 p2,LEX_))
     );

(*. gets the variables (strings) of a term .*)
fun get_vars(term1) = (map free2str) (vars term1); (*["a","b","c"]; *)

(*. counts the negative coefficents in a polynomial .*)
fun count_neg ([]:mv_poly) = 0 
  | count_neg ((c,e)::xs) = if c<0 then 1+count_neg xs
			  else count_neg xs;

(*. help function for is_polynomial  
    checks the order of the operators .*)
fun test_polynomial (Const ("uminus",_) $ Free (str,_)) _ = true (*WN.13.3.03*)
  | test_polynomial (t as Free(str,_)) v = true
  | test_polynomial (t as Const ("op *",_) $ t1 $ t2) v = if v="^" then false
						     else (test_polynomial t1 "*") andalso (test_polynomial t2 "*")
  | test_polynomial (t as Const ("op +",_) $ t1 $ t2) v = if v="*" orelse v="^" then false 
							  else (test_polynomial t1 " ") andalso (test_polynomial t2 " ")
  | test_polynomial (t as Const ("Atools.pow",_) $ t1 $ t2) v = (test_polynomial t1 "^") andalso (test_polynomial t2 "^")
  | test_polynomial _ v = false;  

(*. tests if a term is a polynomial .*)  
fun is_polynomial t = test_polynomial t " ";

(*. help function for is_expanded 
    checks the order of the operators .*)
fun test_exp (t as Free(str,_)) v = true 
  | test_exp (t as Const ("op *",_) $ t1 $ t2) v = if v="^" then false
						     else (test_exp t1 "*") andalso (test_exp t2 "*")
  | test_exp (t as Const ("op +",_) $ t1 $ t2) v = if v="*" orelse v="^" then false 
							  else (test_exp t1 " ") andalso (test_exp t2 " ") 
  | test_exp (t as Const ("op -",_) $ t1 $ t2) v = if v="*" orelse v="^" then false 
							  else (test_exp t1 " ") andalso (test_exp t2 " ")
  | test_exp (t as Const ("Atools.pow",_) $ t1 $ t2) v = (test_exp t1 "^") andalso (test_exp t2 "^")
  | test_exp  _ v = false;


(*. help function for check_coeff: 
    converts the term to a list of coefficients .*) 
fun term2coef' (t as Free(str,_(*typ*))) v :mv_poly option = 
    let
	val x=ref None;
	val len=ref 0;
	val vl=ref [];
	val vh=ref [];
	val i=ref 0;
    in 
	if is_numeral str then
	    (
	     Some [(((the o int_of_str) str),mv_null2(v))] handle _ => None
		 )
	else (* variable *)
	    (
	     len:=length(v);
	     vh:=v;
	     while ((!len)>(!i)) do
		 (
		  if str=hd((!vh)) then
		      (
		       vl:=1::(!vl)
		       )
		  else 
		      (
		       vl:=0::(!vl)
		       );
		      vh:=tl(!vh);
		      i:=(!i)+1    
		      );		
		 Some [(1,rev(!vl))] handle _ => None
	    )
    end
  | term2coef' (Const ("op *",_) $ t1 $ t2) v :mv_poly option= 
    let
	val t1pp=ref [];
	val t2pp=ref [];
	val t1c=ref 0;
	val t2c=ref 0;
    in
	(
	 t1pp:=(#2(hd(the(term2coef' t1 v))));
	 t2pp:=(#2(hd(the(term2coef' t2 v))));
	 t1c:=(#1(hd(the(term2coef' t1 v))));
	 t2c:=(#1(hd(the(term2coef' t2 v))));
	
	 Some [( (!t1c)*(!t2c) ,( (map op+) ((!t1pp)~~(!t2pp)) ) )] handle _ => None 
		
	 )
    end
  | term2coef' (Const ("Atools.pow",_) $ (t1 as Free (str1,_)) $ (t2 as Free (str2,_))) v :mv_poly option= 
    let
	val x=ref None;
	val len=ref 0;
	val vl=ref [];
	val vh=ref [];
	val vtemp=ref [];
	val i=ref 0;	 
    in
    (
     if (not o is_numeral) str1 andalso is_numeral str2 then
	 (
	  len:=length(v);
	  vh:=v;

	  while ((!len)>(!i)) do
	      (
	       if str1=hd((!vh)) then
		   (
		    vl:=((the o int_of_str) str2)::(!vl)
		    )
	       else 
		   (
		    vl:=0::(!vl)
		    );
		   vh:=tl(!vh);
		   i:=(!i)+1     
		   );
	      Some [(1,rev(!vl))] handle _ => None
	      )
     else raise error ("RATIONALS_TERM2COEF_EXCEPTION 1: Invalid term")
	 )
    end
  | term2coef' (Const ("op +",_) $ t1 $ t2) v :mv_poly option= 
    (
     Some ((the(term2coef' t1 v)) @ (the(term2coef' t2 v))) handle _ => None
	 )
  | term2coef' (Const ("op -",_) $ t1 $ t2) v :mv_poly option= 
    (
     Some ((the(term2coef' t1 v)) @ mv_skalar_mul((the(term2coef' t2 v)),1)) handle _ => None
	 )
  | term2coef' (term) v = raise error ("RATIONALS_TERM2COEF_EXCEPTION 2: Invalid term");

(*. checks if all coefficients of a polynomial are positiv (except the first) .*)
fun check_coeff t = (* erste Koeffizient kann <0 sein !!! *)
    if count_neg(tl(the(term2coef' t (get_vars(t)))))=0 then true 
    else false;

(*. checks for expanded term [3] .*)
fun is_expanded t = test_exp t " " andalso check_coeff(t); 

(*WN.7.3.03 Hilfsfunktion f"ur term2poly'*)
fun mk_monom v' p vs = 
    let fun conv p (v: string) = if v'= v then p else 0
    in map (conv p) vs end;
(* mk_monom "y" 5 ["a","b","x","y","z"];
val it = [0,0,0,5,0] : int list*)

(*. this function converts the term representation into the internal representation mv_poly .*)
fun term2poly' (Const ("uminus",_) $ Free (str,_)) v = (*WN.7.3.03*)
    if is_numeral str 
    then Some [((the o int_of_str) ("-"^str), mk_monom "#" 0 v)]
    else Some [(~1, mk_monom str 1 v)]

  | term2poly' (Free(str,_)) v :mv_poly option = 
    let
	val x=ref None;
	val len=ref 0;
	val vl=ref [];
	val vh=ref [];
	val i=ref 0;
    in 
	if is_numeral str then
	    (
	     Some [(((the o int_of_str) str),mv_null2 v)] handle _ => None
		 )
	else (* variable *)
	    (
	     len:=length v;
	     vh:= v;
	     while ((!len)>(!i)) do
		 (
		  if str=hd((!vh)) then
		      (
		       vl:=1::(!vl)
		       )
		  else 
		      (
		       vl:=0::(!vl)
		       );
		      vh:=tl(!vh);
		      i:=(!i)+1    
		      );		
		 Some [(1,rev(!vl))] handle _ => None
	    )
    end
  | term2poly' (Const ("op *",_) $ t1 $ t2) v :mv_poly option= 
    let
	val t1pp=ref [];
	val t2pp=ref [];
	val t1c=ref 0;
	val t2c=ref 0;
    in
	(
	 t1pp:=(#2(hd(the(term2poly' t1 v))));
	 t2pp:=(#2(hd(the(term2poly' t2 v))));
	 t1c:=(#1(hd(the(term2poly' t1 v))));
	 t2c:=(#1(hd(the(term2poly' t2 v))));
	
	 Some [( (!t1c)*(!t2c) ,( (map op+) ((!t1pp)~~(!t2pp)) ) )] 
	 handle _ => None 
		
	 )
    end
  | term2poly' (Const ("Atools.pow",_) $ (t1 as Free (str1,_)) $ 
		      (t2 as Free (str2,_))) v :mv_poly option= 
    let
	val x=ref None;
	val len=ref 0;
	val vl=ref [];
	val vh=ref [];
	val vtemp=ref [];
	val i=ref 0;	 
    in
    (
     if (not o is_numeral) str1 andalso is_numeral str2 then
	 (
	  len:=length(v);
	  vh:=v;

	  while ((!len)>(!i)) do
	      (
	       if str1=hd((!vh)) then
		   (
		    vl:=((the o int_of_str) str2)::(!vl)
		    )
	       else 
		   (
		    vl:=0::(!vl)
		    );
		   vh:=tl(!vh);
		   i:=(!i)+1     
		   );
	      Some [(1,rev(!vl))] handle _ => None
	      )
     else raise error ("RATIONALS_TERM2POLY_EXCEPTION 1: Invalid term")
	 )
    end
  | term2poly' (Const ("op +",_) $ t1 $ t2) v :mv_poly option = 
    (
     Some ((the(term2poly' t1 v)) @ (the(term2poly' t2 v))) handle _ => None
	 )
  | term2poly' (Const ("op -",_) $ t1 $ t2) v :mv_poly option = 
    (
     Some ((the(term2poly' t1 v)) @ mv_skalar_mul((the(term2poly' t2 v)),~1)) handle _ => None
	 )
  | term2poly' (term) v = raise error ("RATIONALS_TERM2POLY_EXCEPTION 2: Invalid term");

(*. translates an Isabelle term into internal representation.
    term2poly
    fn : term ->              (*normalform [2]                    *)
    	 string list ->       (*for ...!!! BITTE DIE ERKLRUNG, 
    			       DIE DU MIR LETZTES MAL GEGEBEN HAST*)
    	 mv_monom list        (*internal representation           *)
    		  option      (*the translation may fail with None*)
.*)
fun term2poly (t:term) v = 
     if is_polynomial t then term2poly' t v
     else raise error ("term2poly: invalid = "^(term2str t));

(*. same as term2poly with automatic detection of the variables .*)
fun term2polyx t = term2poly t (((map free2str) o vars) t); 

(*. checks if the term is in expanded polynomial form and converts it into the internal representation .*)
fun expanded2poly (t:term) v = 
    (*if is_expanded t then*) term2poly' t v
    (*else raise error ("RATIONALS_EXPANDED2POLY_EXCEPTION: Invalid Polynomial")*);

(*. same as expanded2poly with automatic detection of the variables .*)
fun expanded2polyx t = expanded2poly t (((map free2str) o vars) t);

(*. converts a powerproduct into term representation .*)
fun powerproduct2term(xs,v) =  
    let
	val xss=ref xs;
	val vv=ref v;
    in
	(
	 while hd(!xss)=0 do 
	     (
	      xss:=tl(!xss);
	      vv:=tl(!vv)
	      );
	     
	 if list_is_null(tl(!xss)) then 
	     (
	      if hd(!xss)=1 then Free(hd(!vv), HOLogic.realT)
	      else
		  (
		   Const("Atools.pow",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		   Free(hd(!vv), HOLogic.realT) $  Free(str_of_int (hd(!xss)),HOLogic.realT)
		   )
	      )
	 else
	     (
	      if hd(!xss)=1 then 
		  ( 
		   Const("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		   Free(hd(!vv), HOLogic.realT) $
		   powerproduct2term(tl(!xss),tl(!vv))
		   )
	      else
		  (
		   Const("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		   (
		    Const("Atools.pow",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		    Free(hd(!vv), HOLogic.realT) $  Free(str_of_int (hd(!xss)),HOLogic.realT)
		    ) $
		    powerproduct2term(tl(!xss),tl(!vv))
		   )
	      )
	 )
    end;

(*. converts a monom into term representation .*)
(*fun monom2term ((c,e):mv_monom, v:string list) = 
    if c=0 then Free(str_of_int 0,HOLogic.realT)  
    else
	(
	 if list_is_null(e) then
	     ( 
	      Free(str_of_int c,HOLogic.realT)  
	      )
	 else
	     (
	      if c=1 then 
		  (
		   powerproduct2term(e,v)
		   )
	      else
		  (
		   Const("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $
		   Free(str_of_int c,HOLogic.realT)  $
		   powerproduct2term(e,v)
		   )
		  )
	     );*)


(*fun monom2term ((i, is):mv_monom, v) = 
    if list_is_null is 
    then 
	if i >= 0 
	then Free (str_of_int i, HOLogic.realT)
	else Const ("uminus", HOLogic.realT --> HOLogic.realT) $
		   Free ((str_of_int o abs) i, HOLogic.realT)
    else
	if i > 0 
	then Const ("op *", [HOLogic.realT,HOLogic.realT]---> HOLogic.realT) $
		   (Free (str_of_int i, HOLogic.realT)) $
		   powerproduct2term(is, v)
	else Const ("op *", [HOLogic.realT,HOLogic.realT]---> HOLogic.realT) $
		   (Const ("uminus", HOLogic.realT --> HOLogic.realT) $
		   Free ((str_of_int o abs) i, HOLogic.realT)) $
		   powerproduct2term(is, vs);---------------------------*)
fun monom2term ((i, is) : mv_monom, vs) = 
    if list_is_null is 
    then Free (str_of_int i, HOLogic.realT)
    else if i = 1
    then powerproduct2term (is, vs)
    else Const ("op *", [HOLogic.realT, HOLogic.realT] ---> HOLogic.realT) $
	       (Free (str_of_int i, HOLogic.realT)) $
	       powerproduct2term (is, vs);
    
(*. converts the internal polynomial representation into an Isabelle term.*)
fun poly2term' ([] : mv_poly, vs) = Free(str_of_int 0, HOLogic.realT)  
  | poly2term' ([(c, e) : mv_monom], vs) = monom2term ((c, e), vs)
  | poly2term' ((c, e) :: ces, vs) =  
    Const("op +", [HOLogic.realT, HOLogic.realT] ---> HOLogic.realT) $
         poly2term (ces, vs) $ monom2term ((c, e), vs)
and poly2term (xs, vs) = poly2term' (rev (sort (mv_geq LEX_) (xs)), vs);


(*. converts a monom into term representation .*)
(*. ignores the sign of the coefficients => use only for exp-poly functions .*)
fun monom2term2((c,e):mv_monom, v:string list) =  
    if c=0 then Free(str_of_int 0,HOLogic.realT)  
    else
	(
	 if list_is_null(e) then
	     ( 
	      Free(str_of_int (abs(c)),HOLogic.realT)  
	      )
	 else
	     (
	      if abs(c)=1 then 
		  (
		   powerproduct2term(e,v)
		   )
	      else
		  (
		   Const("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $
		   Free(str_of_int (abs(c)),HOLogic.realT)  $
		   powerproduct2term(e,v)
		   )
		  )
	     );

(*. converts the expanded polynomial representation into the term representation .*)
fun exp2term' ([]:mv_poly,vars) =  Free(str_of_int 0,HOLogic.realT)  
  | exp2term' ([(c,e)],vars) =     monom2term((c,e),vars) 			     
  | exp2term' ((c1,e1)::others,vars) =  
    if c1<0 then 	
	Const("op -",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $
	exp2term'(others,vars) $
	( 
	 monom2term2((c1,e1),vars)
	 ) 
    else
	Const("op +",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $
	exp2term'(others,vars) $
	( 
	 monom2term2((c1,e1),vars)
	 );
	
(*. sorts the powerproduct by lexicographic termorder and converts them into 
    a term in polynomial representation .*)
fun poly2expanded (xs,vars) = exp2term'(rev(sort (mv_geq LEX_) (xs)),vars);

(*. converts a polynomial into expanded form .*)
fun polynomial2expanded t =  
    (let 
	val vars=(((map free2str) o vars) t);
    in
	Some (poly2expanded (the (term2poly t vars), vars))
    end) handle _ => None;

(*. converts a polynomial into polynomial form .*)
fun expanded2polynomial t =  
    (let 
	val vars=(((map free2str) o vars) t);
    in
	Some (poly2term (the (expanded2poly t vars), vars))
    end) handle _ => None;


(*. calculates the greatest common divisor of numerator and denominator and seperates it from each .*)
fun step_cancel (t as Const ("HOL.divide",_) $ p1 $ p2) = 
    let
	val p1' = ref [];
	val p2' = ref [];
	val p3  = ref []
	val vars = rev(get_vars(p1) union get_vars(p2));
    in
	(
         p1':= sort (mv_geq LEX_) (the (term2poly p1 vars ));
       	 p2':= sort (mv_geq LEX_) (the (term2poly p2 vars ));
	 p3:= sort (mv_geq LEX_) (mv_gcd (!p1') (!p2'));
	 if (!p3)=[(1,mv_null2(vars))] then 
	     (
	      Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ p1 $ p2
	      )
	 else
	     (

	      p1':=sort (mv_geq LEX_) (#1(mv_division((!p1'),(!p3),LEX_)));
	      p2':=sort (mv_geq LEX_) (#1(mv_division((!p2'),(!p3),LEX_)));
	      
	      if #1(hd(sort (mv_geq LEX_) (!p2'))) (*mv_lc2(!p2',LEX_)*)>0 then
	      (
	       Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
	       $ 
	       (
		Const ("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		poly2term(!p1',vars) $ 
		poly2term(!p3,vars) 
		) 
	       $ 
	       (
		Const ("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		poly2term(!p2',vars) $ 
		poly2term(!p3,vars)
		) 	
	       )	
	      else
	      (
	       p1':=mv_skalar_mul(!p1',~1);
	       p2':=mv_skalar_mul(!p2',~1);
	       p3:=mv_skalar_mul(!p3,~1);
	       (
		Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
		$ 
		(
		 Const ("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		 poly2term(!p1',vars) $ 
		 poly2term(!p3,vars) 
		 ) 
		$ 
		(
		 Const ("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		 poly2term(!p2',vars) $ 
		 poly2term(!p3,vars)
		 ) 	
		)	
	       )	  
	      )
	     )
    end
| step_cancel _ = raise error ("RATIONALS_STEP_CANCEL_EXCEPTION: Invalid fraction"); 


(*. same as step_cancel, this time for expanded forms (input+output) .*)
fun step_cancel_expanded (t as Const ("HOL.divide",_) $ p1 $ p2) = 
    let
	val p1' = ref [];
	val p2' = ref [];
	val p3  = ref []
	val vars = rev(get_vars(p1) union get_vars(p2));
    in
	(
         p1':= sort (mv_geq LEX_) (the (expanded2poly p1 vars ));
       	 p2':= sort (mv_geq LEX_) (the (expanded2poly p2 vars ));
	 p3:= sort (mv_geq LEX_) (mv_gcd (!p1') (!p2'));
	 if (!p3)=[(1,mv_null2(vars))] then 
	     (
	      Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ p1 $ p2
	      )
	 else
	     (

	      p1':=sort (mv_geq LEX_) (#1(mv_division((!p1'),(!p3),LEX_)));
	      p2':=sort (mv_geq LEX_) (#1(mv_division((!p2'),(!p3),LEX_)));
	      
	      if #1(hd(sort (mv_geq LEX_) (!p2')))(* mv_lc2(!p2',LEX_)*)>0 then
	      (
	       Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
	       $ 
	       (
		Const ("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		poly2expanded(!p1',vars) $ 
		poly2expanded(!p3,vars) 
		) 
	       $ 
	       (
		Const ("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		poly2expanded(!p2',vars) $ 
		poly2expanded(!p3,vars)
		) 	
	       )	
	      else
	      (
	       p1':=mv_skalar_mul(!p1',~1);
	       p2':=mv_skalar_mul(!p2',~1);
	       p3:=mv_skalar_mul(!p3,~1);
	       (
		Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
		$ 
		(
		 Const ("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		 poly2expanded(!p1',vars) $ 
		 poly2expanded(!p3,vars) 
		 ) 
		$ 
		(
		 Const ("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		 poly2expanded(!p2',vars) $ 
		 poly2expanded(!p3,vars)
		 ) 	
		)	
	       )	  
	      )
	     )
    end
| step_cancel_expanded _ = raise error ("RATIONALS_STEP_CANCEL_EXCEPTION: Invalid fraction"); 

(*. calculates the greatest common divisor of numerator and denominator and divides each through it .*)
fun direct_cancel (t as Const ("HOL.divide",_) $ p1 $ p2) = 
    let
	val p1' = ref [];
	val p2' = ref [];
	val p3  = ref []
	val vars = rev(get_vars(p1) union get_vars(p2));
    in
	(
	 p1':=sort (mv_geq LEX_) (mv_shorten((the (term2poly p1 vars )),LEX_));
	 p2':=sort (mv_geq LEX_) (mv_shorten((the (term2poly p2 vars )),LEX_));	 
	 p3 :=sort (mv_geq LEX_) (mv_gcd (!p1') (!p2'));

	 if (!p3)=[(1,mv_null2(vars))] then 
	     (
	      (Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ p1 $ p2,[])
	      )
	 else
	     (
	      p1':=sort (mv_geq LEX_) (#1(mv_division((!p1'),(!p3),LEX_)));
	      p2':=sort (mv_geq LEX_) (#1(mv_division((!p2'),(!p3),LEX_)));
	      if #1(hd(sort (mv_geq LEX_) (!p2'))) (*mv_lc2(!p2',LEX_)*)>0 then	      
	      (
	       (
		Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
		$ 
		(
		 poly2term((!p1'),vars)
		 ) 
		$ 
		( 
		 poly2term((!p2'),vars)
		 ) 	
		)
	       ,
	       if mv_grad(!p3)>0 then 
		   [
		    (
		     Const ("Not",[bool]--->bool) $
		     (
		      Const("op =",[HOLogic.realT,HOLogic.realT]--->bool) $
		      poly2term((!p3),vars) $
		      Free("0",HOLogic.realT)
		      )
		     )
		    ]
	       else
		   []
		   )
	      else
		  (
		   p1':=mv_skalar_mul(!p1',~1);
		   p2':=mv_skalar_mul(!p2',~1);
		   if length(!p3)> 2*(count_neg(!p3)) then () else p3 :=mv_skalar_mul(!p3,~1); 
		       (
			Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
			$ 
			(
			 poly2term((!p1'),vars)
			 ) 
			$ 
			( 
			 poly2term((!p2'),vars)
			 ) 	
			,
			if mv_grad(!p3)>0 then 
			    [
			     (
			      Const ("Not",[bool]--->bool) $
			      (
			       Const("op =",[HOLogic.realT,HOLogic.realT]--->bool) $
			       poly2term((!p3),vars) $
			       Free("0",HOLogic.realT)
			       )
			      )
			     ]
			else
			    []
			    )
		       )
		  )
	     )
    end
  | direct_cancel _ = raise error ("RATIONALS_DIRECT_CANCEL_EXCEPTION: Invalid fraction"); 

(*. same es direct_cancel, this time for expanded forms (input+output).*) 
fun direct_cancel_expanded (t as Const ("HOL.divide",_) $ p1 $ p2) =  
    let
	val p1' = ref [];
	val p2' = ref [];
	val p3  = ref []
	val vars = rev(get_vars(p1) union get_vars(p2));
    in
	(
	 p1':=sort (mv_geq LEX_) (mv_shorten((the (expanded2poly p1 vars )),LEX_));
	 p2':=sort (mv_geq LEX_) (mv_shorten((the (expanded2poly p2 vars )),LEX_));	 
	 p3 :=sort (mv_geq LEX_) (mv_gcd (!p1') (!p2'));

	 if (!p3)=[(1,mv_null2(vars))] then 
	     (
	      (Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ p1 $ p2,[])
	      )
	 else
	     (
	      p1':=sort (mv_geq LEX_) (#1(mv_division((!p1'),(!p3),LEX_)));
	      p2':=sort (mv_geq LEX_) (#1(mv_division((!p2'),(!p3),LEX_)));
	      if #1(hd(sort (mv_geq LEX_) (!p2'))) (*mv_lc2(!p2',LEX_)*)>0 then	      
	      (
	       (
		Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
		$ 
		(
		 poly2expanded((!p1'),vars)
		 ) 
		$ 
		( 
		 poly2expanded((!p2'),vars)
		 ) 	
		)
	       ,
	       if mv_grad(!p3)>0 then 
		   [
		    (
		     Const ("Not",[bool]--->bool) $
		     (
		      Const("op =",[HOLogic.realT,HOLogic.realT]--->bool) $
		      poly2expanded((!p3),vars) $
		      Free("0",HOLogic.realT)
		      )
		     )
		    ]
	       else
		   []
		   )
	      else
		  (
		   p1':=mv_skalar_mul(!p1',~1);
		   p2':=mv_skalar_mul(!p2',~1);
		   if length(!p3)> 2*(count_neg(!p3)) then () else p3 :=mv_skalar_mul(!p3,~1); 
		       (
			Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
			$ 
			(
			 poly2expanded((!p1'),vars)
			 ) 
			$ 
			( 
			 poly2expanded((!p2'),vars)
			 ) 	
			,
			if mv_grad(!p3)>0 then 
			    [
			     (
			      Const ("Not",[bool]--->bool) $
			      (
			       Const("op =",[HOLogic.realT,HOLogic.realT]--->bool) $
			       poly2expanded((!p3),vars) $
			       Free("0",HOLogic.realT)
			       )
			      )
			     ]
			else
			    []
			    )
		       )
		  )
	     )
    end
  | direct_cancel_expanded _ = raise error ("RATIONALS_DIRECT_CANCEL_EXCEPTION: Invalid fraction"); 


(*. adds two fractions .*)
fun add_fract ((Const("HOL.divide",_) $ t11 $ t12),(Const("HOL.divide",_) $ t21 $ t22)) =
    let
	val vars=get_vars(t11) union get_vars(t12) union get_vars(t21) union get_vars(t22);
	val t11'=ref (the(term2poly t11 vars));
val _= writeln"### add_fract: done t11"
	val t12'=ref (the(term2poly t12 vars));
val _= writeln"### add_fract: done t12"
	val t21'=ref (the(term2poly t21 vars));
val _= writeln"### add_fract: done t21"
	val t22'=ref (the(term2poly t22 vars));
val _= writeln"### add_fract: done t22"
	val den=ref [];
	val nom=ref [];
	val m1=ref [];
	val m2=ref [];
    in
	
	(
	 den :=sort (mv_geq LEX_) (mv_lcm (!t12') (!t22'));
writeln"### add_fract: done sort mv_lcm";
	 m1  :=sort (mv_geq LEX_) (#1(mv_division(!den,!t12',LEX_)));
writeln"### add_fract: done sort mv_division t12";
	 m2  :=sort (mv_geq LEX_) (#1(mv_division(!den,!t22',LEX_)));
writeln"### add_fract: done sort mv_division t22";
	 nom :=sort (mv_geq LEX_) 
		    (mv_shorten(mv_add(mv_mul(!t11',!m1,LEX_),
				       mv_mul(!t21',!m2,LEX_),
				       LEX_),
				LEX_));
writeln"### add_fract: done sort mv_add";
	 (
	  Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
	  $ 
	  (
	   poly2term((!nom),vars)
	   ) 
	  $ 
	  ( 
	   poly2term((!den),vars)
	   )	      
	  )
	 )	     
    end 
  | add_fract (_,_) = raise error ("RATIONALS_ADD_FRACTION_EXCEPTION: Invalid add_fraction call");

(*. adds two expanded fractions .*)
fun add_fract_exp ((Const("HOL.divide",_) $ t11 $ t12),(Const("HOL.divide",_) $ t21 $ t22)) =
    let
	val vars=get_vars(t11) union get_vars(t12) union get_vars(t21) union get_vars(t22);
	val t11'=ref (the(expanded2poly t11 vars));
	val t12'=ref (the(expanded2poly t12 vars));
	val t21'=ref (the(expanded2poly t21 vars));
	val t22'=ref (the(expanded2poly t22 vars));
	val den=ref [];
	val nom=ref [];
	val m1=ref [];
	val m2=ref [];
    in
	
	(
	 den :=sort (mv_geq LEX_) (mv_lcm (!t12') (!t22'));
	 m1  :=sort (mv_geq LEX_) (#1(mv_division(!den,!t12',LEX_)));
	 m2  :=sort (mv_geq LEX_) (#1(mv_division(!den,!t22',LEX_)));
	 nom :=sort (mv_geq LEX_) (mv_shorten(mv_add(mv_mul(!t11',!m1,LEX_),mv_mul(!t21',!m2,LEX_),LEX_),LEX_));
	 (
	  Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
	  $ 
	  (
	   poly2expanded((!nom),vars)
	   ) 
	  $ 
	  ( 
	   poly2expanded((!den),vars)
	   )	      
	  )
	 )	     
    end 
  | add_fract_exp (_,_) = raise error ("RATIONALS_ADD_FRACTION_EXP_EXCEPTION: Invalid add_fraction call");

(*. adds a list of terms .*)
fun add_list_of_fractions []= (Free("0",HOLogic.realT),[])
  | add_list_of_fractions [x]= direct_cancel x
  | add_list_of_fractions (x::y::xs) = 
    let
	val (t1a,rest1)=direct_cancel(x);
val _= writeln"### add_list_of_fractions xs: has done direct_cancel(x)";
	val (t2a,rest2)=direct_cancel(y);
val _= writeln"### add_list_of_fractions xs: has done direct_cancel(y)";
	val (t3a,rest3)=(add_list_of_fractions (add_fract(t1a,t2a)::xs));
val _= writeln"### add_list_of_fractions xs: has done add_list_of_fraction xs";
	val (t4a,rest4)=direct_cancel(t3a);
val _= writeln"### add_list_of_fractions xs: has done direct_cancel(t3a)";
	val rest=rest1 union rest2 union rest3 union rest4;
    in
	(writeln"### add_list_of_fractions in";
	 (
	 (t4a,rest) 
	 )
	 )
    end;

(*. adds a list of expanded terms .*)
fun add_list_of_fractions_exp []= (Free("0",HOLogic.realT),[])
  | add_list_of_fractions_exp [x]= direct_cancel_expanded x
  | add_list_of_fractions_exp (x::y::xs) = 
    let
	val (t1a,rest1)=direct_cancel_expanded(x);
	val (t2a,rest2)=direct_cancel_expanded(y);
	val (t3a,rest3)=(add_list_of_fractions_exp (add_fract_exp(t1a,t2a)::xs));
	val (t4a,rest4)=direct_cancel_expanded(t3a);
	val rest=rest1 union rest2 union rest3 union rest4;
    in
	(
	 (t4a,rest) 
	 )
    end;

(*. calculates the lcm of a list of mv_poly .*)
fun calc_lcm ([x],var)= (x,var) 
  | calc_lcm ((x::xs),var) = (mv_lcm x (#1(calc_lcm (xs,var))),var);

(*. converts a list of terms to a list of mv_poly .*)
fun t2d([],_)=[] 
  | t2d((t as (Const("HOL.divide",_) $ p1 $ p2))::xs,vars)= (the(term2poly p2 vars)) :: t2d(xs,vars); 

(*. same as t2d, this time for expanded forms .*)
fun t2d_exp([],_)=[]  
  | t2d_exp((t as (Const("HOL.divide",_) $ p1 $ p2))::xs,vars)= (the(expanded2poly p2 vars)) :: t2d_exp(xs,vars);

(*. converts a list of fract terms to a list of their denominators .*)
fun termlist2denominators [] = ([],[])
  | termlist2denominators xs = 
    let	
	val xxs=ref xs;
	val var=ref [];
    in
	var:=[];
	while length(!xxs)>0 do
	    (
	     let 
		 val (t as Const ("HOL.divide",_) $ p1x $ p2x)=hd(!xxs);
	     in
		 (
		  xxs:=tl(!xxs);
		  var:=((get_vars(p2x)) union (get_vars(p1x)) union (!var))
		  )
	     end
	     );
	    (t2d(xs,!var),!var)
    end;

(*. calculates the lcm of a list of mv_poly .*)
fun calc_lcm ([x],var)= (x,var) 
  | calc_lcm ((x::xs),var) = (mv_lcm x (#1(calc_lcm (xs,var))),var);

(*. converts a list of terms to a list of mv_poly .*)
fun t2d([],_)=[] 
  | t2d((t as (Const("HOL.divide",_) $ p1 $ p2))::xs,vars)= (the(term2poly p2 vars)) :: t2d(xs,vars); 

(*. same as t2d, this time for expanded forms .*)
fun t2d_exp([],_)=[]  
  | t2d_exp((t as (Const("HOL.divide",_) $ p1 $ p2))::xs,vars)= (the(expanded2poly p2 vars)) :: t2d_exp(xs,vars);

(*. converts a list of fract terms to a list of their denominators .*)
fun termlist2denominators [] = ([],[])
  | termlist2denominators xs = 
    let	
	val xxs=ref xs;
	val var=ref [];
    in
	var:=[];
	while length(!xxs)>0 do
	    (
	     let 
		 val (t as Const ("HOL.divide",_) $ p1x $ p2x)=hd(!xxs);
	     in
		 (
		  xxs:=tl(!xxs);
		  var:=((get_vars(p2x)) union (get_vars(p1x)) union (!var))
		  )
	     end
	     );
	    (t2d(xs,!var),!var)
    end;

(*. same as termlist2denminators, this time for expanded forms .*)
fun termlist2denominators_exp [] = ([],[])
  | termlist2denominators_exp xs = 
    let	
	val xxs=ref xs;
	val var=ref [];
    in
	var:=[];
	while length(!xxs)>0 do
	    (
	     let 
		 val (t as Const ("HOL.divide",_) $ p1x $ p2x)=hd(!xxs);
	     in
		 (
		  xxs:=tl(!xxs);
		  var:=((get_vars(p2x)) union (get_vars(p1x)) union (!var))
		  )
	     end
	     );
	    (t2d_exp(xs,!var),!var)
    end;

(*. reduces all fractions to the least common denominator .*)
fun com_den(x::xs,denom,den,var)=
    let 
	val (t as Const ("HOL.divide",_) $ p1' $ p2')=x;
	val p2= sort (mv_geq LEX_) (the(term2poly p2' var));
	val p3= #1(mv_division(denom,p2,LEX_));
	val p1var=get_vars(p1');
    in     
	if length(xs)>0 then 
	    if p3=[(1,mv_null2(var))] then
		(
		 Const ("op +",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT)
		 $ 
		 (
		  Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
		  $ 
		  poly2term(the (term2poly p1' p1var),p1var)
		  $ 
		  den	
		  )    
		 $ 
		 #1(com_den(xs,denom,den,var))
		,
		[]
		)
	    else
		(
		 Const ("op +",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
		 $ 
		 (
		  Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
		  $ 
		  (
		   Const ("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		   poly2term(the (term2poly p1' p1var),p1var) $ 
		   poly2term(p3,var)
		   ) 
		  $ 
		  (
		   den
		   ) 	
		  )
		 $ 
		 #1(com_den(xs,denom,den,var))
		,
		[]
		)
	else
	    if p3=[(1,mv_null2(var))] then
		(
		 (
		  Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
		  $ 
		  poly2term(the (term2poly p1' p1var),p1var)
		  $ 
		  den	
		  )
		 ,
		 []
		 )
	     else
		 (
		  Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
		  $ 
		  (
		   Const ("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		   poly2term(the (term2poly p1' p1var),p1var) $ 
		   poly2term(p3,var)
		   ) 
		  $ 
		  den 	
		  ,
		  []
		  )
    end;

(*. same as com_den, this time for expanded forms .*)
fun com_den_exp(x::xs,denom,den,var)=
    let 
	val (t as Const ("HOL.divide",_) $ p1' $ p2')=x;
	val p2= sort (mv_geq LEX_) (the(expanded2poly p2' var));
	val p3= #1(mv_division(denom,p2,LEX_));
	val p1var=get_vars(p1');
    in     
	if length(xs)>0 then 
	    if p3=[(1,mv_null2(var))] then
		(
		 Const ("op +",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT)
		 $ 
		 (
		  Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
		  $ 
		  poly2expanded(the(expanded2poly p1' p1var),p1var)
		  $ 
		  den	
		  )    
		 $ 
		 #1(com_den_exp(xs,denom,den,var))
		,
		[]
		)
	    else
		(
		 Const ("op +",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
		 $ 
		 (
		  Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
		  $ 
		  (
		   Const ("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		   poly2expanded(the(expanded2poly p1' p1var),p1var) $ 
		   poly2expanded(p3,var)
		   ) 
		  $ 
		  (
		   den
		   ) 	
		  )
		 $ 
		 #1(com_den_exp(xs,denom,den,var))
		,
		[]
		)
	else
	    if p3=[(1,mv_null2(var))] then
		(
		 (
		  Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
		  $ 
		  poly2expanded(the(expanded2poly p1' p1var),p1var)
		  $ 
		  den	
		  )
		 ,
		 []
		 )
	     else
		 (
		  Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) 
		  $ 
		  (
		   Const ("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		   poly2expanded(the(expanded2poly p1' p1var),p1var) $ 
		   poly2expanded(p3,var)
		   ) 
		  $ 
		  den 	
		  ,
		  []
		  )
    end;

(* wird aktuell nicht mehr gebraucht, bei rcknderung schon 
-------------------------------------------------------------
(* WN0210???SK brauch ma des berhaupt *)
fun com_den2(x::xs,denom,den,var)=
    let 
	val (t as Const ("HOL.divide",_) $ p1' $ p2')=x;
	val p2= sort (mv_geq LEX_) (the(term2poly p2' var));
	val p3= #1(mv_division(denom,p2,LEX_));
	val p1var=get_vars(p1');
    in     
	if length(xs)>0 then 
	    if p3=[(1,mv_null2(var))] then
		(
		 Const ("op +",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		 poly2term(the(term2poly p1' p1var),p1var) $ 
		 com_den2(xs,denom,den,var)
		)
	    else
		(
		 Const ("op +",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		 (
		   let 
		       val p3'=poly2term(p3,var);
		       val vars= (((map free2str) o vars) p1') union (((map free2str) o vars) p3');
		   in
		       poly2term(sort (mv_geq LEX_) (mv_mul(the(term2poly p1' vars) ,the(term2poly p3' vars),LEX_)),vars)
		   end
		  ) $ 
		 com_den2(xs,denom,den,var)
		)
	else
	    if p3=[(1,mv_null2(var))] then
		(
		 poly2term(the(term2poly p1' p1var),p1var)
		 )
	     else
		 (
		   let 
		       val p3'=poly2term(p3,var);
		       val vars= (((map free2str) o vars) p1') union (((map free2str) o vars) p3');
		   in
		       poly2term(sort (mv_geq LEX_) (mv_mul(the(term2poly p1' vars) ,the(term2poly p3' vars),LEX_)),vars)
		   end
		  )
    end;

(* WN0210???SK brauch ma des berhaupt *)
fun com_den_exp2(x::xs,denom,den,var)=
    let 
	val (t as Const ("HOL.divide",_) $ p1' $ p2')=x;
	val p2= sort (mv_geq LEX_) (the(expanded2poly p2' var));
	val p3= #1(mv_division(denom,p2,LEX_));
	val p1var=get_vars p1';
    in     
	if length(xs)>0 then 
	    if p3=[(1,mv_null2(var))] then
		(
		 Const ("op +",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		 poly2expanded(the (expanded2poly p1' p1var),p1var) $ 
		 com_den_exp2(xs,denom,den,var)
		)
	    else
		(
		 Const ("op +",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
		 (
		   let 
		       val p3'=poly2expanded(p3,var);
		       val vars= (((map free2str) o vars) p1') union (((map free2str) o vars) p3');
		   in
		       poly2expanded(sort (mv_geq LEX_) (mv_mul(the(expanded2poly p1' vars) ,the(expanded2poly p3' vars),LEX_)),vars)
		   end
		  ) $ 
		 com_den_exp2(xs,denom,den,var)
		)
	else
	    if p3=[(1,mv_null2(var))] then
		(
		 poly2expanded(the (expanded2poly p1' p1var),p1var)
		 )
	     else
		 (
		   let 
		       val p3'=poly2expanded(p3,var);
		       val vars= (((map free2str) o vars) p1') union (((map free2str) o vars) p3');
		   in
		       poly2expanded(sort (mv_geq LEX_) (mv_mul(the(expanded2poly p1' vars) ,the(expanded2poly p3' vars),LEX_)),vars)
		   end
		  )
    end;
---------------------------------------------------------*)


(*. searches for an element y of a list ys, which has an gcd not 1 with x .*) 
fun exists_gcd (x,[]) = false 
  | exists_gcd (x,y::ys) = if mv_gcd x y = [(1,mv_null2(#2(hd(x))))] then  exists_gcd (x,ys)
			   else true;

(*. divides each element of the list xs with y .*)
fun list_div ([],y) = [] 
  | list_div (x::xs,y) = 
    let
	val (d,r)=mv_division(x,y,LEX_);
    in
	if r=[] then 
	    d::list_div(xs,y)
	else x::list_div(xs,y)
    end;
    
(*. checks if x is in the list ys .*)
fun in_list (x,[]) = false 
  | in_list (x,y::ys) = if x=y then true
			else in_list(x,ys);

(*. deletes all equal elements of the list xs .*)
fun kill_equal [] = [] 
  | kill_equal (x::xs) = if in_list(x,xs) orelse x=[(1,mv_null2(#2(hd(x))))] then kill_equal(xs)
			 else x::kill_equal(xs);

(*. searches for new factors .*)
fun new_factors [] = []
  | new_factors (list:mv_poly list):mv_poly list = 
    let
	val l = kill_equal list;
	val len = length(l);
    in
	if len>=2 then
	    (
	     let
		 val x::y::xs=l;
		 val gcd=mv_gcd x y;
	     in
		 if gcd=[(1,mv_null2(#2(hd(x))))] then 
		     ( 
		      if exists_gcd(x,xs) then new_factors (y::xs @ [x])
		      else x::new_factors(y::xs)
	             )
		 else gcd::new_factors(kill_equal(list_div(x::y::xs,gcd)))
	     end
	     )
	else
	    if len=1 then [hd(l)]
	    else []
    end;

(*. gets the factors of a list .*)
fun get_factors x = new_factors x; 

(*. multiplies the elements of the list .*)
fun multi_list [] = []
  | multi_list (x::xs) = if xs=[] then x
			 else mv_mul(x,multi_list xs,LEX_);

(*. makes a term out of the elements of the list (polynomial representation) .*)
fun make_term ([],vars) = Free(str_of_int 0,HOLogic.realT) 
  | make_term ((x::xs),vars) = if length(xs)=0 then poly2term(sort (mv_geq LEX_) (x),vars)
			       else
				   (
				    Const ("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
				    poly2term(sort (mv_geq LEX_) (x),vars) $ 
				    make_term(xs,vars)
				    );

(*. factorizes the denominator (polynomial representation) .*)				
fun factorize_den (l,den,vars) = 
    let
	val factor_list=kill_equal( (get_factors l));
	val mlist=multi_list(factor_list);
	val (last,rest)=mv_division(den,multi_list(factor_list),LEX_);
    in
	if rest=[] then
	    (
	     if last=[(1,mv_null2(vars))] then make_term(factor_list,vars)
	     else make_term(last::factor_list,vars)
	     )
	else raise error ("RATIONALS_FACTORIZE_DEN_EXCEPTION: Invalid factor by division")
    end; 

(*. makes a term out of the elements of the list (expanded polynomial representation) .*)
fun make_exp ([],vars) = Free(str_of_int 0,HOLogic.realT) 
  | make_exp ((x::xs),vars) = if length(xs)=0 then poly2expanded(sort (mv_geq LEX_) (x),vars)
			       else
				   (
				    Const ("op *",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
				    poly2expanded(sort (mv_geq LEX_) (x),vars) $ 
				    make_exp(xs,vars)
				    );

(*. factorizes the denominator (expanded polynomial representation) .*)	
fun factorize_den_exp (l,den,vars) = 
    let
	val factor_list=kill_equal( (get_factors l));
	val mlist=multi_list(factor_list);
	val (last,rest)=mv_division(den,multi_list(factor_list),LEX_);
    in
	if rest=[] then
	    (
	     if last=[(1,mv_null2(vars))] then make_exp(factor_list,vars)
	     else make_exp(last::factor_list,vars)
	     )
	else raise error ("RATIONALS_FACTORIZE_DEN_EXP_EXCEPTION: Invalid factor by division")
    end; 

(*. calculates the common denominator of all elements of the list and multiplies .*)
(*. the nominators and denominators with the correct factor .*)
(*. (polynomial representation) .*)
fun step_add_list_of_fractions []=(Free("0",HOLogic.realT),[]:term list)
  | step_add_list_of_fractions [x]= raise error ("RATIONALS_STEP_ADD_LIST_OF_FRACTIONS_EXCEPTION: Nothing to add")
  | step_add_list_of_fractions (xs) = 
    let
        val den_list=termlist2denominators (xs); (* list of denominators *)
	val (denom,var)=calc_lcm(den_list);      (* common denominator *)
	val den=factorize_den(#1(den_list),denom,var); (* faktorisierter Nenner !!! *)
    in
	com_den(xs,denom,den,var)
    end;

(*. calculates the common denominator of all elements of the list and multiplies .*)
(*. the nominators and denominators with the correct factor .*)
(*. (expanded polynomial representation) .*)
fun step_add_list_of_fractions_exp []  = (Free("0",HOLogic.realT),[]:term list)
  | step_add_list_of_fractions_exp [x] = raise error ("RATIONALS_STEP_ADD_LIST_OF_FRACTIONS_EXP_EXCEPTION: Nothing to add")
  | step_add_list_of_fractions_exp (xs)= 
    let
        val den_list=termlist2denominators_exp (xs); (* list of denominators *)
	val (denom,var)=calc_lcm(den_list);      (* common denominator *)
	val den=factorize_den_exp(#1(den_list),denom,var); (* faktorisierter Nenner !!! *)
    in
	com_den_exp(xs,denom,den,var)
    end;

(* wird aktuell nicht mehr gebraucht, bei rcknderung schon 
-------------------------------------------------------------
(* WN0210???SK brauch ma des berhaupt *)
fun step_add_list_of_fractions2 []=(Free("0",HOLogic.realT),[]:term list)
  | step_add_list_of_fractions2 [x]=(x,[])
  | step_add_list_of_fractions2 (xs) = 
    let
        val den_list=termlist2denominators (xs); (* list of denominators *)
	val (denom,var)=calc_lcm(den_list);      (* common denominator *)
	val den=factorize_den(#1(den_list),denom,var);  (* faktorisierter Nenner !!! *)
    in
	(
	 Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
	 com_den2(xs,denom, poly2term(denom,var)(*den*),var) $
	 poly2term(denom,var)
	,
	[]
	)
    end;

(* WN0210???SK brauch ma des berhaupt *)
fun step_add_list_of_fractions2_exp []=(Free("0",HOLogic.realT),[]:term list)
  | step_add_list_of_fractions2_exp [x]=(x,[])
  | step_add_list_of_fractions2_exp (xs) = 
    let
        val den_list=termlist2denominators_exp (xs); (* list of denominators *)
	val (denom,var)=calc_lcm(den_list);      (* common denominator *)
	val den=factorize_den_exp(#1(den_list),denom,var);  (* faktorisierter Nenner !!! *)
    in
	(
	 Const ("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
	 com_den_exp2(xs,denom, poly2term(denom,var)(*den*),var) $
	 poly2expanded(denom,var)
	,
	[]
	)
    end;
---------------------------------------------- *)


(*. converts a term, which contains severel terms seperated by +, into a list of these terms .*)
fun term2list (t as (Const("HOL.divide",_) $ _ $ _)) = [t]
  | term2list (t as (Const("Atools.pow",_) $ _ $ _)) = 
    [Const("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
	  t $ Free("1",HOLogic.realT)
     ]
  | term2list (t as (Free(_,_))) = 
    [Const("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
	  t $  Free("1",HOLogic.realT)
     ]
  | term2list (t as (Const("op *",_) $ _ $ _)) = 
    [Const("HOL.divide",[HOLogic.realT,HOLogic.realT]--->HOLogic.realT) $ 
	  t $ Free("1",HOLogic.realT)
     ]
  | term2list (Const("op +",_) $ t1 $ t2) = term2list(t1) @ term2list(t2)
  | term2list (Const("op -",_) $ t1 $ t2) = 
    raise error ("RATIONALS_TERM2LIST_EXCEPTION: - not implemented yet")
  | term2list _ = raise error ("RATIONALS_TERM2LIST_EXCEPTION: invalid term");

(*.factors out the gcd of nominator and denominator:
   a/b = (a' * gcd)/(b' * gcd),  a,b,gcd  are poly[2].*)
fun factout_p_  (thy:theory) t = Some (step_cancel t,[]:term list); 
fun factout_ (thy:theory) t = Some (step_cancel_expanded t,[]:term list); 

(*.cancels a single fraction with normalform [2]
   resulting in a canceled fraction [2], see factout_ .*)
fun cancel_p_ (thy:theory) t = (*WN.2.6.03 no rewrite -> None !*)
    (let val (t',asm) = direct_cancel(*_expanded ... corrected MG.21.8.03*) t
     in if t = t' then None else Some (t',asm) 
     end) handle _ => None;
(*.the same as above with normalform [3]
  val cancel_ :
      theory ->        (*10.02 unused                                    *)
      term -> 	       (*fraction in normalform [3]                      *)
      (term * 	       (*fraction in normalform [3]                      *)
       term list)      (*casual asumptions in normalform [3]             *)
	  option       (*None: the function is not applicable            *).*)
fun cancel_ (thy:theory) t = Some (direct_cancel_expanded t) handle _ => None;

(*.transforms sums of at least 2 fractions [3] to
   sums with the least common multiple as nominator.*)
fun common_nominator_p_ (thy:theory) t =
((*writeln("### common_nominator_p_ called");*)
    Some (step_add_list_of_fractions(term2list(t))) handle _ => None
);
fun common_nominator_ (thy:theory) t =
    Some (step_add_list_of_fractions_exp(term2list(t))) handle _ => None;

(*.add 2 or more fractions
val add_fraction_p_ :
      theory ->        (*10.02 unused                                    *)
      term -> 	       (*2 or more fractions with normalform [2]         *)
      (term * 	       (*one fraction with normalform [2]                *)
       term list)      (*casual assumptions in normalform [2] WN0210???SK  *)
	  option       (*None: the function is not applicable            *).*)
fun add_fraction_p_ (thy:theory) t = 
(writeln("### add_fraction_p_ called");
    (let val ts = term2list t
     in if 1 < length ts
	then Some (add_list_of_fractions ts)
	else None (*raise error ("RATIONALS_ADD_EXCEPTION: nothing to add")*)
     end) handle _ => None
);
(*.same as add_fraction_p_ but with normalform [3].*)
(*Some (step_add_list_of_fractions2(term2list(t))); *)
fun add_fraction_ (thy:theory) t = 
    if length(term2list(t))>1 
    then Some (add_list_of_fractions_exp(term2list(t))) handle _ => None
    else (*raise error ("RATIONALS_ADD_FRACTION_EXCEPTION: nothing to add")*)
	None;
fun add_fraction_ (thy:theory) t = 
    (if 1 < length (term2list t)
     then Some (add_list_of_fractions_exp (term2list t))
     else (*raise error ("RATIONALS_ADD_FRACTION_EXCEPTION: nothing to add")*)
	 None) handle _ => None;

(*Some (step_add_list_of_fractions2_exp(term2list(t))); *)

(*. brings the term into a normal form .*)
fun norm_rational_ (thy:theory) t = 
    Some (add_list_of_fractions(term2list(t))) handle _ => None; 
fun norm_expanded_rat_ (thy:theory) t = 
    Some (add_list_of_fractions_exp(term2list(t))) handle _ => None; 


(*.evaluates conditions in calculate_Rational.*)
(*make local with FIXX@ME result:term *term list*)
val calc_rat_erls = prep_rls(
  Rls {id = "calc_rat_erls", preconds = [], rew_ord = ("dummy_ord",dummy_ord), 
	 erls = e_rls, srls = Erls, calc = [], (*asm_thm = [], *)
	 rules = 
	 [Calc ("op =",eval_equal "#equal_"),
	  Calc ("Atools.is'_const",eval_const "#is_const_"),
	  Thm ("not_true",num_str not_true),
	  Thm ("not_false",num_str not_false)
	  ], 
	 scr = EmptyScr});


(*.simplifies expressions with numerals;
   does NOT rearrange the term by AC-rewriting; thus terms with variables 
   need to have constants to be commuted together respectively.*)
val calculate_Rational = prep_rls(
    merge_rls "calculate_Rational"
	(Rls {id = "divide", preconds = [], rew_ord = ("dummy_ord",dummy_ord), 
	      erls = calc_rat_erls, srls = Erls, (*asm_thm = [],*) 
	      calc = [], 
	      rules = 
	      [Calc ("HOL.divide"  ,eval_cancel "#divide_"),
	       
	       Thm ("sym_real_minus_divide_eq",
		    num_str (real_minus_divide_eq RS sym)),
	       (*SYM - ?x / ?y = - (?x / ?y)  may come from subst*)
	       
	       Thm ("rat_add",num_str rat_add),
	       (*"[| a is_const; b is_const; c is_const; d is_const |] ==> \
		 \"a / c + b / d = (a * d) / (c * d) + (b * c ) / (d * c)"*)
	       Thm ("rat_add1",num_str rat_add1),
	       (*"[| a is_const; b is_const; c is_const |] ==> \
		 \"a / c + b / c = (a + b) / c"*)
	       Thm ("rat_add2",num_str rat_add2),
	       (*"[| ?a is_const; ?b is_const; ?c is_const |] ==> \
		 \?a / ?c + ?b = (?a + ?b * ?c) / ?c"*)
	       Thm ("rat_add3",num_str rat_add3),
	       (*"[| a is_const; b is_const; c is_const |] ==> \
		 \"a + b / c = (a * c) / c + b / c"\
		 \.... is_const to be omitted here FIXME*)
	       
	       Thm ("rat_mult",num_str rat_mult),
	       (*a / b * (c / d) = a * c / (b * d)*)
	       Thm ("real_times_divide1_eq",num_str real_times_divide1_eq),
	       (*?x * (?y / ?z) = ?x * ?y / ?z*)
	       Thm ("real_times_divide2_eq",num_str real_times_divide2_eq),
	       (*?y / ?z * ?x = ?y * ?x / ?z*)
	       
	       Thm ("real_divide_divide1",num_str real_divide_divide1),
	       (*"?y ~= 0 ==> ?u / ?v / (?y / ?z) = ?u / ?v * (?z / ?y)"*)
	       Thm ("real_divide_divide2_eq",num_str real_divide_divide2_eq),
	       (*"?x / ?y / ?z = ?x / (?y * ?z)"*)
	       
	       Thm ("rat_power", num_str rat_power),
	       (*"(?a / ?b) ^^^ ?n = ?a ^^^ ?n / ?b ^^^ ?n"*)
	       
	       Thm ("mult_cross",num_str mult_cross),
	       (*"[| b ~= 0; d ~= 0 |] ==> (a / b = c / d) = (a * d = b * c)*)
	       Thm ("mult_cross1",num_str mult_cross1),
	       (*"   b ~= 0            ==> (a / b = c    ) = (a     = b * c)*)
	       Thm ("mult_cross2",num_str mult_cross2)
	       (*"           d ~= 0    ==> (a     = c / d) = (a * d =     c)*)
	       ], scr = EmptyScr})
	calculate_Poly);


(*("is_expanded", ("Rational.is'_expanded", eval_is_expanded ""))*)
fun eval_is_expanded (thmid:string) _ 
		       (t as (Const("Rational.is'_expanded", _) $ arg)) thy = 
    if is_expanded arg
    then Some (mk_thmid thmid "" 
			((string_of_cterm o cterm_of (sign_of thy)) arg) "", 
	       Trueprop $ (mk_equality (t, HOLogic.true_const)))
    else Some (mk_thmid thmid "" 
			((string_of_cterm o cterm_of (sign_of thy)) arg) "", 
	       Trueprop $ (mk_equality (t, HOLogic.false_const)))
  | eval_is_expanded _ _ _ _ = None; 

val rational_erls = 
    merge_rls "rational_erls" calculate_Rational 
	      (append_rls "is_expanded" Atools_erls 
			  [Calc ("Rational.is'_expanded", eval_is_expanded "")
			   ]);



(*.3 'reverse-rewrite-sets' for symbolic computation on rationals:
 =================================================================
 A[2] 'cancel_p': .
 A[3] 'cancel': .
 B[2] 'common_nominator_p': transforms summands in a term [2]
         to fractions with the (least) common multiple as nominator.
 B[3] 'norm_rational': normalizes arbitrary algebraic terms (without 
         radicals and transzendental functions) to one canceled fraction,
	 nominator and denominator in polynomial form.

In order to meet isac's requirements for interactive and stepwise calculation,
each 'reverse-rewerite-set' consists of an initialization for the interpreter 
state and of 4 functions, each of which employs rewriting as much as possible.
The signature of these functions are the same in each 'reverse-rewrite-set' 
respectively.*)

(* ************************************************************************* *)


local(*. cancel_p
------------------------
cancels a single fraction consisting of two (uni- or multivariate)
polynomials WN0609???SK[2] into another such a fraction; examples:

	   a^2 + -1*b^2         a + b
        -------------------- = ---------
	a^2 + -2*a*b + b^2     a + -1*b

        a^2    a
        --- = ---
         a     1

Remark: the reverse ruleset does _NOT_ work properly with other input !.*)
(*WN020824 wir werden "uberlegen, wie wir ungeeignete inputs zur"uckweisen*)

val {rules, rew_ord=(_,ro),...} =
    rep_rls (assoc_rls "make_polynomial");
(*WN060829 ... make_deriv does not terminate with 1st expl above,
           see rational.sml --- investigate rulesets for cancel_p ---*)
val {rules, rew_ord=(_,ro),...} =
    rep_rls (assoc_rls "rev_rew_p");

val thy = Rational.thy;

(*.init_state = fn : term -> istate
initialzies the state of the script interpreter. The state is:

type rrlsstate =      (*state for reverse rewriting*)
     (term *          (*the current formula*)
      term *          (*the final term*)
      rule list       (*'reverse rule list' (#)*)
	    list *    (*may be serveral, eg. in norm_rational*)
      (rule *         (*Thm (+ Thm generated from Calc) resulting in ...*)
       (term *        (*... rewrite with ...*)
	term list))   (*... assumptions*)
	  list);      (*derivation from given term to normalform
		       in reverse order with sym_thm;
                       (#) could be extracted from here by (map #1)*).*)
(* val {rules, rew_ord=(_,ro),...} =
       rep_rls (assoc_rls "rev_rew_p")        (*USE ALWAYS, SEE val cancel_p*);
   val (thy, eval_rls, ro) =(Rational.thy, Atools_erls, ro) (*..val cancel_p*);
   val t = t;
   *)
fun init_state thy eval_rls ro t =
    let val Some (t',_) = factout_p_ thy t
        val Some (t'',asm) = cancel_p_ thy t
        val der = reverse_deriv thy eval_rls rules ro None t'
        val der = der @ [(Thm ("real_mult_div_cancel2",
			       num_str real_mult_div_cancel2),
			  (t'',asm))]
        val rs = (distinct_Thm o (map #1)) der
	val rs = filter_out (eq_Thms ["sym_real_add_zero_left",
				      "sym_real_mult_0",
				      "sym_real_mult_1"
				      (*..insufficient,eg.make_Polynomial*)])rs
    in (t,t'',[rs(*here only _ONE_ to ease locate_rule*)],der) end;

(*.locate_rule = fn : rule list -> term -> rule
		      -> (rule * (term * term list) option) list.
  checks a rule R for being a cancel-rule, and if it is,
  then return the list of rules (+ the terms they are rewriting to)
  which need to be applied before R should be applied.
  precondition: the rule is applicable to the argument-term.
arguments:
  rule list: the reverse rule list
  -> term  : ... to which the rule shall be applied
  -> rule  : ... to be applied to term
value:
  -> (rule           : a rule rewriting to ...
      * (term        : ... the resulting term ...
         * term list): ... with the assumptions ( //#0).
      ) list         : there may be several such rules;
		       the list is empty, if the rule has nothing to do
		       with cancelation.*)
(* val () = ();
   *)
fun locate_rule thy eval_rls ro [rs] t r =
    if (id_of_thm r) mem (map (id_of_thm)) rs
    then let val ropt =
		 rewrite_ thy ro eval_rls true (thm_of_thm r) t;
	 in case ropt of
		Some ta => [(r, ta)]
	      | None => (writeln("### locate_rule:  rewrite "^
				 (id_of_thm r)^" "^(term2str t)^" = None");
			 []) end
    else (writeln("### locate_rule:  "^(id_of_thm r)^" not mem rrls");[])
  | locate_rule _ _ _ _ _ _ =
    raise error ("locate_rule: doesnt match rev-sets in istate");

(*.next_rule = fn : rule list -> term -> rule option
  for a given term return the next rules to be done for cancelling.
arguments:
  rule list     : the reverse rule list
  term          : the term for which ...
value:
  -> rule option: ... this rule is appropriate for cancellation;
		  there may be no such rule (if the term is canceled already.*)
(* val thy = Rational.thy;
   val Rrls {rew_ord=(_,ro),...} = cancel;
   val ([rs],t) = (rss,f);
   next_rule thy eval_rls ro [rs] t;(*eval fun next_rule ... before!*)

   val (thy, [rs]) = (Rational.thy, revsets);
   val Rrls {rew_ord=(_,ro),...} = cancel;
   nex [rs] t;
   *)
fun next_rule thy eval_rls ro [rs] t =
    let val der = make_deriv thy eval_rls rs ro None t;
    in case der of
(* val (_,r,_)::_ = der;
   *)
	   (_,r,_)::_ => Some r
	 | _ => None
    end
  | next_rule _ _ _ _ _ =
    raise error ("next_rule: doesnt match rev-sets in istate");

(*.val attach_form = f : rule list -> term -> term
			 -> (rule * (term * term list)) list
  checks an input term TI, if it may belong to a current cancellation, by
  trying to derive it from the given term TG.
arguments:
  term   : TG, the last one in the cancellation agreed upon by user + math-eng
  -> term: TI, the next one input by the user
value:
  -> (rule           : the rule to be applied in order to reach TI
      * (term        : ... obtained by applying the rule ...
         * term list): ... and the respective assumptions.
      ) list         : there may be several such rules;
                       the list is empty, if the users term does not belong
		       to a cancellation of the term last agreed upon.*)
fun attach_form (_:rule list list) (_:term) (_:term) = (*still missing*)
    []:(rule * (term * term list)) list;

in

val cancel_p =
    Rrls {id = "cancel_p", prepat=[],
	  rew_ord=("ord_make_polynomial",
		   ord_make_polynomial false Rational.thy),
	  erls = rational_erls,
	  calc = [("plus"    ,("op +"        ,eval_binop "#add_")),
		  ("times"   ,("op *"        ,eval_binop "#mult_")),
		  ("divide_" ,("HOL.divide"  ,eval_cancel "#divide_")),
		  ("power_"  ,("Atools.pow"  ,eval_binop "#power_"))],
	  (*asm_thm=[("real_mult_div_cancel2","")],*)
	  scr=Rfuns {init_state  = init_state thy Atools_erls ro,
		     normal_form = cancel_p_ thy,
		     locate_rule = locate_rule thy Atools_erls ro,
		     next_rule   = next_rule thy Atools_erls ro,
		     attach_form = attach_form}}
end;(*local*)


local(*.ad (1) 'cancel'
------------------------------
cancels a single fraction consisting of two (uni- or multivariate)
polynomials WN0609???SK[3] into another such a fraction; examples:

	   a^2 - b^2           a + b
        -------------------- = ---------
	a^2 - 2*a*b + b^2      a - *b

Remark: the reverse ruleset does _NOT_ work properly with other input !.*)
(*WN 24.8.02: wir werden "uberlegen, wie wir ungeeignete inputs zur"uckweisen*)

(*
val Some (Rls {rules=rules,rew_ord=(_,ro),...}) = 
    assoc'(!ruleset',"expand_binoms");
*)
val {rules=rules,rew_ord=(_,ro),...} =
    rep_rls (assoc_rls "expand_binoms");
val thy = Rational.thy;

fun init_state thy eval_rls ro t =
    let val Some (t',_) = factout_ thy t;
        val Some (t'',asm) = cancel_ thy t;
        val der = reverse_deriv thy eval_rls rules ro None t';
        val der = der @ [(Thm ("real_mult_div_cancel2",
			       num_str real_mult_div_cancel2),
			  (t'',asm))]
        val rs = map #1 der;
    in (t,t'',[rs],der) end;

fun locate_rule thy eval_rls ro [rs] t r =
    if (id_of_thm r) mem (map (id_of_thm)) rs
    then let val ropt = 
		 rewrite_ thy ro eval_rls true (thm_of_thm r) t;
	 in case ropt of
		Some ta => [(r, ta)]
	      | None => (writeln("### locate_rule:  rewrite "^
				 (id_of_thm r)^" "^(term2str t)^" = None");
			 []) end
    else (writeln("### locate_rule:  "^(id_of_thm r)^" not mem rrls");[])
  | locate_rule _ _ _ _ _ _ = 
    raise error ("locate_rule: doesnt match rev-sets in istate");

fun next_rule thy eval_rls ro [rs] t =
    let val der = make_deriv thy eval_rls rs ro None t;
    in case der of 
(* val (_,r,_)::_ = der;
   *)
	   (_,r,_)::_ => Some r
	 | _ => None
    end
  | next_rule _ _ _ _ _ = 
    raise error ("next_rule: doesnt match rev-sets in istate");

fun attach_form (_:rule list list) (_:term) (_:term) = (*still missing*)
    []:(rule * (term * term list)) list;

val pat = (term_of o the o (parse thy)) "?r/?s";
val pre1 = (term_of o the o (parse thy)) "r is_expanded";
val pre2 = (term_of o the o (parse thy)) "s is_expanded";
val prepat = [([pre1, pre2], pat)];

in


val cancel = 
    Rrls {id = "cancel", prepat=prepat,
	  rew_ord=("ord_make_polynomial",
		   ord_make_polynomial false Rational.thy),
	  erls = rational_erls, 
	  calc = [("plus"    ,("op +"        ,eval_binop "#add_")),
		  ("times"   ,("op *"        ,eval_binop "#mult_")),
		  ("divide_" ,("HOL.divide"  ,eval_cancel "#divide_")),
		  ("power_"  ,("Atools.pow"  ,eval_binop "#power_"))],
	  scr=Rfuns {init_state  = init_state thy Atools_erls ro,
		     normal_form = cancel_ thy, 
		     locate_rule = locate_rule thy Atools_erls ro,
		     next_rule   = next_rule thy Atools_erls ro,
		     attach_form = attach_form}}
end;(*local*)



local(*.ad [2] 'common_nominator_p'
---------------------------------
FIXME Beschreibung .*)


val {rules=rules,rew_ord=(_,ro),...} =
    rep_rls (assoc_rls "make_polynomial");
(*WN060829 ... make_deriv does not terminate with 1st expl above,
           see rational.sml --- investigate rulesets for cancel_p ---*)
val {rules, rew_ord=(_,ro),...} =
    rep_rls (assoc_rls "rev_rew_p");
val thy = Rational.thy;


(*.common_nominator_p_ = fn : theory -> term -> (term * term list) option
  as defined above*)

(*.init_state = fn : term -> istate
initialzies the state of the interactive interpreter. The state is:

type rrlsstate =      (*state for reverse rewriting*)
     (term *          (*the current formula*)
      term *          (*the final term*)
      rule list       (*'reverse rule list' (#)*)
	    list *    (*may be serveral, eg. in norm_rational*)
      (rule *         (*Thm (+ Thm generated from Calc) resulting in ...*)
       (term *        (*... rewrite with ...*)
	term list))   (*... assumptions*)
	  list);      (*derivation from given term to normalform
		       in reverse order with sym_thm;
                       (#) could be extracted from here by (map #1)*).*)
fun init_state thy eval_rls ro t =
    let val Some (t',_) = common_nominator_p_ thy t;
        val Some (t'',asm) = add_fraction_p_ thy t;
        val der = reverse_deriv thy eval_rls rules ro None t';
        val der = der @ [(Thm ("real_mult_div_cancel2",
			       num_str real_mult_div_cancel2),
			  (t'',asm))]
        val rs = (distinct_Thm o (map #1)) der;
	val rs = filter_out (eq_Thms ["sym_real_add_zero_left",
				      "sym_real_mult_0",
				      "sym_real_mult_1"]) rs;
    in (t,t'',[rs(*here only _ONE_*)],der) end;

(* use"knowledge/Rational.ML";
   *)

(*.locate_rule = fn : rule list -> term -> rule
		      -> (rule * (term * term list) option) list.
  checks a rule R for being a cancel-rule, and if it is,
  then return the list of rules (+ the terms they are rewriting to)
  which need to be applied before R should be applied.
  precondition: the rule is applicable to the argument-term.
arguments:
  rule list: the reverse rule list
  -> term  : ... to which the rule shall be applied
  -> rule  : ... to be applied to term
value:
  -> (rule           : a rule rewriting to ...
      * (term        : ... the resulting term ...
         * term list): ... with the assumptions ( //#0).
      ) list         : there may be several such rules;
		       the list is empty, if the rule has nothing to do
		       with cancelation.*)
(* val () = ();
   *)
fun locate_rule thy eval_rls ro [rs] t r =
    if (id_of_thm r) mem (map (id_of_thm)) rs
    then let val ropt =
		 rewrite_ thy ro eval_rls true (thm_of_thm r) t;
	 in case ropt of
		Some ta => [(r, ta)]
	      | None => (writeln("### locate_rule:  rewrite "^
				 (id_of_thm r)^" "^(term2str t)^" = None");
			 []) end
    else (writeln("### locate_rule:  "^(id_of_thm r)^" not mem rrls");[])
  | locate_rule _ _ _ _ _ _ =
    raise error ("locate_rule: doesnt match rev-sets in istate");

(*.next_rule = fn : rule list -> term -> rule option
  for a given term return the next rules to be done for cancelling.
arguments:
  rule list     : the reverse rule list
  term          : the term for which ...
value:
  -> rule option: ... this rule is appropriate for cancellation;
		  there may be no such rule (if the term is canceled already.*)
(* val thy = Rational.thy;
   val Rrls {rew_ord=(_,ro),...} = cancel;
   val ([rs],t) = (rss,f);
   next_rule thy eval_rls ro [rs] t;(*eval fun next_rule ... before!*)

   val (thy, [rs]) = (Rational.thy, revsets);
   val Rrls {rew_ord=(_,ro),...} = cancel;
   nex [rs] t;
   *)
fun next_rule thy eval_rls ro [rs] t =
    let val der = make_deriv thy eval_rls rs ro None t;
    in case der of
(* val (_,r,_)::_ = der;
   *)
	   (_,r,_)::_ => Some r
	 | _ => None
    end
  | next_rule _ _ _ _ _ =
    raise error ("next_rule: doesnt match rev-sets in istate");

(*.val attach_form = f : rule list -> term -> term
			 -> (rule * (term * term list)) list
  checks an input term TI, if it may belong to a current cancellation, by
  trying to derive it from the given term TG.
arguments:
  term   : TG, the last one in the cancellation agreed upon by user + math-eng
  -> term: TI, the next one input by the user
value:
  -> (rule           : the rule to be applied in order to reach TI
      * (term        : ... obtained by applying the rule ...
         * term list): ... and the respective assumptions.
      ) list         : there may be several such rules;
                       the list is empty, if the users term does not belong
		       to a cancellation of the term last agreed upon.*)
fun attach_form (_:rule list list) (_:term) (_:term) = (*still missing*)
    []:(rule * (term * term list)) list;

val pat0 = (term_of o the o (parse thy)) "?r/?s+?u/?v";
val pat1 = (term_of o the o (parse thy)) "?r/?s+?u   ";
val pat2 = (term_of o the o (parse thy)) "?r   +?u/?v";
val prepat = [([HOLogic.true_const], pat0),
	      ([HOLogic.true_const], pat1),
	      ([HOLogic.true_const], pat2)];

in

(*11.02 schnelle L"osung f"ur RL: Bruch auch gek"urzt;
  besser w"are: auf 1 gemeinsamen Bruchstrich, Nenner und Z"ahler unvereinfacht
  dh. wie common_nominator_p_, aber auf 1 Bruchstrich*)
val common_nominator_p =
    Rrls {id = "common_nominator_p", prepat=prepat,
	  rew_ord=("ord_make_polynomial",
		   ord_make_polynomial false Rational.thy),
	  erls = rational_erls,
	  calc = [("plus"    ,("op +"        ,eval_binop "#add_")),
		  ("times"   ,("op *"        ,eval_binop "#mult_")),
		  ("divide_" ,("HOL.divide"  ,eval_cancel "#divide_")),
		  ("power_"  ,("Atools.pow"  ,eval_binop "#power_"))],
	  scr=Rfuns {init_state  = init_state thy Atools_erls ro,
		     normal_form = add_fraction_p_ thy,(*FIXME.WN0211*)
		     locate_rule = locate_rule thy Atools_erls ro,
		     next_rule   = next_rule thy Atools_erls ro,
		     attach_form = attach_form}}
end;(*local*)


local(*.ad [2] 'common_nominator'
---------------------------------
FIXME Beschreibung .*)


val {rules=rules,rew_ord=(_,ro),...} =
    rep_rls (assoc_rls "make_polynomial");
val thy = Rational.thy;


(*.common_nominator_ = fn : theory -> term -> (term * term list) option
  as defined above*)

(*.init_state = fn : term -> istate
initialzies the state of the interactive interpreter. The state is:

type rrlsstate =      (*state for reverse rewriting*)
     (term *          (*the current formula*)
      term *          (*the final term*)
      rule list       (*'reverse rule list' (#)*)
	    list *    (*may be serveral, eg. in norm_rational*)
      (rule *         (*Thm (+ Thm generated from Calc) resulting in ...*)
       (term *        (*... rewrite with ...*)
	term list))   (*... assumptions*)
	  list);      (*derivation from given term to normalform
		       in reverse order with sym_thm;
                       (#) could be extracted from here by (map #1)*).*)
fun init_state thy eval_rls ro t =
    let val Some (t',_) = common_nominator_ thy t;
        val Some (t'',asm) = add_fraction_ thy t;
        val der = reverse_deriv thy eval_rls rules ro None t';
        val der = der @ [(Thm ("real_mult_div_cancel2",
			       num_str real_mult_div_cancel2),
			  (t'',asm))]
        val rs = (distinct_Thm o (map #1)) der;
	val rs = filter_out (eq_Thms ["sym_real_add_zero_left",
				      "sym_real_mult_0",
				      "sym_real_mult_1"]) rs;
    in (t,t'',[rs(*here only _ONE_*)],der) end;

(* use"knowledge/Rational.ML";
   *)

(*.locate_rule = fn : rule list -> term -> rule
		      -> (rule * (term * term list) option) list.
  checks a rule R for being a cancel-rule, and if it is,
  then return the list of rules (+ the terms they are rewriting to)
  which need to be applied before R should be applied.
  precondition: the rule is applicable to the argument-term.
arguments:
  rule list: the reverse rule list
  -> term  : ... to which the rule shall be applied
  -> rule  : ... to be applied to term
value:
  -> (rule           : a rule rewriting to ...
      * (term        : ... the resulting term ...
         * term list): ... with the assumptions ( //#0).
      ) list         : there may be several such rules;
		       the list is empty, if the rule has nothing to do
		       with cancelation.*)
(* val () = ();
   *)
fun locate_rule thy eval_rls ro [rs] t r =
    if (id_of_thm r) mem (map (id_of_thm)) rs
    then let val ropt =
		 rewrite_ thy ro eval_rls true (thm_of_thm r) t;
	 in case ropt of
		Some ta => [(r, ta)]
	      | None => (writeln("### locate_rule:  rewrite "^
				 (id_of_thm r)^" "^(term2str t)^" = None");
			 []) end
    else (writeln("### locate_rule:  "^(id_of_thm r)^" not mem rrls");[])
  | locate_rule _ _ _ _ _ _ =
    raise error ("locate_rule: doesnt match rev-sets in istate");

(*.next_rule = fn : rule list -> term -> rule option
  for a given term return the next rules to be done for cancelling.
arguments:
  rule list     : the reverse rule list
  term          : the term for which ...
value:
  -> rule option: ... this rule is appropriate for cancellation;
		  there may be no such rule (if the term is canceled already.*)
(* val thy = Rational.thy;
   val Rrls {rew_ord=(_,ro),...} = cancel;
   val ([rs],t) = (rss,f);
   next_rule thy eval_rls ro [rs] t;(*eval fun next_rule ... before!*)

   val (thy, [rs]) = (Rational.thy, revsets);
   val Rrls {rew_ord=(_,ro),...} = cancel_p;
   nex [rs] t;
   *)
fun next_rule thy eval_rls ro [rs] t =
    let val der = make_deriv thy eval_rls rs ro None t;
    in case der of
(* val (_,r,_)::_ = der;
   *)
	   (_,r,_)::_ => Some r
	 | _ => None
    end
  | next_rule _ _ _ _ _ =
    raise error ("next_rule: doesnt match rev-sets in istate");

(*.val attach_form = f : rule list -> term -> term
			 -> (rule * (term * term list)) list
  checks an input term TI, if it may belong to a current cancellation, by
  trying to derive it from the given term TG.
arguments:
  term   : TG, the last one in the cancellation agreed upon by user + math-eng
  -> term: TI, the next one input by the user
value:
  -> (rule           : the rule to be applied in order to reach TI
      * (term        : ... obtained by applying the rule ...
         * term list): ... and the respective assumptions.
      ) list         : there may be several such rules;
                       the list is empty, if the users term does not belong
		       to a cancellation of the term last agreed upon.*)
fun attach_form (_:rule list list) (_:term) (_:term) = (*still missing*)
    []:(rule * (term * term list)) list;

val pat0 =  (term_of o the o (parse thy)) "?r/?s+?u/?v";
val pat01 = (term_of o the o (parse thy)) "?r/?s-?u/?v";
val pat1 =  (term_of o the o (parse thy)) "?r/?s+?u   ";
val pat11 = (term_of o the o (parse thy)) "?r/?s-?u   ";
val pat2 =  (term_of o the o (parse thy)) "?r   +?u/?v";
val pat21 = (term_of o the o (parse thy)) "?r   -?u/?v";
val prepat = [([HOLogic.true_const], pat0),
	      ([HOLogic.true_const], pat01),
	      ([HOLogic.true_const], pat1),
	      ([HOLogic.true_const], pat11),
	      ([HOLogic.true_const], pat2),
	      ([HOLogic.true_const], pat21)];


in

val common_nominator =
    Rrls {id = "common_nominator", prepat=prepat,
	  rew_ord=("ord_make_polynomial",
		   ord_make_polynomial false Rational.thy),
	  erls = rational_erls,
	  calc = [("plus"    ,("op +"        ,eval_binop "#add_")),
		  ("times"   ,("op *"        ,eval_binop "#mult_")),
		  ("divide_" ,("HOL.divide"  ,eval_cancel "#divide_")),
		  ("power_"  ,("Atools.pow"  ,eval_binop "#power_"))],
	  (*asm_thm=[("real_mult_div_cancel2","")],*)
	  scr=Rfuns {init_state  = init_state thy Atools_erls ro,
		     normal_form = add_fraction_ (*NOT common_nominator_*) thy,
		     locate_rule = locate_rule thy Atools_erls ro,
		     next_rule   = next_rule thy Atools_erls ro,
		     attach_form = attach_form}}

end;(*local*)


(*##*)
end;(*struct*)

open RationalI;
(*##*)

(*.the expression contains + - * ^ / only ?.*)
fun is_ratpolyexp (Free _) = true
  | is_ratpolyexp (Const ("op +",_) $ Free _ $ Free _) = true
  | is_ratpolyexp (Const ("op -",_) $ Free _ $ Free _) = true
  | is_ratpolyexp (Const ("op *",_) $ Free _ $ Free _) = true
  | is_ratpolyexp (Const ("Atools.pow",_) $ Free _ $ Free _) = true
  | is_ratpolyexp (Const ("HOL.divide",_) $ Free _ $ Free _) = true
  | is_ratpolyexp (Const ("op +",_) $ t1 $ t2) = 
               ((is_ratpolyexp t1) andalso (is_ratpolyexp t2))
  | is_ratpolyexp (Const ("op -",_) $ t1 $ t2) = 
               ((is_ratpolyexp t1) andalso (is_ratpolyexp t2))
  | is_ratpolyexp (Const ("op *",_) $ t1 $ t2) = 
               ((is_ratpolyexp t1) andalso (is_ratpolyexp t2))
  | is_ratpolyexp (Const ("Atools.pow",_) $ t1 $ t2) = 
               ((is_ratpolyexp t1) andalso (is_ratpolyexp t2))
  | is_ratpolyexp (Const ("HOL.divide",_) $ t1 $ t2) = 
               ((is_ratpolyexp t1) andalso (is_ratpolyexp t2))
  | is_ratpolyexp _ = false;

(*("is_ratpolyexp", ("Rational.is'_ratpolyexp", eval_is_ratpolyexp ""))*)
fun eval_is_ratpolyexp (thmid:string) _ 
		       (t as (Const("Rational.is'_ratpolyexp", _) $ arg)) thy =
    if is_ratpolyexp arg
    then Some (mk_thmid thmid "" 
			((string_of_cterm o cterm_of (sign_of thy)) arg) "", 
	       Trueprop $ (mk_equality (t, HOLogic.true_const)))
    else Some (mk_thmid thmid "" 
			((string_of_cterm o cterm_of (sign_of thy)) arg) "", 
	       Trueprop $ (mk_equality (t, HOLogic.false_const)))
  | eval_is_ratpolyexp _ _ _ _ = None; 



(*-------------------18.3.03 --> struct <-----------vvv--*)
val add_fractions_p = common_nominator_p; (*FIXXXME:eilig f"ur norm_Rational*)

(*.discard binary minus, shift unary minus into -1*; 
   unary minus before numerals are put into the numeral by parsing;
   contains absolute minimum of thms for context in norm_Rational .*)
val discard_minus = prep_rls(
  Rls {id = "discard_minus", preconds = [], rew_ord = ("dummy_ord",dummy_ord), 
      erls = e_rls, srls = Erls, calc = [], (*asm_thm = [],*)
      rules = [Thm ("real_diff_minus", num_str real_diff_minus),
	       (*"a - b = a + -1 * b"*)
	       Thm ("sym_real_mult_minus1",num_str (real_mult_minus1 RS sym))
	       (*- ?z = "-1 * ?z"*)
	       ],
      scr = Script ((term_of o the o (parse thy)) 
      "empty_script")
      }):rls;
(*erls for calculate_Rational; make local with FIXX@ME result:term *term list*)
val powers_erls = prep_rls(
  Rls {id = "powers_erls", preconds = [], rew_ord = ("dummy_ord",dummy_ord), 
      erls = e_rls, srls = Erls, calc = [], (*asm_thm = [],*)
      rules = [Calc ("Atools.is'_atom",eval_is_atom "#is_atom_"),
	       Calc ("Atools.is'_even",eval_is_even "#is_even_"),
	       Calc ("op <",eval_equ "#less_"),
	       Thm ("not_false", not_false),
	       Thm ("not_true", not_true),
	       Calc ("op +",eval_binop "#add_")
	       ],
      scr = Script ((term_of o the o (parse thy)) 
      "empty_script")
      }:rls);
(*.all powers over + distributed; atoms over * collected, other distributed
   contains absolute minimum of thms for context in norm_Rational .*)
val powers = prep_rls(
  Rls {id = "powers", preconds = [], rew_ord = ("dummy_ord",dummy_ord), 
      erls = powers_erls, srls = Erls, calc = [], (*asm_thm = [],*)
      rules = [Thm ("realpow_multI", num_str realpow_multI),
	       (*"(r * s) ^^^ n = r ^^^ n * s ^^^ n"*)
	       Thm ("realpow_pow",num_str realpow_pow),
	       (*"(a ^^^ b) ^^^ c = a ^^^ (b * c)"*)
	       Thm ("realpow_oneI",num_str realpow_oneI),
	       (*"r ^^^ 1 = r"*)
	       Thm ("realpow_minus_even",num_str realpow_minus_even),
	       (*"n is_even ==> (- r) ^^^ n = r ^^^ n" ?-->discard_minus?*)
	       Thm ("realpow_minus_odd",num_str realpow_minus_odd),
	       (*"Not (n is_even) ==> (- r) ^^^ n = -1 * r ^^^ n"*)
	       
	       (*----- collect atoms over * -----*)
	       Thm ("realpow_two_atom",num_str realpow_two_atom),	
	       (*"r is_atom ==> r * r = r ^^^ 2"*)
	       Thm ("realpow_plus_1",num_str realpow_plus_1),		
	       (*"r is_atom ==> r * r ^^^ n = r ^^^ (n + 1)"*)
	       Thm ("realpow_addI_atom",num_str realpow_addI_atom),
	       (*"r is_atom ==> r ^^^ n * r ^^^ m = r ^^^ (n + m)"*)

	       (*----- distribute none-atoms -----*)
	       Thm ("realpow_def_atom",num_str realpow_def_atom),
	       (*"[| 1 < n; not(r is_atom) |]==>r ^^^ n = r * r ^^^ (n + -1)"*)
	       Thm ("realpow_eq_oneI",num_str realpow_eq_oneI),
	       (*"1 ^^^ n = 1"*)
	       Calc ("op +",eval_binop "#add_")
	       ],
      scr = Script ((term_of o the o (parse thy)) 
      "empty_script")
      }:rls);
(*.contains absolute minimum of thms for context in norm_Rational.*)
val rat_mult_divide = prep_rls(
  Rls {id = "rat_mult_divide", preconds = [], 
       rew_ord = ("dummy_ord",dummy_ord), 
      erls = e_rls, srls = Erls, calc = [], (*asm_thm = [],*)
      rules = [Thm ("rat_mult",num_str rat_mult),
	       (*(1)"?a / ?b * (?c / ?d) = ?a * ?c / (?b * ?d)"*)
	       Thm ("real_times_divide1_eq",num_str real_times_divide1_eq),
	       (*(2)"?a * (?c / ?d) = ?a * ?c / ?d" must be [2],
	       otherwise inv.to a / b / c = ...*)
	       Thm ("real_times_divide2_eq",num_str real_times_divide2_eq),
	       (*"?a / ?b * ?c = ?a * ?c / ?b" order weights x^^^n too much
		     and does not commute a / b * c ^^^ 2 !*)
	       
	       Thm ("real_divide_divide1_eq", real_divide_divide1_eq),
	       (*"?x / (?y / ?z) = ?x * ?z / ?y"*)
	       Thm ("real_divide_divide2_eq", real_divide_divide2_eq),
	       (*"?x / ?y / ?z = ?x / (?y * ?z)"*)
	       Calc ("HOL.divide"  ,eval_cancel "#divide_")
	       ],
      scr = Script ((term_of o the o (parse thy)) "empty_script")
      }:rls);
(*.contains absolute minimum of thms for context in norm_Rational.*)
val reduce_0_1_2 = prep_rls(
  Rls{id = "reduce_0_1_2", preconds = [], rew_ord = ("dummy_ord", dummy_ord),
      erls = e_rls,srls = Erls,calc = [],(*asm_thm = [],*)
      rules = [(*Thm ("real_divide_1",num_str real_divide_1),
		 "?x / 1 = ?x" unnecess.for normalform*)
	       Thm ("real_mult_1",num_str real_mult_1),                 
	       (*"1 * z = z"*)
	       (*Thm ("real_mult_minus1",num_str real_mult_minus1),
	       "-1 * z = - z"*)
	       (*Thm ("real_minus_mult_cancel",num_str real_minus_mult_cancel),
	       "- ?x * - ?y = ?x * ?y"*)

	       Thm ("real_mult_0",num_str real_mult_0),        
	       (*"0 * z = 0"*)
	       Thm ("real_add_zero_left",num_str real_add_zero_left),
	       (*"0 + z = z"*)
	       (*Thm ("real_add_minus",num_str real_add_minus),
	       "?z + - ?z = 0"*)

	       Thm ("sym_real_mult_2",num_str (real_mult_2 RS sym)),	
	       (*"z1 + z1 = 2 * z1"*)
	       Thm ("real_mult_2_assoc",num_str real_mult_2_assoc),
	       (*"z1 + (z1 + k) = 2 * z1 + k"*)

	       Thm ("real_0_divide",num_str real_0_divide)
	       (*"0 / ?x = 0"*)
	       ], scr = EmptyScr}:rls);

(*erls for calculate_Rational; 
  make local with FIXX@ME result:term *term list WN0609???SKMG*)
val norm_rat_erls = prep_rls(
  Rls {id = "norm_rat_erls", preconds = [], rew_ord = ("dummy_ord",dummy_ord), 
      erls = e_rls, srls = Erls, calc = [], (*asm_thm = [],*)
      rules = [Calc ("Atools.is'_const",eval_const "#is_const_")
	       ],
      scr = Script ((term_of o the o (parse thy)) 
      "empty_script")
      }:rls);
(*.consists of rls containing the absolute minimum of thms.*)
(*040209: this version has been used by RL for his equations,
which is now replaced by MGs version below
vvv OLD VERSION !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*)
val norm_Rational = prep_rls(
  Rls {id = "norm_Rational", preconds = [], rew_ord = ("dummy_ord",dummy_ord), 
      erls = norm_rat_erls, srls = Erls, calc = [], (*asm_thm = [],*)
      rules = [(*sequence given by operator precedence*)
	       Rls_ discard_minus,
	       Rls_ powers,
	       Rls_ rat_mult_divide,
	       Rls_ expand,
	       Rls_ reduce_0_1_2,
	       (*^^^^^^^^^ from RL -- not the latest one vvvvvvvvv*)
	       Rls_ order_add_mult,
	       Rls_ collect_numerals,
	       Rls_ add_fractions_p,
	       Rls_ cancel_p
	       ],
      scr = Script ((term_of o the o (parse thy)) 
      "empty_script")
      }:rls);
val norm_Rational_parenthesized = prep_rls(
  Seq {id = "norm_Rational_parenthesized", preconds = []:term list, 
       rew_ord = ("dummy_ord", dummy_ord),
      erls = Atools_erls, srls = Erls,
      calc = [], (*asm_thm = [],*)
      rules = [Rls_  norm_Rational, (*from RL -- not the latest one*)
	       Rls_ discard_parentheses
	       ],
      scr = EmptyScr
      }:rls);      


(*-------------------18.3.03 --> struct <-----------^^^--*)



theory' := overwritel (!theory', [("Rational.thy",Rational.thy)]);


(*WN030318???SK: simplifies all but cancel and common_nominator*)
val simplify_rational = 
    merge_rls "simplify_rational" expand_binoms
    (append_rls "divide" calculate_Rational
		[Thm ("real_divide_1",num_str real_divide_1),
		 (*"?x / 1 = ?x"*)
		 Thm ("rat_mult",num_str rat_mult),
		 (*(1)"?a / ?b * (?c / ?d) = ?a * ?c / (?b * ?d)"*)
		 Thm ("real_times_divide1_eq",num_str real_times_divide1_eq),
		 (*(2)"?a * (?c / ?d) = ?a * ?c / ?d" must be [2],
		 otherwise inv.to a / b / c = ...*)
		 Thm ("real_times_divide2_eq",num_str real_times_divide2_eq),
		 (*"?a / ?b * ?c = ?a * ?c / ?b"*)
		 Thm ("add_minus",num_str add_minus),
		 (*"?a + ?b - ?b = ?a"*)
		 Thm ("add_minus1",num_str add_minus1),
		 (*"?a - ?b + ?b = ?a"*)
		 Thm ("real_divide_minus1",num_str real_divide_minus1)
		 (*"?x / -1 = - ?x"*)
(*
,
		 Thm ("",num_str )
*)
		 ]);

(*---------vvv-------------MG ab 1.07.2003--------------vvv-----------*)

(* ------------------------------------------------------------------ *)
(*                  Simplifier fr beliebige Buchterme                *) 
(* ------------------------------------------------------------------ *)
(*----------------------- norm_Rational_mg ---------------------------*)
(*. description of the simplifier see MG-DA.p.56ff .*)
(* ------------------------------------------------------------------- *)
val common_nominator_p_rls = prep_rls(
  Rls {id = "common_nominator_p_rls", preconds = [],
       rew_ord = ("dummy_ord",dummy_ord), 
	 erls = e_rls, srls = Erls, calc = [],
	 rules = 
	 [Rls_ common_nominator_p
	  (*FIXME.WN0401 ? redesign Rrls - use exhaustively on a term ?
	    FIXME.WN0510 unnecessary nesting: introduce RRls_ : rls -> rule*)
	  ], 
	 scr = EmptyScr});
(* ------------------------------------------------------------------- *)
val cancel_p_rls = prep_rls(
  Rls {id = "cancel_p_rls", preconds = [],
       rew_ord = ("dummy_ord",dummy_ord), 
	 erls = e_rls, srls = Erls, calc = [],
	 rules = 
	 [Rls_ cancel_p
	  (*FIXME.WN.0401 ? redesign Rrls - use exhaustively on a term ?*)
	  ], 
	 scr = EmptyScr});
(* -------------------------------------------------------------------- *)
(*. makes 'normal' fractions; 'is_polyexp' inhibits double fractions;
    used in initial part norm_Rational_mg, see example DA-M02-main.p.60.*)
val rat_mult_poly = prep_rls(
  Rls {id = "rat_mult_poly", preconds = [],
       rew_ord = ("dummy_ord",dummy_ord), 
	 erls =  append_rls "e_rls-is_polyexp" e_rls
	         [Calc ("Poly.is'_polyexp", eval_is_polyexp "")], 
	 srls = Erls, calc = [],
	 rules = 
	 [Thm ("rat_mult_poly_l",num_str rat_mult_poly_l),
	  (*"?c is_polyexp ==> ?c * (?a / ?b) = ?c * ?a / ?b"*)
	  Thm ("rat_mult_poly_r",num_str rat_mult_poly_r)
	  (*"?c is_polyexp ==> ?a / ?b * ?c = ?a * ?c / ?b"*)
	  ], 
	 scr = EmptyScr});
(* ------------------------------------------------------------------ *)
(*. makes 'normal' fractions; 'is_polyexp' inhibits double fractions;
    used in looping part norm_Rational_rls, see example DA-M02-main.p.60 
    .. WHERE THE LATTER DOES ALWAYS WORK, BECAUSE erls = e_rls, 
    I.E. THE RESPECTIVE ASSUMPTION IS STORED AND Thm APPLIED; WN051028 
    ... WN0609???MG.*)
val rat_mult_div_pow = prep_rls(
  Rls {id = "rat_mult_div_pow", preconds = [], 
       rew_ord = ("dummy_ord",dummy_ord), 
       erls = e_rls,
       (*FIXME.WN051028 append_rls "e_rls-is_polyexp" e_rls
			[Calc ("Poly.is'_polyexp", eval_is_polyexp "")],
         with this correction ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ we get 
	 error "rational.sml.sml: diff.behav. in norm_Rational_mg 29" etc.
         thus we decided to go on with this flaw*)
		 srls = Erls, calc = [],
      rules = [Thm ("rat_mult",num_str rat_mult),
	       (*"?a / ?b * (?c / ?d) = ?a * ?c / (?b * ?d)"*)
	       Thm ("rat_mult_poly_l",num_str rat_mult_poly_l),
	       (*"?c is_polyexp ==> ?c * (?a / ?b) = ?c * ?a / ?b"*)
	       Thm ("rat_mult_poly_r",num_str rat_mult_poly_r),
	       (*"?c is_polyexp ==> ?a / ?b * ?c = ?a * ?c / ?b"*)

	       Thm ("real_divide_divide1_mg", real_divide_divide1_mg),
	       (*"y ~= 0 ==> (u / v) / (y / z) = (u * z) / (y * v)"*)
	       Thm ("real_divide_divide1_eq", real_divide_divide1_eq),
	       (*"?x / (?y / ?z) = ?x * ?z / ?y"*)
	       Thm ("real_divide_divide2_eq", real_divide_divide2_eq),
	       (*"?x / ?y / ?z = ?x / (?y * ?z)"*)
	       Calc ("HOL.divide"  ,eval_cancel "#divide_"),
	      
	       Thm ("rat_power", num_str rat_power)
		(*"(?a / ?b) ^^^ ?n = ?a ^^^ ?n / ?b ^^^ ?n"*)
	       ],
      scr = Script ((term_of o the o (parse thy)) "empty_script")
      }:rls);
(* ------------------------------------------------------------------ *)
val rat_reduce_1 = prep_rls(
  Rls {id = "rat_reduce_1", preconds = [], 
       rew_ord = ("dummy_ord",dummy_ord), 
       erls = e_rls, srls = Erls, calc = [], 
       rules = [Thm ("real_divide_1",num_str real_divide_1),
		(*"?x / 1 = ?x"*)
		Thm ("real_mult_1",num_str real_mult_1)           
		(*"1 * z = z"*)
		],
       scr = Script ((term_of o the o (parse thy)) "empty_script")
       }:rls);
(* ------------------------------------------------------------------ *)
(*. looping part of norm_Rational(*_mg*) .*)
val norm_Rational_rls = prep_rls(
   Rls {id = "norm_Rational_rls", preconds = [], 
       rew_ord = ("dummy_ord",dummy_ord), 
       erls = norm_rat_erls, srls = Erls, calc = [],
       rules = [Rls_ common_nominator_p_rls,
		Rls_ rat_mult_div_pow,
		Rls_ make_rat_poly_with_parentheses,
		Rls_ cancel_p_rls,(*FIXME:cancel_p does NOT order sometimes*)
		Rls_ rat_reduce_1
		],
       scr = Script ((term_of o the o (parse thy)) "empty_script")
       }:rls);
(* ------------------------------------------------------------------ *)
(*040109 'norm_Rational'(by RL) replaced by 'norm_Rational_mg'(MG)
 just be renaming:*)
val norm_Rational(*_mg*) = prep_rls(
   Seq {id = "norm_Rational"(*_mg*), preconds = [], 
       rew_ord = ("dummy_ord",dummy_ord), 
       erls = norm_rat_erls, srls = Erls, calc = [],
       rules = [Rls_ discard_minus_,
		Rls_ rat_mult_poly,(* removes double fractions like a/b/c    *)
		Rls_ make_rat_poly_with_parentheses, (*WN0510 also in(#)below*)
		Rls_ cancel_p_rls, (*FIXME.MG:cancel_p does NOT order sometim*)
		Rls_ norm_Rational_rls,   (* the main rls, looping (#)       *)
		Rls_ discard_parentheses_ (* mult only                       *)
		],
       scr = Script ((term_of o the o (parse thy)) "empty_script")
       }:rls);
(* ------------------------------------------------------------------ *)


ruleset' := overwritelthy thy (!ruleset',
  [("calculate_Rational", calculate_Rational),
   ("calc_rat_erls",calc_rat_erls),
   ("rational_erls", rational_erls),
   ("cancel_p", cancel_p),
   ("cancel", cancel),
   ("common_nominator_p", common_nominator_p),
   ("common_nominator_p_rls", common_nominator_p_rls),
   ("common_nominator"  , common_nominator),
   ("discard_minus", discard_minus),
   ("powers_erls", powers_erls),
   ("powers", powers),
   ("rat_mult_divide", rat_mult_divide),
   ("reduce_0_1_2", reduce_0_1_2),
   ("rat_reduce_1", rat_reduce_1),
   ("norm_rat_erls", norm_rat_erls),
   ("norm_Rational", norm_Rational),
   ("norm_Rational_rls", norm_Rational_rls),
   ("norm_Rational_parenthesized", norm_Rational_parenthesized),
   ("rat_mult_poly", rat_mult_poly),
   ("rat_mult_div_pow", rat_mult_div_pow),
   ("cancel_p_rls", cancel_p_rls)
   ]);

calclist':= overwritel (!calclist', 
   [("is_expanded", ("Rational.is'_expanded", eval_is_expanded ""))
    ]);

(** problems **)

store_pbt
 (prep_pbt Rational.thy "pbl_simp_rat" [] e_pblID
 (["rational","simplification"],
  [("#Given" ,["term t_"]),
   ("#Where" ,["t_ is_ratpolyexp"]),
   ("#Find"  ,["normalform n_"])
  ],
  append_rls "e_rls" e_rls [(*for preds in where_*)], 
  Some "Simplify t_", 
  [["simplification","of_rationals"]]));

(** methods **)

(*WN061025 this methods script is copied from (auto-generated) script
  of norm_Rational in order to ease repair on inform*)
store_met
    (prep_met Rational.thy "met_simp_rat" [] e_metID
	      (["simplification","of_rationals"],
	       [("#Given" ,["term t_"]),
		("#Where" ,["t_ is_ratpolyexp"]),
		("#Find"  ,["normalform n_"])
		],
	       {rew_ord'="tless_true",
		rls' = e_rls,
		calc = [], srls = e_rls, 
		prls = append_rls "simplification_of_rationals_prls" e_rls 
				[(*for preds in where_*)
				 Calc ("Rational.is'_ratpolyexp", 
				       eval_is_ratpolyexp "")],
		crls = e_rls, nrls = norm_Rational_rls},
"Script SimplifyScript (t_::real) =                              \
\  ((Try (Rewrite_Set discard_minus_ False) @@                   \
\    Try (Rewrite_Set rat_mult_poly False) @@                    \
\    Try (Rewrite_Set make_rat_poly_with_parentheses False) @@   \
\    Try (Rewrite_Set cancel_p_rls False) @@                     \
\    (Repeat                                                     \
\     ((Try (Rewrite_Set common_nominator_p_rls False) @@        \
\       Try (Rewrite_Set rat_mult_div_pow False) @@              \
\       Try (Rewrite_Set make_rat_poly_with_parentheses False) @@\
\       Try (Rewrite_Set cancel_p_rls False) @@                  \
\       Try (Rewrite_Set rat_reduce_1 False)))) @@               \
\    Try (Rewrite_Set discard_parentheses_ False))               \
\    t_)"
	       ));

(* use"../IsacKnowledge/Rational.ML";
   use"IsacKnowledge/Rational.ML";
   use"Rational.ML";
   *)

